@@ -1,6 +1,7 @@
-/* Ergo, version 3.4, a program for linear scaling electronic structure
+/* Ergo, version 3.5, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2014 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -317,11 +318,11 @@ find_range_index(int atomIndex, int noOfRanges, const basis_set_range_struct* ra
 }
 
 
-static const basisset_struct* 
+static const basisset_info* 
 select_basis_set(int atomIndex, 
 		 int noOfRanges,
 		 const basis_set_range_struct* rangeList,
-		 const basisset_struct* basissetDefault)
+		 const basisset_info* basissetDefault)
 {
   int rangeIndex = find_range_index(atomIndex, noOfRanges, rangeList);
   if(rangeIndex < 0)
@@ -350,14 +351,14 @@ select_basis_set(int atomIndex,
 static int 
 setup_shells_multi_basis_getcount(const Atom* atomList,
 				  int noOfAtoms,
-				  const basisset_struct* basissetDefault,
+				  const basisset_info* basissetDefault,
 				  int noOfRanges,
 				  const basis_set_range_struct* rangeList)
 {
   int noOfShells = 0;
   for(int i = 0; i < noOfAtoms; i++) {
     int z = (int)atomList[i].charge;
-    const basisset_struct* basissetCurrAtom = select_basis_set(i, noOfRanges, rangeList, basissetDefault);
+    const basisset_info* basissetCurrAtom = select_basis_set(i, noOfRanges, rangeList, basissetDefault);
     int noOfShellsCurrAtom = basissetCurrAtom->atoms[z].noOfShells;
     if(noOfShellsCurrAtom <= 0) {
       do_output(LOG_CAT_ERROR, LOG_AREA_INTEGRALS, 
@@ -375,7 +376,7 @@ static int
 setup_shells_multi_basis(const IntegralInfo& integralInfo,
 			 const Atom* atomList,
 			 int noOfAtoms,
-			 const basisset_struct* basissetDefault, 
+			 const basisset_info* basissetDefault, 
 			 ShellSpecStruct* shell_list,
 			 int noOfShells,
 			 int noOfRanges,
@@ -388,7 +389,7 @@ setup_shells_multi_basis(const IntegralInfo& integralInfo,
   SquareFuncIntegrator sfi;
   for(int i = 0; i < noOfAtoms; i++) {
     int z = (int)atomList[i].charge;
-    const basisset_struct* basissetCurrAtom = select_basis_set(i, noOfRanges, rangeList, basissetDefault);
+    const basisset_info* basissetCurrAtom = select_basis_set(i, noOfRanges, rangeList, basissetDefault);
     int noOfShellsCurrAtom = basissetCurrAtom->atoms[z].noOfShells;
     if(noOfShellsCurrAtom <= 0) {
       do_output(LOG_CAT_ERROR, LOG_AREA_INTEGRALS, "error in setup_shells_multi_basis: element %i is not supported by selected basis set?", z);
@@ -437,11 +438,54 @@ setup_shells_multi_basis(const IntegralInfo& integralInfo,
 
 
 
+void BasisInfoStruct::addBasisfuncsForPoint(ergo_real x,
+					    ergo_real y,
+					    ergo_real z,
+					    int shellType,
+					    ergo_real exponent,
+					    const IntegralInfo & integralInfo,
+					    int print_raw,
+					    int do_normalization,
+					    int skip_sort_shells) {
+  // Create "dummy atom" at given pos.
+  Atom dummyAtom;
+  dummyAtom.charge = 1;
+  dummyAtom.coords[0] = x;
+  dummyAtom.coords[1] = y;
+  dummyAtom.coords[2] = z;
+  // Create "dummy basis set" defining shell types for atoms with charge 1.
+  basisset_info basisset;
+  basisset.atoms.resize(2);
+  for(int kk = 0; kk < 2; kk++) {
+    // Use one shell of each type, up to given shellType.
+    int nShells = shellType + 1;
+    if(nShells >= MAX_NO_OF_SHELLS_PER_ATOM)
+      throw std::runtime_error("Error in BasisInfoStruct::addBasisfuncsForPoint: (nShells >= MAX_NO_OF_SHELLS_PER_ATOM).");
+    basisset.atoms[kk].noOfShells = nShells;
+    for(int i = 0; i <= shellType; i++) {
+      basisset.atoms[kk].shells[i].type = i;
+      basisset.atoms[kk].shells[i].contrCount = 1;
+      basisset.atoms[kk].shells[i].shell_ID = i;
+      basisset.atoms[kk].shells[i].exponentList[0] = exponent;
+      basisset.atoms[kk].shells[i].coeffList[0] = 1;
+    }
+  }
+  // OK, now the "dummy basis set" is done.
+  addBasisfuncsForAtomList(&dummyAtom, // atomList
+			   1, // int noOfAtoms,
+			   basisset,
+			   0, // noOfRanges,
+			   NULL, // rangeList,
+			   integralInfo,
+			   print_raw,
+			   do_normalization,
+			   skip_sort_shells);
+}
 
 
 void BasisInfoStruct::addBasisfuncsForAtomList(const Atom* atomList,
 					       int noOfAtoms,
-					       const basisset_struct* basissetDefault,
+					       const basisset_info & basissetDefault,
 					       int noOfRanges,
 					       const basis_set_range_struct* rangeList,
 					       const IntegralInfo& integralInfo,
@@ -451,7 +495,7 @@ void BasisInfoStruct::addBasisfuncsForAtomList(const Atom* atomList,
 
   int noOfShellsToAdd = setup_shells_multi_basis_getcount(atomList,
 							  noOfAtoms,
-                                                          basissetDefault,
+                                                          &basissetDefault,
                                                           noOfRanges,
                                                           rangeList);
   if(noOfShellsToAdd <= 0)
@@ -469,13 +513,13 @@ void BasisInfoStruct::addBasisfuncsForAtomList(const Atom* atomList,
   if(setup_shells_multi_basis(integralInfo,
 			      atomList,
 			      noOfAtoms,
-                              basissetDefault,
+                              &basissetDefault,
                               &shell_list_new[noOfShells],
                               noOfShellsToAdd,
                               noOfRanges,
                               rangeList,
                               use_6_d_funcs) != 0)
-    throw std::runtime_error("error in setup_shells_multi_basis_getcount");
+    throw std::runtime_error("error in setup_shells_multi_basis");
 
   for(int i = 0; i < noOfShellsNew; i++) {
     for(int kk = 0; kk < shell_list_new[i].noOfContr; kk++) {
@@ -493,7 +537,7 @@ void BasisInfoStruct::addBasisfuncsForAtomList(const Atom* atomList,
   }
 
   if(this->shellList)
-    delete this->shellList;
+    delete [] this->shellList;
   this->shellList = shell_list_new;
   this->noOfShells = noOfShellsNew;
   
@@ -560,8 +604,7 @@ int BasisInfoStruct::addBasisfuncsForMolecule(const Molecule& molecule,
     ERGO_SPREFIX "/basis",
     ERGO_SPREFIX
   };
-  basisset_struct* basissetDefault = new basisset_struct;
-  memset(basissetDefault, 0, sizeof(basisset_struct));
+  basisset_info* basissetDefault = new basisset_info();
   
   basis_set_range_struct rangeListTemp[noOfRanges];
   memset(rangeListTemp, 0, noOfRanges*sizeof(basis_set_range_struct));
@@ -573,7 +616,7 @@ int BasisInfoStruct::addBasisfuncsForMolecule(const Molecule& molecule,
       return -1;
     }
 
-  if(read_basisset_file(basissetDefault, basisset_filename_default, 6, dirv,
+  if(read_basisset_file(*basissetDefault, basisset_filename_default, 6, dirv,
                         print_raw) != 0)
     {
       do_output(LOG_CAT_ERROR, LOG_AREA_INTEGRALS, "error in read_basisset_file for standard basis set.");
@@ -587,8 +630,8 @@ int BasisInfoStruct::addBasisfuncsForMolecule(const Molecule& molecule,
     if(rangeList[rangeIndex].count <= 0)
       rangeListTemp[rangeIndex].basisset = NULL;
     else {
-      rangeListTemp[rangeIndex].basisset = new basisset_struct;
-      if(read_basisset_file(rangeListTemp[rangeIndex].basisset, rangeList[rangeIndex].basisSetFileName, 6, dirv,
+      rangeListTemp[rangeIndex].basisset = new basisset_info();
+      if(read_basisset_file(*rangeListTemp[rangeIndex].basisset, rangeList[rangeIndex].basisSetFileName, 6, dirv,
 			    print_raw) != 0) {
 	do_output(LOG_CAT_ERROR, LOG_AREA_INTEGRALS, "error in read_basisset_file for rangeIndex = %i", rangeIndex);
 	delete basissetDefault;
@@ -599,7 +642,7 @@ int BasisInfoStruct::addBasisfuncsForMolecule(const Molecule& molecule,
 
   addBasisfuncsForAtomList(molecule.getAtomListPtr(),
 			   molecule.getNoOfAtoms(),
-			   basissetDefault,
+			   *basissetDefault,
 			   noOfRanges,
 			   rangeListTemp,
 			   integralInfo,
