@@ -1,6 +1,7 @@
-/* Ergo, version 3.4, a program for linear scaling electronic structure
+/* Ergo, version 3.5, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2014 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -29,6 +30,7 @@
 #ifndef BASISSET_HEADER
 #define BASISSET_HEADER
 
+#include <vector>
 #include "realtype.h"
 #include "polydegree.h"
 
@@ -60,12 +62,22 @@ typedef struct
   basisset_shell_struct shells[MAX_NO_OF_SHELLS_PER_ATOM];
 } basisset_atom_struct;
 
-typedef struct
-{
-  basisset_atom_struct atoms[MAX_NO_OF_ATOM_TYPES];
-} basisset_struct;
+/* ELIAS NOTE 2014-08-02: FIXME: The basisset_struct struct is too
+   big, causing problems when used as a
+   chttl::Basic<basisset_struct>. The easiest fix may be to make the
+   list dynamically allocated, and implement a simple chunk type for
+   it instead of using chttl::Basic.  */
+struct basisset_info {
+  std::vector<basisset_atom_struct> atoms;
+  basisset_info();
+  void clear();
+  // Stuff needed for Chunks&Tasks usage
+  void write_to_buffer ( char * dataBuffer, size_t const bufferSize ) const;
+  size_t get_size() const;
+  void assign_from_buffer ( char const * dataBuffer, size_t const bufferSize);
+};
 
-int read_basisset_file(basisset_struct* result, 
+int read_basisset_file(basisset_info & result, 
 		       const char* fileName,
 		       int dirc, 
 		       const char *dirv[],
