@@ -1,6 +1,7 @@
-/* Ergo, version 3.4, a program for linear scaling electronic structure
+/* Ergo, version 3.5, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2014 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -44,6 +45,8 @@ int get_dens_from_fock_full(int n,
 			    ergo_real factor,
 			    ergo_real electronicTemperature,
 			    ergo_real & resultEntropyTerm,
+			    ergo_real & resultHomoLumoGap,
+			    int store_all_eigenvalues_to_file, 
 			    ergo_real * const lumoVec = 0,
 			    ergo_real * const homoVec = 0);
 
