@@ -1,6 +1,7 @@
-/* Ergo, version 3.4, a program for linear scaling electronic structure
+/* Ergo, version 3.5, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2014 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -33,22 +34,12 @@
     Pawel Salek, 2004.06, Himmelbjerg - initial implementation.
 */
 
- 
-/* strictly conform to XOPEN ANSI C standard */
-#if !defined(SYS_DEC)
-/* XOPEN compliance is missing on old Tru64 4.0E Alphas and pow() prototype
- * is not specified. */
-#define _XOPEN_SOURCE          500
-#define _XOPEN_SOURCE_EXTENDED 1
-#endif
-/* Use BSD's strncasecmp() */
-#define _BSD_SOURCE 1
-
 #include <assert.h>
 #include <math.h>
 
 #include <ctype.h>
 #include <string.h>
+#include <strings.h> /* Needed for strncasecmp() */
 #include <stdio.h>
 #include <stdlib.h>
  
