@@ -1,6 +1,7 @@
-/* Ergo, version 3.4, a program for linear scaling electronic structure
+/* Ergo, version 3.5, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2014 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -32,17 +33,11 @@
    (c) Pawel Salek, pawsa@theochem.kth.se, sep 2001
 */
 
-/* strictly conform to XOPEN ANSI C standard */
-#define _XOPEN_SOURCE          500
-#define _XOPEN_SOURCE_EXTENDED 1
-
-/* Use BSD's strncasecmp() */
-#define _BSD_SOURCE 1
-
 #include <ctype.h>
 //#include <math.h>
 #include <stdio.h>
 #include <string.h>
+#include <strings.h> /* Needed for strncasecmp() */
 #include <stdlib.h>
 
 #define __CVERSION__
