@@ -1,6 +1,7 @@
-/* Ergo, version 3.4, a program for linear scaling electronic structure
+/* Ergo, version 3.5, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2014 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -35,8 +36,6 @@
    (c) P. Salek, oct 2003 - the working implementation.
 */
 
-#define _XOPEN_SOURCE 600
-
 #include <math.h>
 #include <stddef.h>
 
@@ -83,12 +82,6 @@ lb94_read(const char* conf_line)
     return 1;
 }
 
-/* lb94_energy:
-   lb94 threshold is needed to avoid numerical problems on 0/0
-   divisions.  The problems are small but it is better to be on the
-   safe side.
-*/
-static const real LB94_THRESHOLD = 1e-14;
 static const real BETA = 0.05;
 
 static real
