@@ -1,6 +1,7 @@
-/* Ergo, version 3.4, a program for linear scaling electronic structure
+/* Ergo, version 3.5, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2014 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -99,7 +100,7 @@ test_first(const char* fun_name, EnergyFunc func, FirstOrderFun first_func)
 		real rho   = i/40.0;
 		real ngrad = j/40.0;
 		real gracos= k/20.0;
-		FunDensProp dt, dp = { 0.5*rho,0.2*rho, 0.2*ngrad,0.2*ngrad };
+		FunDensProp dt, dp = { (real)0.5*rho,(real)0.2*rho, (real)0.2*ngrad,(real)0.2*ngrad };
 		dp.gradab = dp.grada*dp.gradb*gracos;
 		/* TEST df1000 */
 		drho = rho*1e-4;
@@ -158,7 +159,7 @@ test_second(const char* fname,
 		real rho   = i/40.0;
 		real ngrad = j/40.0;
 		real gracos= k/20.0;
-		FunDensProp dt, dp = {0.5*rho, 0.2*rho, 0.5*ngrad, 0.3*ngrad};
+		FunDensProp dt, dp = {(real)0.5*rho, (real)0.2*rho, (real)0.5*ngrad, (real)0.3*ngrad};
 		dp.gradab = dp.grada*dp.gradb*gracos;
 		drv2_clear(&d);
 		second_fun(&d, 1, &dp);
@@ -223,7 +224,7 @@ test_third(const char* fname,
 		real rho   = i/40.0;
 		real ngrad = j/40.0;
 		real gracos= k/20.0;
-		FunDensProp dt, dp = {0.5*rho, 0.3*rho, 0.5*ngrad, 1*ngrad};
+		FunDensProp dt, dp = {(real)0.5*rho, (real)0.3*rho, (real)0.5*ngrad, (real)1*ngrad};
 		dp.gradab = dp.grada*dp.gradb*gracos;
 		drv3_clear(&d);
 		third_fun(&d, 1, &dp);
@@ -314,7 +315,7 @@ test_fourth(const char* fname,
 		real rho   = i/40.0;
 		real ngrad = j/40.0;
 		real gracos= k/20.0;
-		FunDensProp dt, dp = {0.5*rho, 0.3*rho, 0.5*ngrad, 1*ngrad};
+		FunDensProp dt, dp = {(real)0.5*rho, (real)0.3*rho, (real)0.5*ngrad, (real)1*ngrad};
 		dp.gradab = dp.grada*dp.gradb*gracos;
 		drv4_clear(&d);	fourth_fun(&d, 1, &dp);
 		drv3_clear(&m); third_fun(&m, 1, &dp);
