@@ -1,6 +1,7 @@
-/* Ergo, version 3.4, a program for linear scaling electronic structure
+/* Ergo, version 3.5, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2014 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -31,13 +32,6 @@
    (c), Pawel Salek, pawsa@theochem.kth.se, sep 2001, nov 2002
 */
 
-/* strictly conform to XOPEN ANSI C standard */
-#if !defined(SYS_DEC)
-/* XOPEN compliance is missing on old Tru64 4.0E Alphas */
-#define _XOPEN_SOURCE          600
-#define _XOPEN_SOURCE_EXTENDED 1
-#endif
-
 #include <math.h>
 #include <stddef.h>
 #include <stdlib.h>
