@@ -1,6 +1,7 @@
-/* Ergo, version 3.4, a program for linear scaling electronic structure
+/* Ergo, version 3.5, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2014 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -31,8 +32,6 @@
 
  */
 
-#define _BSD_SOURCE 1
-
 #include <stdarg.h>
 #include <stddef.h>
 #include <stdio.h>
@@ -114,9 +113,9 @@ enum FunError
 fun_select_by_name(const char *conf_string)
 {
     int ok, i;
-    char func_name[20];
+    char func_name[40];
 
-    sscanf(conf_string,"%20s", func_name);
+    sscanf(conf_string,"%39s", func_name);
     for(i=0; available_functionals[i]; i++)
         if(strcasecmp(available_functionals[i]->name, func_name)==0) {
             selected_func = available_functionals[i];
