@@ -1,6 +1,7 @@
-/* Ergo, version 3.4, a program for linear scaling electronic structure
+/* Ergo, version 3.5, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2014 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -609,7 +610,14 @@ Stream::saveAtomGridInBox(unsigned iAtom, const Box& box,
                           FILE *stream) const
 {
   /* MAX_BATCH_LENGTH Must be larger than the densest angular grid. */
-  static const unsigned MAX_BATCH_LENGTH = 8192;
+  /* Elias note 2016-07-08: The MAX_BATCH_LENGTH value here was
+     decreased from 8192 to 4096 because the 8192 was apparently too
+     large when running on a Mac, it seemed like the threads ran out
+     of stack space then. */
+  /* FIXME: there does not seem to be proper checking if the
+     MAX_BATCH_LENGTH value is too small, it just gives wrong results
+     without detecting the error. */
+  static const unsigned MAX_BATCH_LENGTH = 4096;
   const AtomicGrid& atom = atoms[relevantAtoms[iAtom]];
   real dist = box.getDistanceTo(atom.center.v);
   const RadialGrid& rGrid = atom.getRadialGrid();
