@@ -1,6 +1,7 @@
-/* Ergo, version 3.4, a program for linear scaling electronic structure
+/* Ergo, version 3.5, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2014 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -32,9 +33,6 @@
 
    This module evaluates DFT contribution KS matrix.
 */
-/* strictly conform to XOPEN ANSI C standard */
-#define _XOPEN_SOURCE          500
-#define _XOPEN_SOURCE_EXTENDED 1
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -65,7 +63,7 @@ static pthread_mutex_t dft_prop_mutex = PTHREAD_MUTEX_INITIALIZER;
 #include "utilities.h"
 #include "matrix_utilities.h"
 #include "grid_matrix.h"
-#include "xc_evaluators.hpp"
+#include "xc_evaluators.h"
 
 /* restrict hints should not be necessary... */
 #if !defined(restrict)
