@@ -1,6 +1,7 @@
-/* Ergo, version 3.4, a program for linear scaling electronic structure
+/* Ergo, version 3.5, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2014 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,6 +26,20 @@
  * For further information about Ergo, see <http://www.ergoscf.org>.
  */
 
+/** @file tdhf_dynamics.cc
+
+ Implementation of TDHF electron dynamics, explicitly propagating the
+ electron density in time in the precence of a time-dependent electric
+ field.
+
+ This implementation was done to reproduce the results in the article
+ by Li et al: Phys. Chem. Chem. Phys., 2005, 7, 233–239.
+
+ DOI link for that article: http://dx.doi.org/10.1039/B415849K
+
+ @author: Elias Rudberg <em>responsible</em>.
+*/
+
 #include <cstring>
 #include "tdhf_dynamics.h"
 #include "scf_utils.h"
@@ -64,7 +79,7 @@ static void get_Fock_matrix_in_ort_basis(const BasisInfoStruct & basisInfo,
 {
   Util::TimeMeter timeMeter;
 
-  do_output(LOG_CAT_INFO, LOG_AREA_SCF, "get_Fock_matrix_in_ort_basis() starting!");
+  do_output(LOG_CAT_INFO, LOG_AREA_ED, "get_Fock_matrix_in_ort_basis() starting!");
 
   normalMatrix Z;
   Z.resetSizesAndBlocks(size_block_info, size_block_info);
@@ -136,7 +151,7 @@ static void get_Fock_matrix_in_ort_basis(const BasisInfoStruct & basisInfo,
   refF.resetSizesAndBlocks(size_block_info, size_block_info);
   refF = refFockMatrix;
   ergo_real frobDiff = normalMatrix::frob_diff(F_nonort, refF);
-  do_output(LOG_CAT_INFO, LOG_AREA_SCF, "get_Fock_matrix_in_ort_basis() frobDiff = %9.4g.", (double)frobDiff);
+  do_output(LOG_CAT_INFO, LOG_AREA_ED, "get_Fock_matrix_in_ort_basis() frobDiff = %9.4g.", (double)frobDiff);
 
   // Now transform final result back to orthogonal basis.
   normalMatrix ZTF;
@@ -144,9 +159,9 @@ static void get_Fock_matrix_in_ort_basis(const BasisInfoStruct & basisInfo,
   ZTF = ZT * F_nonort;
   F_ort = ZTF * Z;
 
-  timeMeter.print(LOG_AREA_SCF, "get_Fock_matrix_in_ort_basis");
+  timeMeter.print(LOG_AREA_ED, "get_Fock_matrix_in_ort_basis");
 
-  do_output(LOG_CAT_INFO, LOG_AREA_SCF, "get_Fock_matrix_in_ort_basis() finishing OK!");
+  do_output(LOG_CAT_INFO, LOG_AREA_ED, "get_Fock_matrix_in_ort_basis() finishing OK!");
 }
 
 
@@ -292,7 +307,7 @@ static void compute_exp_of_matrix(ComplexMatrix & U,
     X_to_pow_k = tmp;
     U.add(X_to_pow_k, preFactor);
   }
-  timeMeter.print(LOG_AREA_SCF, "compute_exp_of_matrix");
+  timeMeter.print(LOG_AREA_ED, "compute_exp_of_matrix");
 }
 
 
@@ -348,7 +363,7 @@ void do_tdhf_dynamics(const BasisInfoStruct & basisInfo,
 		      const triangMatrix & invCholFactor,
 		      const ED::Params & params)
 {
-  do_output(LOG_CAT_INFO, LOG_AREA_SCF, "do_tdhf_dynamics() starting!");
+  do_output(LOG_CAT_INFO, LOG_AREA_ED, "do_tdhf_dynamics() starting!");
 
   // First get density matrix to orthogonal basis.
 
@@ -369,7 +384,7 @@ void do_tdhf_dynamics(const BasisInfoStruct & basisInfo,
     
   D_ort *= (ergo_real)0.5;
   
-  do_output(LOG_CAT_INFO, LOG_AREA_SCF, "SCF_restricted::do_electron_dynamics(), D_ort done.");
+  do_output(LOG_CAT_INFO, LOG_AREA_ED, "SCF_restricted::do_electron_dynamics(), D_ort done.");
 
   // Test D_ort by converting back to non-orthogonal basis and comparing to D.
   symmMatrix D_nonort_test;
@@ -378,7 +393,7 @@ void do_tdhf_dynamics(const BasisInfoStruct & basisInfo,
   D_nonort_test = invCholFactor * D_nonort_test * transpose(invCholFactor);
   ergo_real D_frob_diff = symmMatrix::frob_diff(D_nonort_test, densityMatrix);
 
-  do_output(LOG_CAT_INFO, LOG_AREA_SCF, 
+  do_output(LOG_CAT_INFO, LOG_AREA_ED, 
 	    "SCF_restricted::do_electron_dynamics(), D_ort test D_frob_diff = %8.3g.", (double)D_frob_diff);
 
   // Get matrices needed for computation of instantaneous dipole.
@@ -456,10 +471,10 @@ void do_tdhf_dynamics(const BasisInfoStruct & basisInfo,
 				    matOpts.size_block_info,
 				    matOpts.permutationHML) != 0)
       {
-	do_output(LOG_CAT_ERROR, LOG_AREA_SCF, "error in compute_h_core_matrix_sparse");
+	do_output(LOG_CAT_ERROR, LOG_AREA_ED, "error in compute_h_core_matrix_sparse");
 	throw "error in compute_h_core_matrix_sparse";
       }
-    timeMeterHCore.print(LOG_AREA_SCF, "TDHF dynamics step H_core matrix computation");
+    timeMeterHCore.print(LOG_AREA_ED, "TDHF dynamics step H_core matrix computation");
 
     // Get Fock matrix. To do this, first transform density matrix re
     // and im parts to non-orthogonal basis.
@@ -482,7 +497,7 @@ void do_tdhf_dynamics(const BasisInfoStruct & basisInfo,
     ComplexMatrix D_ort_new;
     D_ort_new.initialize(matOpts.size_block_info);
     do_exp_transform_BCH(D_ort_new, X, D_ort_prev, matOpts.size_block_info);
-    do_output(LOG_CAT_INFO, LOG_AREA_SCF, 
+    do_output(LOG_CAT_INFO, LOG_AREA_ED, 
 	      "SCF_restricted::do_electron_dynamics(), do_exp_transform_BCH done!");
 #else
     const int n = basisInfo.noOfBasisFuncs;
@@ -543,12 +558,12 @@ void do_tdhf_dynamics(const BasisInfoStruct & basisInfo,
 	instantaneousDipole_im = 
 	  vectorLength(instantaneousDipole_x, instantaneousDipole_y, instantaneousDipole_z);
       }
-      do_output(LOG_CAT_INFO, LOG_AREA_SCF, "Data for dipole plot: %15.0f %15.10f %15.10f (time was really %15.10f)", 
+      do_output(LOG_CAT_INFO, LOG_AREA_ED, "Data for dipole plot: %15.0f %15.10f %15.10f (time was really %15.10f)", 
 		(double)t, (double)instantaneousDipole_re, (double)instantaneousDipole_im, (double)t);
     }
 
-    timeMeterCurrStep.print(LOG_AREA_SCF, "TDHF dynamics complete step");
+    timeMeterCurrStep.print(LOG_AREA_ED, "TDHF dynamics complete step");
   }
 
-  do_output(LOG_CAT_INFO, LOG_AREA_SCF, "do_tdhf_dynamics() finished!");
+  do_output(LOG_CAT_INFO, LOG_AREA_ED, "do_tdhf_dynamics() finished!");
 }
