@@ -1,6 +1,7 @@
-/* Ergo, version 3.4, a program for linear scaling electronic structure
+/* Ergo, version 3.5, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2014 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -46,11 +47,16 @@
 #include <sys/types.h>
 #include <sys/times.h>
 #include <errno.h>
+#include <sstream>
 
 #ifdef _OPENMP
 #include <omp.h>
 #endif
 
+#ifdef USE_CHUNKS_AND_TASKS
+#include "chunks_and_tasks.h"
+#endif
+
 #include "atom_labels.h"
 #include "density_description_file.h"
 #include "ergo_scripted.h"
@@ -649,7 +655,7 @@ int
 es_run(const char *mode, int save_pot)
 {
   do_output(LOG_CAT_INFO, LOG_AREA_MAIN, "running %s\n", mode);
-
+  
   if(!ergoIntegralInfo)
     ergoIntegralInfo = new IntegralInfo(true);
   
@@ -744,19 +750,25 @@ es_run(const char *mode, int save_pot)
   scf.starting_guess_disturbance  = var_get_realSCF("starting_guess_disturbance");
   scf.purification_subspace_err_limit = var_get_realSCF("purification_subspace_err_limit");
   scf.purification_eigvalue_err_limit = var_get_realSCF("purification_eigvalue_err_limit");
+  scf.purification_with_acceleration = var_get_intSCF("purification_with_acceleration");
+  scf.use_new_stopping_criterion = var_get_intSCF("use_new_stopping_criterion");
   scf.puri_eig_acc_factor_for_guess = var_get_realSCF("puri_eig_acc_factor_for_guess");
   scf.gap_expected_lower_bound = var_get_realSCF("gap_expected_lower_bound");
   scf.shift_using_prev_density_matrix = var_get_realSCF("shift_using_prev_density_matrix");
   scf.electronic_temperature = var_get_realSCF("electronic_temperature");
-  scf.purification_truncation_norm = mat::getNormType( var_get_stringSCF("purification_truncation_norm") );  
-  scf.break_on_energy_increase    = var_get_intSCF("break_on_energy_increase");
+  scf.purification_truncation_norm = mat::getNormType( var_get_stringSCF("purification_truncation_norm") ); 
+  scf.purification_stop_crit_norm  = mat::getNormType( var_get_stringSCF("purification_stop_crit_norm") ); 
+  scf.break_on_energy_increase     = var_get_intSCF("break_on_energy_increase");
   scf.create_basis_func_coord_file = var_get_intSCF("create_basis_func_coord_file");
   scf.output_homo_and_lumo_eigenvectors = var_get_intSCF("output_homo_and_lumo_eigenvectors");
+  scf.eigensolver_accuracy        = var_get_realSCF("eigensolver_accuracy");
+  scf.eigensolver_maxiter         = var_get_intSCF("eigensolver_maxiter");
   scf.create_mtx_file_S           = var_get_intSCF("create_mtx_file_S");
   scf.create_mtx_file_H_core      = var_get_intSCF("create_mtx_file_H_core");
   scf.create_mtx_files_F          = var_get_intSCF("create_mtx_files_F");
   scf.create_mtx_files_D          = var_get_intSCF("create_mtx_files_D");
   scf.create_mtx_files_dipole     = var_get_intSCF("create_mtx_files_dipole");
+  scf.create_mtx_files_S_and_quit = var_get_intSCF("create_mtx_files_S_and_quit");
   scf.create_2el_integral_m_file  = var_get_intSCF("create_2el_integral_m_file");
   scf.force_restricted            = var_get_intSCF("force_restricted");
   scf.force_unrestricted          = var_get_intSCF("force_unrestricted");
@@ -774,6 +786,7 @@ es_run(const char *mode, int save_pot)
   scf.use_diagonalization         = var_get_intSCF("use_diagonalization");
   scf.use_diag_on_error           = var_get_intSCF("use_diag_on_error");
   scf.use_diag_on_error_guess     = var_get_intSCF("use_diag_on_error_guess");
+  scf.store_all_eigenvalues_to_file = var_get_intSCF("store_all_eigenvalues_to_file");
   scf.output_mulliken_pop         = var_get_intSCF("output_mulliken_pop");
   scf.output_density_images       = var_get_intSCF("output_density_images");
   scf.output_density_images_only  = var_get_intSCF("output_density_images_only");
@@ -800,6 +813,10 @@ es_run(const char *mode, int save_pot)
   scf.no_of_core_electrons        = var_get_intSCF("no_of_core_electrons");
   scf.skip_H_core                 = var_get_intSCF("skip_H_core");
   scf.purification_create_m_files = var_get_intSCF("purification_create_m_files");
+  scf.create_checkpoints  = var_get_intSCF("create_checkpoints");
+  scf.checkpoint_IDstr        = var_get_stringSCF("checkpoint_IDstr");
+  scf.eigenvectors_method         = var_get_stringSCF("eigenvectors_method");
+  scf.eigenvectors_iterative_method  = var_get_stringSCF("eigenvectors_iterative_method");
   scf.purification_use_rand_perturbation_for_alleigsint = var_get_intSCF("purification_use_rand_perturbation_for_alleigsint");
   scf.use_stochastic_orbs               = var_get_intSCF("use_stochastic_orbs");
   scf.stochastic_orbs_no_of_vectors     = var_get_intSCF("stochastic_orbs_no_of_vectors");
@@ -1008,6 +1025,7 @@ es_run(const char *mode, int save_pot)
 	if(do_CI(*Basis_info,
 		 *ergoIntegralInfo,
 		 ci_options,
+		 ergo.molecule,
 		 &S[0],
 		 &H_1[0],
 		 &F_a[0],
@@ -1088,15 +1106,16 @@ es_run(const char *mode, int save_pot)
     do_output_time(LOG_CAT_ERROR, LOG_AREA_MAIN, "Time of exception: ");
   }  
   catch (const std::exception& e) {
-    do_output(LOG_CAT_ERROR, LOG_AREA_MAIN, "Exception caught: %s\n", e.what());
-    fprintf(stderr, "Exception caught: %s\n", e.what());
+    do_output(LOG_CAT_ERROR, LOG_AREA_MAIN, "Exception (std::exception) caught in es_run: '%s'\n", e.what());
+    fprintf(stderr, "Exception (std::exception) caught in es_run: '%s'\n", e.what());
   } catch (const char* s) {
-    do_output(LOG_CAT_ERROR, LOG_AREA_MAIN, "Exception caught: %s\n", s);
-    fprintf(stderr, "Exception caught: %s\n", s);
+    do_output(LOG_CAT_ERROR, LOG_AREA_MAIN, "Exception (char*) caught in es_run: '%s'\n", s);
+    fprintf(stderr, "Exception (char*) caught in es_run: '%s'\n", s);
   }
   grid_free_files();
   if(var_get_intMA("write_to_file"))
     es_rmdir_with_content(subdir.c_str());
+   
   return 0;
 }
 
@@ -1490,13 +1509,13 @@ Ergo::registerInputVariables()
 #undef KL
 
 #define KL ergo.XC_params
-  KW(KL,type,                         VAR_STRING, "Turbo","Type of the radial quadrature, one of Turbo and LMG.");
+  KW(KL,type,                         VAR_STRING, "Turbo","Type of the radial quadrature, one of Turbo, LMG and HICU.");
   KW(KL,sparse_mode,                  VAR_INT,    1,"Enable sparse mode for DFT exchange-correlation matrix computation.");
   KW(KL,radint,                       VAR_FLOAT,  5e-9,"Accuracy of the radial integration for atomic grids");
   KW(KL,force_cubic_boxes,            VAR_INT,    0,"Forces cubic boxes in grid generation as opposed to rectangular cuboid");
   KW(KL,box_size,                     VAR_FLOAT,  5.0,"Upper limit on grid box size");
   KW(KL,angmin,                       VAR_INT,    6,"Minimal order of pruned angular atomic grids");
-  KW(KL,angint,                       VAR_INT,    29,"Default order of angular atomc grids");
+  KW(KL,angint,                       VAR_INT,    29,"Default order of angular atomic grids");
   KW(KL,hicu_max_error,               VAR_FLOAT,  1e-7,"Threshold value (max error per box) for Hierarchical Cubature (HiCu) DFT grid generation.");
   KW(KL,hicu_box_size,                VAR_FLOAT,  1.5,"Box size for Hierarchical Cubature (HiCu) DFT grid generation.");
   KW(KL,hicu_start_box_size_debug,    VAR_FLOAT,  0.0,"Debug box size param for Hierarchical Cubature (HiCu) DFT grid generation.");
@@ -1533,7 +1552,7 @@ Ergo::registerInputVariables()
 
 #define KL ergo.ed_params
 #define KWED(n,type,d) KW(ergo.ed_params,n,type,edOptions.n,d)
-  KW(ergo.ed_params,electron_dynamics_field_type,   VAR_STRING, "none", "Field type to use in electron dynamics simulation: \"none\", \"dc-pulse\", \"ac-pulse\".");
+  KW(ergo.ed_params,field_type,   VAR_STRING, "none", "Field type to use in electron dynamics simulation: \"none\", \"dc-pulse\", \"ac-pulse\".");
   KWED(max_time, VAR_FLOAT, "How long time to run electron dynamics. Unit: a.u.");
   KWED(timestep, VAR_FLOAT, "Timestep to use in electron dynamics. Unit: a.u.");
   KWED(dc_pulse_strength, VAR_FLOAT, "Strength of DC pulse to use in electron dynamics. Unit: a.u.");
@@ -1570,13 +1589,18 @@ Ergo::registerInputVariables()
   KWSCF(scan_step_factor,             VAR_FLOAT,"Step factor to use for \"accuracy scans\" of J, K, Vxc matrices.");
   KWSCF(purification_create_m_files,  VAR_INT,"Create m files with information about purification."),
   KWSCF(purification_use_rand_perturbation_for_alleigsint, VAR_INT,"Use random perturbation to attempt getting easier Lanczos convergence when determining min/max eigenvalues of Fock matrix before purification."),
+  KW(ergo.scf_params, checkpoint_IDstr, VAR_STRING, "", "ID for file with saved state (parameters in the GetDensFromFock class) before recursive expansion on every SCF cycle.");
+  KWSCF(create_checkpoints,           VAR_INT,"Save state (parameters in the GetDensFromFock class) before recursive expansion on every SCF cycle.");
+  KW(ergo.scf_params, eigenvectors_method, VAR_STRING, "square", "Method for computation of HOMO and LUMO molecular orbital coefficients. Value: square or projection");
+  KW(ergo.scf_params, eigenvectors_iterative_method, VAR_STRING, "lanczos", "Iterative method for computation of HOMO and LUMO molecular orbital coefficients. Value: power or lanczos");
 #endif
   KWSCF(use_simple_dense_H_core,      VAR_INT,"Use simple dense matrix computation of 1-electron (core) Hamiltonian matrix.");
   KWSCF(use_diis_always,              VAR_INT,"Always use DIIS even if the energy goes up.");
   KWSCF(write_overlap_matrix,         VAR_INT,"Write overlap matrix to file.");
   KWSCF(use_diagonalization,          VAR_INT,"Use diagonalization instead of purification.");
   KWSCF(use_diag_on_error,            VAR_INT,"Use diagonalization if purification fails.");
-  KWSCF(use_diag_on_error_guess,      VAR_INT,"Use diagonalization if purification fails during starting guess projection.");
+  KWSCF(use_diag_on_error_guess,      VAR_INT,"Use diagonalization if purification fails during starting guess projection.");    
+  KWSCF(store_all_eigenvalues_to_file, VAR_INT,"Store eigenvalues of the Hamiltonian matrix when using diagonalization (use_diagonalization flag must be 1).");
   KWSCF(output_mulliken_pop,          VAR_INT,"Output Mulliken population analysis stuff (atomic charges and atomic spin densities) after SCF finished.");
   KWSCF(output_density_images,        VAR_INT,"Output density image files (including Gabedit gcube density files) after SCF finished.");
   KWSCF(output_density_images_only,   VAR_INT,"Output density image files (including Gabedit gcube density files) directly, without any SCF procedure, using the given starting guess density.");
@@ -1601,11 +1625,14 @@ Ergo::registerInputVariables()
   KWSCF(error_maxabs_for_diis,        VAR_FLOAT,"Do not use DIIS if error measure is larger than this value.");
   KWSCF(purification_subspace_err_limit,        VAR_FLOAT,"Requested accuracy in the occupied invariant subspace of the density matrix resulting from purification as measured by the sinus of the largest canonical angle between the exact and approximate subspaces.");
   KWSCF(purification_eigvalue_err_limit,        VAR_FLOAT,"Requested accuracy in eigenvalues of the density matrix resulting from purification.");
+  KWSCF(purification_with_acceleration, VAR_INT,"Use acceleration when doing purification.");
+  KWSCF(use_new_stopping_criterion,    VAR_INT,"Use new parameterless stopping criterion.");
   KWSCF(puri_eig_acc_factor_for_guess, VAR_FLOAT,"When doing purification of starting guess density, use the normal eigvalue_err_limit multiplied by this factor.");
   KWSCF(gap_expected_lower_bound,        VAR_FLOAT,"Expected lower bound of HOMO-LUMO gap, used in early SCF iterations when accurate gap info is not yet available.");
   KWSCF(shift_using_prev_density_matrix, VAR_FLOAT, "As input to density matrix construction, Fock/KS matrix modified by subtracting previous density matrix times this factor. This is a way to artificially increase the HOMO-LUMO gap. A.k.a. level shifting.");
   KWSCF(electronic_temperature,          VAR_FLOAT,"Electronic temperature for Fermi-Dirac smearing in density matrix construction. Unit: a.u.");
-  KW(ergo.scf_params,purification_truncation_norm,   VAR_STRING, "mixed","Matrix norm to be used for truncation of small matrix elements in purification, one of frob, mixed, and eucl.");
+  KW(ergo.scf_params,purification_truncation_norm,  VAR_STRING, "mixed","Matrix norm to be used for truncation of small matrix elements in purification, one of frob, mixed, and eucl.");
+  KW(ergo.scf_params,purification_stop_crit_norm,   VAR_STRING, "mixed","Matrix norm to be used for the estimation of the convergence order in purification, one of frob, mixed, and eucl.");
   KW(ergo.scf_params,electric_field_z,VAR_FLOAT, scfOptions.electric_field[2],"External electric field, z component.");
   KW(ergo.scf_params,electric_field_y,VAR_FLOAT, scfOptions.electric_field[1],"External electric field, y component.");
   KW(ergo.scf_params,electric_field_x,VAR_FLOAT, scfOptions.electric_field[0],"External electric field, x component.");
@@ -1615,11 +1642,14 @@ Ergo::registerInputVariables()
   KWSCF(break_on_energy_increase,     VAR_INT,"Break SCF procedure if energy increases.");
   KWSCF(create_basis_func_coord_file, VAR_INT,"Write text file with coordinates of basis functions.");
   KWSCF(output_homo_and_lumo_eigenvectors, VAR_INT,"Output HOMO and LUMO molecular orbital coefficients.");
+  KWSCF(eigensolver_accuracy,         VAR_FLOAT, "The accuracy for the eigenvalue problem solver.");
+  KWSCF(eigensolver_maxiter,          VAR_INT, "The maximum number of iterations for the eigenvalue problem solver.");
   KWSCF(create_mtx_file_S,            VAR_INT,"Write overlap matrix to file in matrix market format (mtx).");
   KWSCF(create_mtx_file_H_core,       VAR_INT,"Write core Hamiltonian matrix to file in matrix market format (mtx).");
   KWSCF(create_mtx_files_F,           VAR_INT,"Write effective Hamiltonian matrices to file in matrix market format (mtx).");
   KWSCF(create_mtx_files_D,           VAR_INT,"Write density matrices to file in matrix market format (mtx).");
   KWSCF(create_mtx_files_dipole,      VAR_INT,"Write dipole matrices to file in matrix market format (mtx).");
+  KWSCF(create_mtx_files_S_and_quit,  VAR_INT,"Write overlap matrix to file in matrix market format (mtx) using different basis function orderings, and then quit.");
 
   KW(ergo.scf_params,calculation_identifier,   VAR_STRING, "N/A","String to identify the calculation, used for example in mtx-files.");
 
@@ -1707,25 +1737,74 @@ es_warranty(void)
   puts(ERGO_LICENSE_TEXT_LONG "\n\n");
 }
 
+struct filename_or_string_struct {
+  std::string filename;
+  std::string str;
+};
+
 int
 main(int argc, char *argv[])
 {
+ 
+  int numThreads_CHT = 1, numWorkers_CHT = 1;
+  size_t cache_size = 0;
+
   try {
     enable_output();
-  do_output(LOG_CAT_INFO, LOG_AREA_MAIN, ERGO_LICENSE_TEXT_LONG);
-  bool executed_something = false;
-  FILE *inp_file = NULL;
-  ergo.registerInputVariables();
-  es_set_nthreads_string("detect");
-  dft_init();
-  do_output(LOG_CAT_INFO, LOG_AREA_MAIN, "ERGO version %s\n", VERSION);
+    do_output(LOG_CAT_INFO, LOG_AREA_MAIN, ERGO_LICENSE_TEXT_LONG);
+    FILE *inp_file = NULL;
+    ergo.registerInputVariables();
+    es_set_nthreads_string("detect");
+    dft_init();
+    do_output(LOG_CAT_INFO, LOG_AREA_MAIN, "ERGO version %s\n", VERSION);
 
-  for(int i = 1; i < argc; i++) {
-    if(strcmp(argv[i], "-e") == 0) {
+    // Below we go through the input arguments and create a list of filename_or_string_struct that we will process later.
+    std::list<filename_or_string_struct> filename_or_string_list;
+
+    for(int i = 1; i < argc; i++) {
+      if(strcmp(argv[i], "-w") == 0) {
+	if(i+1<argc) {
+	  do_output(LOG_CAT_INFO, LOG_AREA_MAIN, "parsing string: %s\n", argv[i+1]);
+	  numWorkers_CHT = atoi(argv[i+1]);
+	  do_output(LOG_CAT_INFO, LOG_AREA_MAIN, "(ChT) number of workers: %s\n", argv[i+1]);
+	  printf("(ChT) number of workers: %s\n", argv[i+1]);
+	} else {
+	  fprintf(stderr, "option -w encountered without argument\n");
+	  return 1;
+	}
+	i++;
+      }
+      else if(strcmp(argv[i], "-t") == 0) {
+	if(i+1<argc) {
+	  do_output(LOG_CAT_INFO, LOG_AREA_MAIN, "parsing string: %s\n", argv[i+1]);
+	  numThreads_CHT = atoi(argv[i+1]);
+	  do_output(LOG_CAT_INFO, LOG_AREA_MAIN, "(ChT) number of threads: %s\n", argv[i+1]);
+	  printf("(ChT) number of threads: %s\n", argv[i+1]);
+	} else {
+	  fprintf(stderr, "option -t encountered without argument\n");
+	  return 1;
+	}
+      i++;
+    }
+      else if(strcmp(argv[i], "-cs") == 0) {
+	if(i+1<argc) {
+	  do_output(LOG_CAT_INFO, LOG_AREA_MAIN, "parsing string: %s\n", argv[i+1]);
+	  std::istringstream iss(argv[i+1]);
+	  iss >> cache_size;
+	  do_output(LOG_CAT_INFO, LOG_AREA_MAIN, "(ChT) cache size: %s\n", argv[i+1]);
+	  printf("(ChT) cache size: %s\n", argv[i+1]);
+	} else {
+	  fprintf(stderr, "option -cs encountered without argument\n");
+	  return 1;
+	}
+      i++;
+    }
+    else if(strcmp(argv[i], "-e") == 0) {
       if(i+1<argc) {
-        do_output(LOG_CAT_INFO, LOG_AREA_MAIN, "parsing string: %s\n", argv[i+1]);
-        ergo_parse_string(argv[i+1]);
-        executed_something = true;
+	// Input line found; add it to list to process later.
+	filename_or_string_struct tmp;
+	tmp.str = argv[i+1];
+	filename_or_string_list.push_back(tmp);
       } else {
         fprintf(stderr, "option -e encountered without argument\n");
         return 1;
@@ -1773,19 +1852,40 @@ main(int argc, char *argv[])
         return 1;
       }
       i++;
-    } else if(strcmp(argv[i], "-h") == 0) {
+    }
+#ifdef USE_CHUNKS_AND_TASKS
+    else if(strcmp(argv[i], "-h") == 0) {
       printf("Usage: ergo [args...]\n"
              "args can be: input file name\n"
-             "             -e \"input line\"\n"
-             "             -d VARIABLE_NAME variable to describe\n"
-             "             -m molecule file name\n"
-             "             -c extra-charges molecule file name\n"
-             "             -h this message\n"
+	     "             -w  number of workers (default 1)\n"
+	     "             -t  number of threads (default 1)\n"
+	     "             -cs cache size in MB  (default 0)\n"
+             "             -e  \"input line\"\n"
+             "             -d  VARIABLE_NAME variable to describe\n"
+             "             -m  molecule file name\n"
+             "             -c  extra-charges molecule file name\n"
+             "             -h  this message\n"
              "Arguments are interpreted in the order of encounter.\n"
              "Predefined variables:\n");
       es_print_help();
       return 0;
-    } else if(strcmp(argv[i], "-d") == 0) {
+    } 
+#else
+    else if(strcmp(argv[i], "-h") == 0) {
+      printf("Usage: ergo [args...]\n"
+             "args can be: input file name\n"
+             "             -e  \"input line\"\n"
+             "             -d  VARIABLE_NAME variable to describe\n"
+             "             -m  molecule file name\n"
+             "             -c  extra-charges molecule file name\n"
+             "             -h  this message\n"
+             "Arguments are interpreted in the order of encounter.\n"
+             "Predefined variables:\n");
+      es_print_help();
+      return 0;
+    } 
+#endif
+    else if(strcmp(argv[i], "-d") == 0) {
       if(i+1<argc) {
         const struct variable *v = es_find_var(NULL, argv[i+1]);
         if (v) {
@@ -1802,35 +1902,82 @@ main(int argc, char *argv[])
     } else if(strcmp(argv[i], "-b") == 0) {
       benchmark_mm();
     } else {
-      inp_file = fopen(argv[i], "rt");
-      if (!inp_file) {
-        fprintf(stderr, "Could not open '%s' for reading input.\n", argv[i]);
-        return 1;
-      }
-      do_output(LOG_CAT_INFO, LOG_AREA_MAIN, "reading input from %s\n", argv[i]);
-      ergo_parse_file(inp_file);
-      fclose(inp_file);
-      executed_something = true;
+      // Input file name found; add it to list to process later.
+      filename_or_string_struct tmp;
+      tmp.filename = argv[i];
+      filename_or_string_list.push_back(tmp);
     }
-  }
-  if(!executed_something) {
-     if(isatty(fileno(stdin))) {
-       printf(ERGO_LICENSE_TEXT_BRIEF "\n\n");
-       printf("ERGO is ready!\n> ");
     }
-    ergo_parse_file(stdin);
-  }
-  if(ergoIntegralInfo)
-    delete ergoIntegralInfo;
-  if(Basis_info)
-    delete Basis_info;
 
-  }
+    // OK, now we have gone through all input arguments and any input
+    // lines and/or input filenames have been placed in the list
+    // filename_or_string_list.
+
+#ifdef USE_CHUNKS_AND_TASKS
+    cht::extras::setNWorkers(numWorkers_CHT);
+    cht::extras::setNoOfWorkerThreads(numThreads_CHT);
+    cht::setOutputLevel(cht::Output::Info);
+    cht::setOutputMode(cht::Output::AllInTheEnd);
+    cht::extras::Cache::Mode cache_mode = cht::extras::Cache::Enabled;
+    size_t cacheMemoryUsageLimit = cache_size; // MB
+    cht::extras::setCacheMode(cache_mode);
+    cht::extras::setCacheSize(cacheMemoryUsageLimit);
+    cht::start();
+    do_output(LOG_CAT_INFO, LOG_AREA_MAIN, "Use Chunks and Tasks parallel library");
+    do_output(LOG_CAT_INFO, LOG_AREA_MAIN, "Set number of workers to %d and number of threads to %d", numWorkers_CHT, numThreads_CHT);
+#else
+    do_output(LOG_CAT_INFO, LOG_AREA_MAIN, "Not using CHT, so values numWorkers_CHT=%d and numThreads_CHT=%d will be ignored.", numWorkers_CHT, numThreads_CHT);
+#endif
+
+    // Now process any input lines and/or input filenames.
+    bool executed_something = false;
+    std::list<filename_or_string_struct>::iterator it = filename_or_string_list.begin();
+    while(it != filename_or_string_list.end()) {
+      if(it->str.length() != 0) {
+	// Process input line
+	do_output(LOG_CAT_INFO, LOG_AREA_MAIN, "parsing string: %s\n", it->str.c_str());
+	ergo_parse_string(it->str.c_str());
+	executed_something = true;
+      }
+      else {
+	assert(it->filename.length() != 0);
+	// Process input file
+	inp_file = fopen(it->filename.c_str(), "rt");
+	if (!inp_file) {
+	  fprintf(stderr, "Could not open '%s' for reading input.\n", it->filename.c_str());
+	  return 1;
+	}
+	do_output(LOG_CAT_INFO, LOG_AREA_MAIN, "reading input from file %s\n", it->filename.c_str());
+	ergo_parse_file(inp_file);
+	fclose(inp_file);
+	executed_something = true;
+      }
+      it++;
+    }
+
+    if(!executed_something) {
+      if(isatty(fileno(stdin))) {
+	printf(ERGO_LICENSE_TEXT_BRIEF "\n\n");
+	printf("ERGO is ready!\n> ");
+      }
+      ergo_parse_file(stdin);
+    }
+
+#ifdef USE_CHUNKS_AND_TASKS
+    cht::stop();
+#endif
+
+      if(ergoIntegralInfo)
+	delete ergoIntegralInfo;
+      if(Basis_info)
+	delete Basis_info;
+
+    }
   catch (mat::AcceptableMaxIter & e) {
-    do_output(LOG_CAT_ERROR, LOG_AREA_MAIN, "");
-    do_output(LOG_CAT_ERROR, LOG_AREA_MAIN, "=============================================================");
-    do_output(LOG_CAT_ERROR, LOG_AREA_MAIN, "mat::AcceptableMaxIter caught in ergo main: '%s'", e.what());
-    do_output_time(LOG_CAT_ERROR, LOG_AREA_MAIN, "Time of exception: ");
+      do_output(LOG_CAT_ERROR, LOG_AREA_MAIN, "");
+      do_output(LOG_CAT_ERROR, LOG_AREA_MAIN, "=============================================================");
+      do_output(LOG_CAT_ERROR, LOG_AREA_MAIN, "mat::AcceptableMaxIter caught in ergo main: '%s'", e.what());
+      do_output_time(LOG_CAT_ERROR, LOG_AREA_MAIN, "Time of exception: ");
     do_output(LOG_CAT_ERROR, LOG_AREA_MAIN, "=============================================================");
   }
   catch (mat::Failure & e) {
@@ -1869,7 +2016,7 @@ main(int argc, char *argv[])
     do_output_time(LOG_CAT_ERROR, LOG_AREA_MAIN, "Time of exception: ");
     do_output(LOG_CAT_ERROR, LOG_AREA_MAIN, "=============================================================");
   }
-
+  
   return 0;
 }
 /* ===================================================================
