@@ -1,6 +1,7 @@
-/* Ergo, version 3.4, a program for linear scaling electronic structure
+/* Ergo, version 3.5, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2014 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -68,6 +69,7 @@ class hermite_conversion_info_struct {
   hermite_conversion_element_struct_ptr list_left_simple [HERMITE_CONVERSION_MAX_N+1];
   int counters_right_simple[HERMITE_CONVERSION_MAX_N+1];
   int counters_left_simple [HERMITE_CONVERSION_MAX_N+1];
+  void clear_lists();
  public:
   void init(const monomial_info_struct & monomial_info);
   hermite_conversion_info_struct();
