@@ -1,6 +1,7 @@
-/* Ergo, version 3.4, a program for linear scaling electronic structure
+/* Ergo, version 3.5, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2014 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -1288,7 +1289,7 @@ getDmatLimitMatrixCSRList(csr_matrix_struct* dmatLimitMatrixCSRList,
 			  const int* const* basisFuncGroupList,
 			  const int* basisFuncGroupCounterList)
 {
-  do_output(LOG_CAT_INFO, LOG_AREA_SCF, "getDmatLimitMatrixCSRList start.");
+  do_output(LOG_CAT_INFO, LOG_AREA_INTEGRALS, "getDmatLimitMatrixCSRList start.");
   memset(dmatLimitMatrixCSRList, 0, numberOfLevels*sizeof(csr_matrix_struct));
   for(int levelNumber = 0; levelNumber < numberOfLevels; levelNumber++) {
     // Populate dmatElementList with info for this level, one row at a time.
@@ -1305,7 +1306,7 @@ getDmatLimitMatrixCSRList(csr_matrix_struct* dmatLimitMatrixCSRList,
 				       &colind[0], 
 				       &values[0],
 				       nValuesCurrRow) != 0) {
-	do_output(LOG_CAT_ERROR, LOG_AREA_SCF, "error in ergo_CSR_get_values_singlerow.");
+	do_output(LOG_CAT_ERROR, LOG_AREA_INTEGRALS, "error in ergo_CSR_get_values_singlerow.");
 	return -1;
       }
       std::vector<dmatElementStruct> dmatElementList(nValuesCurrRow);
@@ -1342,7 +1343,7 @@ getDmatLimitMatrixCSRList(csr_matrix_struct* dmatLimitMatrixCSRList,
       for(int i = 0; i < (int)dmatElementListList[row].size(); i++)
 	dmatElementList[count++] = dmatElementListList[row][i];
     if(count != nTot) {
-      do_output(LOG_CAT_ERROR, LOG_AREA_SCF, "error in getDmatLimitMatrixCSRList: (count != nTot).");
+      do_output(LOG_CAT_ERROR, LOG_AREA_INTEGRALS, "error in getDmatLimitMatrixCSRList: (count != nTot).");
       return -1;
     }
     // OK, single long list created.
@@ -1369,7 +1370,7 @@ getDmatLimitMatrixCSRList(csr_matrix_struct* dmatLimitMatrixCSRList,
 		       &rowind2[0],
 		       &colind2[0]) != 0)
       {
-	do_output(LOG_CAT_ERROR, LOG_AREA_SCF, "error in ergo_CSR_create for dmatLimitMatrixCSRList.");
+	do_output(LOG_CAT_ERROR, LOG_AREA_INTEGRALS, "error in ergo_CSR_create for dmatLimitMatrixCSRList.");
 	return -1;
       }
     for(int i = 0; i < nReduced; i++) {
@@ -2052,7 +2053,7 @@ compute_K_by_boxes(const BasisInfoStruct & basisInfo,
 			     &rowind[0],
 			     &colind[0]) != 0)
 	    {
-	      do_output(LOG_CAT_ERROR, LOG_AREA_SCF, "error in ergo_CSR_create for dmatLimitMatrixCSRList.");
+	      do_output(LOG_CAT_ERROR, LOG_AREA_INTEGRALS, "error in ergo_CSR_create for dmatLimitMatrixCSRList.");
 	      return -1;
 	    }
 	  for(int i = 0; i < nn; i++)
@@ -2071,7 +2072,7 @@ compute_K_by_boxes(const BasisInfoStruct & basisInfo,
 				   basisFuncGroupList, 
 				   basisFuncGroupCounterList) != 0)
 	{
-	  do_output(LOG_CAT_ERROR, LOG_AREA_SCF, "error in getDmatLimitMatrixCSRList.");
+	  do_output(LOG_CAT_ERROR, LOG_AREA_INTEGRALS, "error in getDmatLimitMatrixCSRList.");
 	  return -1;
 	}
     }
