@@ -1,6 +1,7 @@
-/* Ergo, version 3.4, a program for linear scaling electronic structure
+/* Ergo, version 3.5, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2014 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -270,8 +271,13 @@ typedef int* intPtr;
 struct listsStruct {
   int** longList;
   int* longListCounterList;
-  listsStruct() : longList(0), longListCounterList(0) { }
+  int n_saved;
+  listsStruct() : longList(0), longListCounterList(0), n_saved(0) { }
   ~listsStruct() { 
+    for(int i = 0; i < n_saved; i++) {
+      if(longList[i])
+	delete [] longList[i];
+    }
     delete [] longList;
     delete [] longListCounterList;
   }
@@ -280,6 +286,7 @@ struct listsStruct {
     longListCounterList = new int[n];
     memset(longList, 0, n*sizeof(int*));
     memset(longListCounterList, 0, n*sizeof(int));
+    n_saved= n;
   }
 };
 
@@ -448,6 +455,11 @@ identify_needed_elements(ergo_real threshold,
 	delete [] newLongList;
       }
     }
+
+  // Cleanup
+  for(int i = 0; i < nThreads; i++)
+    delete threadParamsList[i];
+
   return 0;
 }
 
