@@ -1,6 +1,7 @@
-/* Ergo, version 3.4, a program for linear scaling electronic structure
+/* Ergo, version 3.5, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2014 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -1431,7 +1432,7 @@ int mainFun(int argc,char* argv[]) {
      */
       
 
-    /* *********  Gersgorin test: symmetric matrix     */ {
+    /* *********  Gershgorin test: symmetric matrix     */ {
       symmMatrix syA;
       syA.resetSizesAndBlocks(rows,cols);
       syA.randomZeroStructure(0.2);
@@ -1462,10 +1463,10 @@ int mainFun(int argc,char* argv[]) {
 	tmp2 = diag[col] + tmp1;
 	refMax = (tmp2 > refMax ? tmp2 : refMax);
       }
-      std::cout<<"\nGersgorin eigenvalue bounds test: \n";
+      std::cout<<"\nGershgorin eigenvalue bounds test: \n";
       real gersMin;
       real gersMax;
-      syA.gersgorin(gersMin, gersMax);
+      syA.gershgorin(gersMin, gersMax);
       std::cout
 	<<"Reference: "
 	<<std::setprecision(10)<<std::setw(15)
@@ -1480,7 +1481,7 @@ int mainFun(int argc,char* argv[]) {
 	std::cout<<"   ERROR" <<std::endl;
 	std::exit(1);
       } 
-    } /* End of Gersgorin test symmetric matrix */
+    } /* End of Gershgorin test symmetric matrix */
     
     /* *********  Memory usage     */ {
       symmMatrix syA;
