@@ -1,6 +1,7 @@
-/* Ergo, version 3.4, a program for linear scaling electronic structure
+/* Ergo, version 3.5, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2014 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -32,6 +33,8 @@
 #include <string>
 #include <sstream>
 #include <iomanip> /* For setprecision */
+#include <iostream>
+#include <cstdlib>
 #include "Allocator.h"
 
 namespace mat {
@@ -71,8 +74,31 @@ class AllocatorManager
       it++;
     }
     // We did not find any non-full existing allocator. Need to add a new one.
-    Allocator<Treal>* newAllocator = new Allocator<Treal>(noOfRealsPerBuffer, 
-							 noOfBuffers);
+    /* ELIAS NOTE 2016-06-30: Important to catch exceptions here so
+       that we unlock the mutex in case allocation fails, otherwise
+       program may hang later when trying to lock the mutex e.g. to
+       delete something after catching the exception elsewhere in the
+       code. This happened with some test runs on Triolith and it was
+       very annoying, took a long time to figure out why the runs
+       hanged. */
+    Allocator<Treal>* newAllocator;
+    try {
+      newAllocator = new Allocator<Treal>(noOfRealsPerBuffer,
+					  noOfBuffers);
+    }
+    catch (const std::bad_alloc & e) {
+      size_t noOfBytesPerAllocator = noOfBuffers * noOfRealsPerBuffer * sizeof(Treal);
+      size_t totNoOfBytesAllocated = list.size() * noOfBytesPerAllocator;
+      std::cerr << "Error in AllocatorManager::alloc(): std::bad_alloc exception caught. Usage before error: list.size() = " << list.size()
+		<< " --> " << (double)totNoOfBytesAllocated/1000000000 << " GB used. peakListSize = " << peakListSize << std::endl;
+      pthread_mutex_unlock(&mutex);
+      throw e;
+    }
+    catch(...) {
+      std::cerr << "Error in AllocatorManager::alloc(): exception caught (but not std::bad_alloc!?!)." << std::endl;
+      pthread_mutex_unlock(&mutex);
+      throw std::runtime_error("Error in AllocatorManager::alloc(): exception caught (but not std::bad_alloc!?!).");
+    }
     list.push_back(newAllocator);
     if(list.size() > peakListSize)
       peakListSize = list.size();
@@ -127,10 +153,11 @@ class AllocatorManager
  AllocatorManager() : noOfRealsPerBuffer(0), noOfBuffers(0), peakListSize(0) {
     pthread_mutex_init(&mutex, NULL);
   }
-  ~AllocatorManager() { 
-    if(!list.empty())
-      throw std::runtime_error("Error in AllocatorManager destructor: "
-			       "not empty.");
+  ~AllocatorManager() {
+    if(!list.empty()) {
+      std::cerr << "Error in AllocatorManager destructor: not empty." << std::endl;
+      abort();
+    }
     // Go through list to free any allocators that are left.
     typename std::list< Allocator<Treal>* >::iterator it = list.begin();
     while(it != list.end()) {
