@@ -1,6 +1,7 @@
-/* Ergo, version 3.4, a program for linear scaling electronic structure
+/* Ergo, version 3.5, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2014 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -47,7 +48,7 @@
 #include "FileWritable.h"
 #include "Failure.h"
 #include "matInclude.h"
-
+#include <errno.h>
 
 namespace mat {
   unsigned int FileWritable::nObjects = 0;
@@ -93,6 +94,9 @@ namespace mat {
 	/* FIXME: We should not be doing memory allocation in error handling. */
 	std::string errstr("FileWritable::writeToFile(): write to file failed : " );
 	errstr += e.what();
+	errstr += "\n";
+	errstr += "Error (strerror(errno)): ";
+	errstr += strerror(errno);
 	throw strdup(errstr.c_str());
       }
       // Free memory used by object.
@@ -125,6 +129,9 @@ namespace mat {
 	std::stringstream ss;
 	ss << "Exception std::ifstream::failure caught in FileWritable::readFromFile() when trying to open file. fileName = '" << fileName << "'. what(): '" << e.what() << "'.";
 	std::string errstr = ss.str();
+	errstr += "\n";
+        errstr += "Error (strerror(errno)): ";
+        errstr += strerror(errno);
 	std::cerr << errstr << std::endl;
 	throw std::runtime_error(errstr);
       }
@@ -138,6 +145,9 @@ namespace mat {
 	/* FIXME: We should not be doing memory allocation in error handling. */
 	std::string errstr("FileWritable::readFromFile(): read from file failed : " );
 	errstr += e.what();
+	errstr += "\n";
+        errstr += "Error (strerror(errno)): ";
+        errstr += strerror(errno);
 	throw strdup(errstr.c_str());
       }
       // delete file
@@ -148,6 +158,130 @@ namespace mat {
     ++Stats::instance().countRead[obj_type_id()];
   }
 
+
+  static long int get_file_size(const char* fileName);
+
+ void FileWritable::copyFromFile(const char* sourceFileName)
+ {
+   clear();
+
+   if (FileWritable::active) 
+     {
+       // Open source file for reading.
+       std::ifstream inFile;
+       inFile.exceptions( std::ifstream::eofbit  | 
+			  std::ifstream::failbit | 
+			  std::ifstream::badbit );
+       inFile.open(sourceFileName, std::ios::in|std::ios::binary);
+       if (!inFile.is_open())
+	 throw std::runtime_error("FileWritable copyFromFile error: unable to open file for reading.");
+       // Open destination file for writing.
+       std::ofstream outFile;
+       outFile.exceptions( std::ifstream::eofbit  | 
+			   std::ifstream::failbit | 
+			   std::ifstream::badbit );
+       outFile.open(fileName, std::ios::out|std::ios::binary);
+       if (!outFile.is_open())
+	 throw std::runtime_error("FileWritable copyFromFile error: unable to open file for writing.");
+       // Copy data.
+       try {
+	 outFile << inFile.rdbuf();
+       }
+       catch (std::ofstream::failure& e) {
+	 throw std::runtime_error("FileWritable copyFromFile error: failed to copy data. Out of disk space?");
+       }
+       inFile.close();
+       outFile.close();
+       /* Verify that file was really copied. */
+       long int sz1 = get_file_size(fileName);
+       long int sz2 = get_file_size(sourceFileName);
+       if(sz1 < 0 || sz2 < 0 || sz1 != sz2)
+	 throw std::runtime_error("FileWritable copyFromFile error: file sizes do not match after copying.");
+
+     }
+   else
+     {
+       std::ifstream file;
+       // open file
+       file.exceptions( std::ifstream::eofbit  | 
+			std::ifstream::failbit | 
+			std::ifstream::badbit );
+       file.open(sourceFileName, std::ios::in|std::ios::binary);
+
+       if (!file.is_open())
+	 throw "FileWritable copyFromFile error: unable to open file.";
+
+       this->readFromFileProt(file);
+       file.close();
+
+       writeToFile(); // since active is false, it will just change a flag objectIsOnFile to true
+     }
+
+
+ }
+
+  void FileWritable::copyToFile(const char* destFileName)
+  {
+    if (!objectIsOnFile)
+      throw Failure("FileWritable::copyToFile(): Object is not on file.");
+
+    if (FileWritable::active) // if ergo flag is set, it means matrix is in the temporary file, 
+                              // thus just copy a file
+      {
+	// Open source file for reading.
+	std::ifstream inFile;
+	inFile.exceptions( std::ifstream::eofbit  | 
+			   std::ifstream::failbit | 
+			   std::ifstream::badbit );
+	inFile.open(fileName, std::ios::in|std::ios::binary);
+	if (!inFile.is_open())
+	  throw std::runtime_error("FileWritable copyToFile: unable to open file for reading.");
+	// Open destination file for writing.
+	std::ofstream outFile;
+	outFile.exceptions( std::ifstream::eofbit  | 
+			    std::ifstream::failbit | 
+			    std::ifstream::badbit );
+	outFile.open(destFileName, std::ios::out|std::ios::binary);
+	if (!outFile.is_open())
+	  throw std::runtime_error("FileWritable copyToFile error: unable to open file for writing.");
+	// Copy data.
+	try {
+	  outFile << inFile.rdbuf();
+	}
+	catch (std::ofstream::failure& e) {
+	  throw std::runtime_error("FileWritable copyToFile error: failed to copy data. Out of disk space?");
+	}
+	inFile.close();
+	outFile.close();
+	/* Verify that file was really copied. */
+	long int sz1 = get_file_size(fileName);
+	long int sz2 = get_file_size(destFileName);
+	if(sz1 < 0 || sz2 < 0 || sz1 != sz2)
+	  throw std::runtime_error("FileWritable copyToFile error: file sizes do not match after copying.");
+      }
+    else // flag is not set, create a new file
+      {
+	readFromFile(); // since active is false, it will just change a flag objectIsOnFile to false
+	std::ofstream file(destFileName, std::ios::out|std::ios::binary);
+	file.exceptions( std::ofstream::eofbit  | 
+			 std::ofstream::failbit | 
+			 std::ofstream::badbit );
+	if (!file.is_open())
+	  throw "FileWritable copyToFile: unable to open file. ";
+	try {
+	  this->writeToFileProt(file);
+	  file.close();
+	}
+	catch (std::ofstream::failure& e) {
+	  throw std::runtime_error("FileWritable copyToFile: failed to copy data.");
+	}
+	writeToFile(); // since active is false, it will just change a flag objectIsOnFile to true
+      }
+  }
+
+
+
+
   long int FileWritable::fileSize() {
     if ( !isOnFile() )
       throw std::runtime_error("Attempt to get file size for object "
