@@ -1,6 +1,7 @@
-/* Ergo, version 3.4, a program for linear scaling electronic structure
+/* Ergo, version 3.5, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2014 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -82,6 +83,10 @@ namespace mat {
      */
     void readFromFile();
 
+    void copyToFile(const char* destFileName);
+
+    void copyFromFile(const char* sourceFileName);
+
     /** Check if object is on file.
      */
     bool isOnFile() { return objectIsOnFile; }
