@@ -1,6 +1,7 @@
-/* Ergo, version 3.4, a program for linear scaling electronic structure
+/* Ergo, version 3.5, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2014 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -299,8 +300,8 @@ namespace mat {
     }
     Treal eucl_thresh(Treal const threshold);
     
-    inline void gersgorin(Treal& lmin, Treal& lmax) {
-      this->matrixPtr->gersgorin(lmin, lmax);
+    inline void gershgorin(Treal& lmin, Treal& lmax) {
+      this->matrixPtr->gershgorin(lmin, lmax);
     }    
     static inline Treal trace_ab
       (const MatrixGeneral<Treal, Tmatrix>& A,
