@@ -1,6 +1,7 @@
-/* Ergo, version 3.4, a program for linear scaling electronic structure
+/* Ergo, version 3.5, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2014 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -125,6 +126,12 @@ namespace mat {
       (std::vector<int> & rowind, 
        std::vector<int> & colind, 
        std::vector<Treal> & values) const {
+      rowind.resize(0);
+      colind.resize(0);
+      values.resize(0);
+      rowind.reserve(nnz());
+      colind.reserve(nnz());
+      values.reserve(nnz());
       this->matrixPtr->syGetAllValues(rowind, colind, values);
     }
     /**< Get all values and corresponding row and column index lists,
