@@ -1,6 +1,7 @@
-/* Ergo, version 3.4, a program for linear scaling electronic structure
+/* Ergo, version 3.5, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2014 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -330,10 +331,11 @@ namespace mat{
 			  "Telement>&): "
 			  "Incorrect dimensions for matrix-vector product");
 	  else {
+	    int A_nrows = A.nrows();
 #ifdef _OPENMP
 #pragma omp parallel for if(A.level() == Params::getMatrixParallelLevel()) num_threads(Params::getNProcs()) default(shared) schedule(dynamic) 
 #endif
-	    for (int row = 0; row < A.nrows(); row++) {
+	    for (int row = 0; row < A_nrows; row++) {
 	      MAT_OMP_START;
 	      Telement::gemv(tA, alpha, A(row, 0), x(0), beta_tmp, y(row));
 	      for (int col = 1; col < A.ncols(); col++) 
@@ -353,10 +355,11 @@ namespace mat{
 			  "Telement>&): "
 			  "Incorrect dimensions for matrix-vector product");
 	  else {
+	    int A_ncols = A.ncols();
 #ifdef _OPENMP
 #pragma omp parallel for if(A.level() == Params::getMatrixParallelLevel()) num_threads(Params::getNProcs()) default(shared) schedule(dynamic) 
 #endif
-	    for (int col = 0; col < A.ncols(); col++) {
+	    for (int col = 0; col < A_ncols; col++) {
 	      MAT_OMP_START;
 	      Telement::gemv(tA, alpha, A(0, col), x(0), beta_tmp, y(col));
 	      for (int row = 1; row < A.nrows(); row++)
@@ -415,19 +418,21 @@ namespace mat{
 #endif
 	{
 	  /* Diagonal */
+	  int A_ncols = A.ncols();
 #ifdef _OPENMP
 #pragma omp for  schedule(dynamic)
 #endif
-	  for (int rc = 0; rc < A.ncols(); rc++) {
+	  for (int rc = 0; rc < A_ncols; rc++) {
 	    MAT_OMP_START;
 	    Telement::symv(uplo, alpha, A(rc,rc), x(rc), beta_tmp, y(rc));
 	    MAT_OMP_END;
 	  }
 	  /* Upper triangle */
+	  int A_nrows = A.nrows();
 #ifdef _OPENMP
 #pragma omp for  schedule(dynamic)
 #endif
-	  for (int row = 0; row < A.nrows() - 1; row++) {
+	  for (int row = 0; row < A_nrows - 1; row++) {
 	    MAT_OMP_START;
 	    for (int col = row + 1; col < A.ncols(); col++)
 	      Telement::gemv(false, alpha, A(row, col), x(col), 1.0, y(row));
@@ -437,7 +442,7 @@ namespace mat{
 #ifdef _OPENMP
 #pragma omp for  schedule(dynamic)
 #endif
-	  for (int row = 1; row < A.nrows(); row++) {
+	  for (int row = 1; row < A_nrows; row++) {
 	    MAT_OMP_START;
 	    for (int col = 0; col < row; col++)
 	      Telement::gemv(true, alpha, A(col, row), x(col), 1.0, y(row));
