@@ -1,6 +1,7 @@
-/* Ergo, version 3.4, a program for linear scaling electronic structure
+/* Ergo, version 3.5, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2014 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -55,7 +56,13 @@ namespace mat {
       return !vectorPtr.haveDataStructureGet();
     }
 
+    inline void clear_structure(){
+      vectorPtr.haveDataStructureSet(false);
+    }
 
+    VectorGeneral(SizesAndBlocks const & newRows):vectorPtr(new Tvector) {
+      resetSizesAndBlocks(newRows);      
+    }
     VectorGeneral():vectorPtr(new Tvector) {}
     explicit VectorGeneral(const VectorGeneral<Treal, Tvector>& other)
       :FileWritable(other), vectorPtr(new Tvector) {
