@@ -1,6 +1,7 @@
-/* Ergo, version 3.4, a program for linear scaling electronic structure
+/* Ergo, version 3.5, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2014 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -52,9 +53,9 @@ int template_lapack_stemr(const char *jobz, const char *range, const integer *n,
     integer i__, j;
     Treal r1, r2;
     integer jj;
-    Treal cs;
+    Treal cs = 0;
     integer in;
-    Treal sn, wl, wu;
+    Treal sn = 0, wl, wu;
     integer iil, iiu;
     Treal eps, tmp;
     integer indd, iend, jblk, wend;
