@@ -1,6 +1,7 @@
-/* Ergo, version 3.4, a program for linear scaling electronic structure
+/* Ergo, version 3.5, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2014 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -37,6 +38,7 @@
 #include "machine_epsilon.h"
 #include "units.h"
 #include "SCF_statistics.h"
+#include "AllocatorManager.h"
 
 
 SCF_general::SCF_general(const Molecule& molecule_,
@@ -131,6 +133,15 @@ SCF_general::SCF_general(const Molecule& molecule_,
     write_matrix_in_matrix_market_format( S_symm, matOpts.inversePermutationHML, "S_matrix", 
 					  ss_id.str(), scfopts.method_and_basis_set );
   }
+  if( scfopts.create_mtx_files_S_and_quit == 1 ) {
+    // Write overlap matrix in matrix market format using different basis function orderings, and then quit.
+    create_mtx_files_with_different_orderings(S_symm,
+					      scfopts.calculation_identifier,
+					      scfopts.method_and_basis_set,
+					      matOpts.inversePermutationHML,
+					      basisInfo);
+    throw "Breaking because create_mtx_files_S_and_quit was set.";
+  }
   if ( scfopts.create_basis_func_coord_file == 1 ) {
     write_basis_func_coord_file(basisInfo);
   }
@@ -327,6 +338,147 @@ SCF_general::SCF_general(const Molecule& molecule_,
   energy_of_valence = 0; // only used when "core density matrix" is used
   energy_reference = 0; // only used when "core density matrix" is used
   electronicEntropyTerm = 0;
+
+
+  /*
+    Set parameters in the GetDensFromFock class.
+    All parameters must be set up, otherwise you will get exception.
+   */
+  
+  // common parameters 
+  DensFromFock.set_general_params(n,
+	             		  matOpts.size_block_info);
+
+#ifdef USE_CHUNKS_AND_TASKS
+  DensFromFock.set_cht_matrix_params(scfopts.cht_leavesSizeMax, scfopts.cht_blocksize);
+#endif
+
+  DensFromFock.set_truncationNormPurification(scfopts.purification_truncation_norm);
+  DensFromFock.set_stopCriterionNormPurification(scfopts.purification_stop_crit_norm);
+  DensFromFock.set_invCholFactor(invCholFactor, invCholFactor_euclnorm);
+  DensFromFock.set_gap_expected_lower_bound(scfopts.gap_expected_lower_bound);
+  DensFromFock.set_sparsity_plots_resolution_m(scfopts.sparsity_plots_resolution_m);
+  DensFromFock.set_purification_maxmul(scfopts.purification_maxmul);
+
+
+  int set = 1;
+
+  if( scfopts.do_sparsity_investigation == set)
+    DensFromFock.set_do_sparsity_investigation();
+  else
+    DensFromFock.unset_do_sparsity_investigation();
+
+   
+
+  if( scfopts.do_comparison_to_simple_purification == set )
+    DensFromFock.set_do_comparison_to_simple_purification();
+  else
+    DensFromFock.unset_do_comparison_to_simple_purification();
+
+
+  if( scfopts.do_puri_mmul_tests == set )
+    DensFromFock.set_do_puri_mmul_tests();
+  else
+    DensFromFock.unset_do_puri_mmul_tests();
+
+
+
+  if( scfopts.purification_create_m_files == set )
+    DensFromFock.set_purification_create_m_files();
+  else
+    DensFromFock.unset_purification_create_m_files();
+
+
+
+  if( scfopts.output_homo_and_lumo_eigenvectors == set )
+    DensFromFock.set_output_homo_and_lumo_eigenvectors();
+  else
+    DensFromFock.unset_output_homo_and_lumo_eigenvectors();
+
+
+
+  /*
+    do not plot figures from the purification by default
+    set flag before calling function for purification
+    unset after calling function for purification
+    otherwise, it will be output from all its calls
+  */
+  DensFromFock.unset_generate_figures();
+
+
+  if( scfopts.purification_ignore_failure == set )
+    DensFromFock.set_purification_ignore_failure();
+  else
+    DensFromFock.unset_purification_ignore_failure();
+
+
+
+  if( scfopts.purification_use_rand_perturbation_for_alleigsint == set )
+    DensFromFock.set_purification_use_rand_perturbation_for_alleigsint();
+  else
+    DensFromFock.unset_purification_use_rand_perturbation_for_alleigsint();
+
+
+  // purification or something else?
+  if(scfopts.use_stochastic_orbs != set && scfopts.use_diagonalization != set) // then use purification
+    DensFromFock.set_use_purification();
+  else
+    DensFromFock.unset_use_purification();
+
+  // purification settings
+  if(DensFromFock.get_use_purification() == set)
+    {
+      DensFromFock.set_purification_limits(scfopts.purification_subspace_err_limit,
+					   scfopts.purification_eigvalue_err_limit, 
+					   scfopts.puri_eig_acc_factor_for_guess);
+
+      if(scfopts.purification_with_acceleration == set)
+	DensFromFock.set_use_acceleration();
+      else
+	DensFromFock.unset_use_acceleration();
+
+      if(scfopts.use_new_stopping_criterion == set)
+	DensFromFock.set_use_new_stopping_criterion();
+      else
+	DensFromFock.unset_use_new_stopping_criterion();
+    }
+
+
+  if(scfopts.use_diag_on_error == set)
+    DensFromFock.set_use_diag_on_error();
+  else
+    DensFromFock.unset_use_diag_on_error();
+
+  if(scfopts.use_diag_on_error_guess == set)
+    DensFromFock.set_use_diag_on_error_guess();
+  else
+    DensFromFock.unset_use_diag_on_error_guess();
+
+
+
+
+  // diagonalization settings
+  if(scfopts.use_diagonalization == set)
+    DensFromFock.set_use_diagonalization();
+  else
+    DensFromFock.unset_use_diagonalization();
+  
+  // may be needed if scfopts.use_diag_on_error is set
+  DensFromFock.set_diagonalization_params(scfopts.electronic_temperature,
+					  S_symm);
+
+  if(scfopts.use_diagonalization == set && scfopts.store_all_eigenvalues_to_file == set)
+    DensFromFock.set_store_all_eigenvalues_to_file();
+  else
+    DensFromFock.unset_store_all_eigenvalues_to_file();
+  
+
+  // stochastic orbitals settings
+  if(scfopts.use_stochastic_orbs == set)
+    DensFromFock.set_use_stochastic_orbitals();
+  else
+    DensFromFock.unset_use_stochastic_orbitals();
+ 
 }
 
 
@@ -419,10 +571,13 @@ void SCF_general::do_SCF_iterations()
     throw "exiting after scfopts.output_density_images_only";
   }
 
+
   if(scfopts.use_stochastic_orbs == 1)
     prepare_stochastic_orbitals();
 
-  // Use at least two iterations if use_simple_starting_guess used.
+
+
+  // Use at least two iterations if disimple_starting_guess used.
   int min_number_of_iterations = scfopts.min_number_of_iterations;
   if(scfopts.use_simple_starting_guess && min_number_of_iterations < 2)
     min_number_of_iterations = 2;
@@ -430,6 +585,7 @@ void SCF_general::do_SCF_iterations()
   ergo_real stepLength = scfopts.step_length_start;
   ergo_real best_energy_so_far = 0;
   int step = 0;
+  SCF_step = 0;
   int noOfFailuresInARow = 0;
   int restartCount = 0;
   int noOfImprovementsInARow = 0;
@@ -457,6 +613,7 @@ void SCF_general::do_SCF_iterations()
       curr_cycle_stats->add_value("no_of_basis_func", basisInfo.noOfBasisFuncs);
       curr_cycle_stats->add_value("sparse_matrix_block_size", matOpts.sparse_matrix_block_size);
       step++;
+      SCF_step++;
       curr_cycle_stats->add_value("scf_cycle", step);
       char infoString[888];
       sprintf(infoString, "Beginning of SCF cycle %i: ", step);
@@ -523,6 +680,10 @@ void SCF_general::do_SCF_iterations()
 	}
       if(errorMeasure < scfopts.convergence_threshold && step >= min_number_of_iterations)
 	{
+	  // Write Fock matrix in matrix market format in the last SCF cycle
+	  if ( scfopts.create_mtx_files_F == 1 )
+	    create_mtx_files_F( step );
+
 	  do_output(LOG_CAT_RESULTS, LOG_AREA_SCF, "CONVERGED after %3i iterations.", step);
 	  do_output(LOG_CAT_RESULTS, LOG_AREA_SCF, "FINAL ENERGY: %22.11f", (double)energy);
 	  report_final_results();
@@ -532,6 +693,10 @@ void SCF_general::do_SCF_iterations()
       // Check if max number of iterations reached
       if(scfopts.max_number_of_iterations > 0 && step >= scfopts.max_number_of_iterations)
 	{
+	  // Write Fock matrix in matrix market format in the last SCF cycle
+	  if ( scfopts.create_mtx_files_F == 1 )
+	    create_mtx_files_F( step );
+
 	  do_output(LOG_CAT_INFO, LOG_AREA_SCF, "Maximum number of SCF iterations reached. Breaking SCF procedure.");
 	  do_output(LOG_CAT_INFO, LOG_AREA_SCF, "Gave up after %i iterations.", step);
 	  break;
@@ -695,9 +860,12 @@ void SCF_general::do_SCF_iterations()
       curr_cycle_stats->stop_timer("get_new_density_matrix");
       
       // At this point a new density matrix has just been computed, so the electronic entropy term has also been computed in the nonzero-temperature case.
-      if(scfopts.electronic_temperature > 0)
+      if(scfopts.electronic_temperature > 0) {
+	do_output(LOG_CAT_INFO, LOG_AREA_SCF, "Electronic temperature: %22.11f a.u. = %22.2f Kelvin", 
+		  (double)scfopts.electronic_temperature, (double)scfopts.electronic_temperature/UNIT_one_Kelvin);
 	do_output(LOG_CAT_INFO, LOG_AREA_SCF, "Electronic entropy term: %22.11f, energy with entropy term included: %22.11f", 
 		  (double)electronicEntropyTerm, (double)(energy + electronicEntropyTerm));
+      }
 
       if(step > 1)
 	report_density_difference();
@@ -726,8 +894,7 @@ void SCF_general::do_SCF_iterations()
 
       if ( scfopts.output_homo_and_lumo_eigenvectors ) {
 	// Write homo and lumo eigenvectors to file
-	create_homo_eigvec_file();
-	create_lumo_eigvec_file();
+	create_eigenvectors_files();
 	create_gabedit_file();
       }
             
@@ -746,6 +913,8 @@ void SCF_general::do_SCF_iterations()
 
   timeMeterScfMainLoop.print(LOG_AREA_SCF, "Main SCF loop");
   output_current_memory_usage(LOG_AREA_SCF, "after main SCF loop");
+  std::string allocStatsStr = mat::AllocatorManager<ergo_real>::instance().getStatistics();
+  do_output(LOG_CAT_INFO, LOG_AREA_SCF, "After main SCF loop: %s", allocStatsStr.c_str());
 
   if(scfopts.output_density_at_every_step == 1 && step == 1) {
     Util::TimeMeter timeMeterWriteDensityToFile;
