@@ -1,6 +1,7 @@
-/* Ergo, version 3.4, a program for linear scaling electronic structure
+/* Ergo, version 3.5, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2014 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -32,7 +33,6 @@
 #include "utilities.h"
 #include "diis_restricted.h"
 #include "density_projection.h"
-#include "densfromf_general.h"
 #include "density_description_file.h"
 #include "matrix_utilities.h"
 #include "matrix_utilities_gao.h"
@@ -40,6 +40,7 @@
 #include "units.h"
 #include "atom_labels.h"
 #include "integral_matrix_wrappers.h"
+#include "dipole_moment.h"
 
 
 SCF_restricted::SCF_restricted(const Molecule& molecule_,
@@ -65,7 +66,13 @@ SCF_restricted::SCF_restricted(const Molecule& molecule_,
                   matOpts_,
 		  threshold_integrals_1el_input)
 {
+
   DIIS = new DIISManagerRestricted;
+
+
+  DensFromFock.do_restricted_calculations(); // set factor = 2
+  DensFromFock.set_no_occupied_orbs( noOfElectrons/2 );
+ 
 }
 
 
@@ -108,6 +115,8 @@ void SCF_restricted::initialize_matrices()
 					   matOpts.size_block_info);
   F_ort_prev.resetSizesAndBlocks(matOpts.size_block_info,
 					   matOpts.size_block_info);
+  // D_ort_prev.resetSizesAndBlocks(matOpts.size_block_info,
+  //                                 matOpts.size_block_info); 
   bestFockMatrixSoFar.resetSizesAndBlocks(matOpts.size_block_info,
 					   matOpts.size_block_info);
   bestFockMatrixSoFar2.resetSizesAndBlocks(matOpts.size_block_info,
@@ -140,7 +149,8 @@ void SCF_restricted::get_starting_guess_density()
       symmMatrix* matrixList[2];
       matrixList[0] = &densityMatrix;
 
-      if(load_density_and_project_sparse(guessDmatFileName,
+      if(load_density_and_project_sparse(DensFromFock,
+					 guessDmatFileName,
 					 noOfDensityMatrices,
 					 &integralInfo, 
 					 basisInfo,
@@ -149,20 +159,7 @@ void SCF_restricted::get_starting_guess_density()
 					 &noOfElectrons,
 					 matOpts.size_block_info,
 					 matOpts.permutationHML,
-					 matOpts.sparse_threshold,
-					 invCholFactor,
-					 invCholFactor_euclnorm,
-					 scfopts.gap_expected_lower_bound,
-					 scfopts.purification_eigvalue_err_limit * scfopts.puri_eig_acc_factor_for_guess,
-					 scfopts.purification_subspace_err_limit,
-					 scfopts.purification_truncation_norm,
-					 scfopts.purification_maxmul,
-					 scfopts.purification_create_m_files,
-					 scfopts.use_diagonalization,
-					 scfopts.use_diag_on_error_guess,
-					 scfopts.purification_ignore_failure,
-					 scfopts.purification_use_rand_perturbation_for_alleigsint,
-					 scfopts.electronic_temperature) != 0)
+					 matOpts.sparse_threshold) != 0)
 	{
 	  do_output(LOG_CAT_ERROR, LOG_AREA_SCF, "error in load_density_and_project_sparse");
 	  throw "error in load_density_and_project_sparse";
@@ -186,57 +183,34 @@ void SCF_restricted::get_starting_guess_density()
       else
 	{
 	  do_output(LOG_CAT_INFO, LOG_AREA_SCF, 
-		    "calling get_dens_from_fock_general to diagonalize H_core for starting guess, n = %i, sparse_threshold = %g", 
+		    "calling get_dens_from_fock to diagonalize H_core for starting guess, n = %i, sparse_threshold = %g", 
 		    n, (double)matOpts.sparse_threshold);
 	  
 	  symmMatrix F_ort_prev_dummy;
 	  F_ort_prev_dummy.resetSizesAndBlocks(matOpts.size_block_info,
 					   matOpts.size_block_info);
 	  F_ort_prev_dummy.writeToFile();
-	  intervalType homoInterval_dummy1(-1e22,1e22);
-	  intervalType lumoInterval_dummy1(-1e22,1e22);
-	  intervalType homoInterval_dummy2(-1e22,1e22);
-	  intervalType lumoInterval_dummy2(-1e22,1e22);
 	  densityMatrix.writeToFile();
-	  std::map<std::string, double> puri_stats;
-	  ergo_real entropyTermDummy = 0;
-	  if(get_dens_from_fock_general(n,
-					noOfElectrons / 2,
-					scfopts.use_diagonalization,
-					scfopts.use_diag_on_error_guess,
-					scfopts.electronic_temperature,
-					densityMatrix,
-					2,
-					entropyTermDummy,
-					H_core_Matrix,
-					homoInterval_dummy1,
-					lumoInterval_dummy1,
-					S_symm,
-					invCholFactor,
-					invCholFactor_euclnorm,
-					scfopts.gap_expected_lower_bound,
-					matOpts.size_block_info,
-					F_ort_prev_dummy,
-					homoInterval_dummy2,
-					lumoInterval_dummy2,
-					scfopts.purification_eigvalue_err_limit,
-					scfopts.purification_subspace_err_limit,
-					scfopts.purification_truncation_norm,
-					scfopts.purification_maxmul,
-					scfopts.purification_create_m_files,
-					scfopts.purification_ignore_failure,
-					scfopts.purification_use_rand_perturbation_for_alleigsint,
-					scfopts.use_stochastic_orbs,
-					stochastic_orbitals,
-					"",
-					puri_stats,
-					scfopts.do_sparsity_investigation,
-					scfopts.sparsity_plots_resolution_m,
-					scfopts.do_comparison_to_simple_purification,
-					scfopts.do_puri_mmul_tests) != 0)
+
+	  DensFromFock.clean_eigs_intervals();
+
+	  int use_diag_on_error = DensFromFock.get_use_diag_on_error();
+
+	  if(DensFromFock.get_use_diag_on_error_guess() == 1)
+	    DensFromFock.set_use_diag_on_error();
+      
+
+  	  if(DensFromFock.get_dens_from_fock(H_core_Matrix,
+					     densityMatrix,
+					     F_ort_prev_dummy) != 0)
 	    {
 	      throw "SCF_restricted::get_starting_guess_density: Error in get_dens_from_fock_general";
 	    }
+
+	  if( use_diag_on_error != 1 )
+	    DensFromFock.unset_use_diag_on_error();
+
+
 	} // END ELSE use H_core
     } // END ELSE no dmat given
 
@@ -245,6 +219,8 @@ void SCF_restricted::get_starting_guess_density()
   densityMatrix.writeToFile();
 
   densityMatrix_core.writeToFile(); // densityMatrix_core (if used) also needs to be written to file
+
+  
 }
 
 
@@ -269,6 +245,8 @@ void SCF_restricted::initialize_homo_lumo_limits()
   intervalType hugeInterval(-1e22, 1e22);
   homoInterval_F_ort_prev = hugeInterval;
   lumoInterval_F_ort_prev = hugeInterval;
+  homo2Interval_F_ort_prev = hugeInterval;
+  lumo2Interval_F_ort_prev = hugeInterval;
   homoInterval_Fprev = hugeInterval;
   lumoInterval_Fprev = hugeInterval;
 }
@@ -816,47 +794,26 @@ void SCF_restricted::get_new_density_matrix()
 	      "do_sparsity_investigation_reppuri, running purification three times using different norms.");
     mat::normType ntype[3] = {mat::frobNorm, mat::mixedNorm, mat::euclNorm};
     std::map<std::string, double> puri_stats;
+
+    
     for (int i = 0; i<3; i++) {      
       // Copy input parameters that will be overwritten by purification
       symmMatrix F_ort_prev_copy(F_ort_prev);
       intervalType homoInterval_F_ort_prev_copy(homoInterval_F_ort_prev);
       intervalType lumoInterval_F_ort_prev_copy(lumoInterval_F_ort_prev);
-      if(get_dens_from_fock_general(n,
-				    noOfElectrons / 2,
-				    scfopts.use_diagonalization,
-				    scfopts.use_diag_on_error,
-				    scfopts.electronic_temperature,
-				    densityMatrix,
-				    2,
-				    electronicEntropyTerm,
-				    FockMatrix, // should be same as Fprev now
-				    homoInterval_Fprev, // output
-				    lumoInterval_Fprev, // output
-				    S_symm,
-				    invCholFactor,
-				    invCholFactor_euclnorm,
-				    scfopts.gap_expected_lower_bound,
-				    matOpts.size_block_info,
-				    F_ort_prev_copy,
-				    homoInterval_F_ort_prev_copy,
-				    lumoInterval_F_ort_prev_copy,
-				    scfopts.purification_eigvalue_err_limit,
-				    scfopts.purification_subspace_err_limit,
-				    ntype[i],
-				    scfopts.purification_maxmul,
-				    scfopts.purification_create_m_files,
-				    scfopts.purification_ignore_failure,
-				    scfopts.purification_use_rand_perturbation_for_alleigsint,
-				    scfopts.use_stochastic_orbs,
-				    stochastic_orbitals,
-				    "investigation_"+mat::getNormTypeString(ntype[i])+"_",
-				    puri_stats,
-				    scfopts.do_sparsity_investigation,
-				    scfopts.sparsity_plots_resolution_m,
-				    0,
-				    scfopts.do_puri_mmul_tests) != 0)
+
+ 
+      DensFromFock.clean_puri_stats();
+      DensFromFock.set_stats_prefix( "investigation_"+mat::getNormTypeString(ntype[i])+"_");
+      DensFromFock.set_eigs_F_ort_prev( homoInterval_F_ort_prev_copy,
+					lumoInterval_F_ort_prev_copy);
+      DensFromFock.set_truncationNormPurification(ntype[i]);
+
+      if(DensFromFock.get_dens_from_fock(FockMatrix,
+					 densityMatrix,
+					 F_ort_prev_copy) != 0)
 	{
-	  throw "SCF_restricted::get_new_density_matrix: Error in get_dens_from_fock_general";
+	  throw "SCF_restricted::get_new_density_matrix: Error in get_dens_from_fock";
 	}
       
     } // end for       
@@ -869,7 +826,11 @@ void SCF_restricted::get_new_density_matrix()
 				  scfopts.sparsity_plots_resolution_m);
     densityMatrix.writeToFile();      
     do_output(LOG_CAT_INFO, LOG_AREA_SCF, "do_sparsity_investigation for purification finished, continuing as usual.");    
+
+    DensFromFock.get_puri_stats(puri_stats);
     this->curr_cycle_stats->add_values(puri_stats);
+    DensFromFock.set_stats_prefix("");
+
   } // end if investigation
 
 
@@ -893,92 +854,76 @@ void SCF_restricted::get_new_density_matrix()
     F_effective = &F_modified;
   }
 
-  std::map<std::string, double> puri_stats;
-  if(get_dens_from_fock_general(n,
-				noOfElectrons / 2,
-				scfopts.use_diagonalization,
-				scfopts.use_diag_on_error,
-				scfopts.electronic_temperature,
-				densityMatrix,
-				2,
-				electronicEntropyTerm,
-				*F_effective, // should normally be same as Fprev now
-				homoInterval_Fprev, // output
-				lumoInterval_Fprev, // output
-				S_symm,
-				invCholFactor,
-				invCholFactor_euclnorm,
-				scfopts.gap_expected_lower_bound,
-				matOpts.size_block_info,
-				F_ort_prev,
-				homoInterval_F_ort_prev,
-				lumoInterval_F_ort_prev,
-				scfopts.purification_eigvalue_err_limit,
-				scfopts.purification_subspace_err_limit,
-				scfopts.purification_truncation_norm,
-				scfopts.purification_maxmul,
-				scfopts.purification_create_m_files,
-				scfopts.purification_ignore_failure,
-				scfopts.purification_use_rand_perturbation_for_alleigsint,
-				scfopts.use_stochastic_orbs,
-				stochastic_orbitals,
-				"",
-				puri_stats,
-				scfopts.do_sparsity_investigation,
-				scfopts.sparsity_plots_resolution_m,
-				scfopts.do_comparison_to_simple_purification,
-				scfopts.do_puri_mmul_tests,
-				&eigVecLUMO,
-				&eigVecHOMO) != 0)
-    {
-      throw "SCF_restricted::get_new_density_matrix: Error in get_dens_from_fock_general";
+  DensFromFock.set_eigs_F_ort_prev(homoInterval_F_ort_prev, lumoInterval_F_ort_prev);
+  DensFromFock.set_eigs2_F_ort_prev(homo2Interval_F_ort_prev, lumo2Interval_F_ort_prev);
+  DensFromFock.clean_puri_stats();
+  DensFromFock.set_generate_figures(SCF_step);
+
+  // if eigenvectors are needed, set params
+  int use_init_guess = 0;
+  if(scfopts.use_prev_vector_as_initial_guess == 1 && 
+     scfopts.min_number_of_iterations <= SCF_step &&
+     SCF_step > 1 &&
+     !eigVecLUMO.is_empty() && !eigVecHOMO.is_empty()) // ensure that we computed vectors in previous cycle
+    use_init_guess = 1;
+    
+  // if we use new purification
+  if(SCF_step > 1 && 
+     DensFromFock.get_use_purification() == 1 &&
+     DensFromFock.get_output_homo_and_lumo_eigenvectors() == 1)
+    {   
+      DensFromFock.compute_eigenvectors(scfopts.eigenvectors_method, 
+					scfopts.eigenvectors_iterative_method, 
+					scfopts.number_of_eigenpairs, 
+					scfopts.eigensolver_accuracy, 
+					scfopts.eigensolver_maxiter,
+					use_init_guess,
+					scfopts.try_eigv_on_next_iteration_if_fail);
+
+      // if we need more eigenpairs, initialize pointers in the DensFromFock class
+      if(scfopts.number_of_eigenpairs == 2)
+	DensFromFock.set_second_eigenvectors_pointers(&eigVecLUMO2, &eigVecHOMO2);
     }
+
+  if(scfopts.create_checkpoints == 1)
+    {
+      Util::TimeMeter timeMeter;
+      DensFromFock.create_checkpoint(*F_effective, // should normally be same as Fprev now
+				     F_ort_prev,
+				     &eigVecLUMO,
+				     &eigVecHOMO, 
+				     scfopts.checkpoint_IDstr);
+      timeMeter.print(LOG_AREA_SCF, "in SCF_restricted DensFromFock::create_checkpoint took");
+    }
+
+  if(DensFromFock.get_dens_from_fock(*F_effective, // should normally be same as Fprev now
+				     densityMatrix,
+				     F_ort_prev,
+				     &eigVecLUMO,
+			             &eigVecHOMO) != 0)
+    {
+      throw "SCF_restricted::get_new_density_matrix: Error in get_dens_from_fock";
+    }
+  
+	
+  DensFromFock.get_eigs_F_ort_prev(homoInterval_F_ort_prev, lumoInterval_F_ort_prev);
+  DensFromFock.get_eigs2_F_ort_prev(homo2Interval_F_ort_prev, lumo2Interval_F_ort_prev);
+  DensFromFock.get_eigs_Fprev(homoInterval_Fprev, lumoInterval_Fprev);
+
+  std::map<std::string, double> puri_stats;
+  DensFromFock.get_puri_stats(puri_stats);
   this->curr_cycle_stats->add_values(puri_stats);
 
+  DensFromFock.unset_generate_figures();
+  electronicEntropyTerm = DensFromFock.get_result_entropy_term();
+
   if(scfopts.compute_core_density == 1) {
-    std::map<std::string, double> puri_stats_unused;
-    intervalType homoInterval_dummy1(-1e22,1e22);
-    intervalType lumoInterval_dummy1(-1e22,1e22);
-    intervalType homoInterval_dummy2(-1e22,1e22);
-    intervalType lumoInterval_dummy2(-1e22,1e22);
-    if(get_dens_from_fock_general(n,
-				  scfopts.no_of_core_electrons / 2,
-				  scfopts.use_diagonalization,
-				  scfopts.use_diag_on_error,
-				  scfopts.electronic_temperature,
-				  densityMatrix_core,
-				  2,
-				  electronicEntropyTerm,
-				  *F_effective, // should normally be same as Fprev now
-				  homoInterval_dummy1, // output
-				  lumoInterval_dummy1, // output
-				  S_symm,
-				  invCholFactor,
-				  invCholFactor_euclnorm,
-				  scfopts.gap_expected_lower_bound,
-				  matOpts.size_block_info,
-				  F_ort_prev,
-				  homoInterval_dummy2,
-				  lumoInterval_dummy2,
-				  scfopts.purification_eigvalue_err_limit,
-				  scfopts.purification_subspace_err_limit,
-				  scfopts.purification_truncation_norm,
-				  scfopts.purification_maxmul,
-				  scfopts.purification_create_m_files,
-				  scfopts.purification_ignore_failure,
-				  scfopts.purification_use_rand_perturbation_for_alleigsint,
-				  scfopts.use_stochastic_orbs,
-				  stochastic_orbitals,
-				  "",
-				  puri_stats_unused,
-				  0,
-				  0,
-				  0,
-				  0,
-				  NULL,
-				  NULL) != 0)
+    DensFromFock.clean_eigs_intervals();
+    if(DensFromFock.get_dens_from_fock(*F_effective,
+				       densityMatrix_core,
+				       F_ort_prev) != 0)
       {
-	throw "SCF_restricted::get_new_density_matrix: Error in get_dens_from_fock_general for core density matrix.";
+	throw "SCF_restricted::get_new_density_matrix: Error in get_dens_from_fock for core density matrix.";
       }
   }
 
@@ -1058,6 +1003,7 @@ void SCF_restricted::save_full_matrices_for_matlab()
 }
 
 
+
 void SCF_restricted::output_density_images()
 {
   Util::TimeMeter timeMeter;
@@ -1095,6 +1041,7 @@ void SCF_restricted::output_density_images()
 }
 
 
+
 static ergo_real rand_minus1_to_1(int rand_param) {
   if(rand_param == 0) {
     // ELIAS NOTE 2013-11-14: this is the old bad implementation that gives a slightly "unfair" distribution in the [-1,1] interval.
@@ -1306,6 +1253,9 @@ void SCF_restricted::prepare_stochastic_orbitals() {
 #endif
     }
   }
+
+  DensFromFock.set_stochastic_orbitals_params(stochastic_orbitals);
+
   do_output(LOG_CAT_INFO, LOG_AREA_SCF, "SCF_restricted::prepare_stochastic_orbitals finished OK.");
   timeMeter.print(LOG_AREA_SCF, "SCF_restricted::prepare_stochastic_orbitals");
 }
@@ -1389,7 +1339,7 @@ void SCF_restricted::compute_dipole_moment()
 {
   do_output(LOG_CAT_INFO, LOG_AREA_SCF, "SCF_restricted::compute_dipole_moment");
   densityMatrix.readFromFile();  
-  get_dipole_moment(densityMatrix, basisInfo, matOpts.size_block_info, matOpts.permutationHML, molecule);
+  get_dipole_moment(densityMatrix, basisInfo, matOpts.size_block_info, matOpts.permutationHML, molecule, LOG_AREA_SCF, "SCF");
   densityMatrix.writeToFile();
 }
 
@@ -1434,49 +1384,42 @@ void SCF_restricted::create_mtx_files_D(int const scfIter)
   densityMatrix.writeToFile();
 }
 
-void SCF_restricted::create_homo_eigvec_file() const
+
+void SCF_restricted::create_eigenvectors_files() const
 {
-  if (eigVecHOMO.is_empty()) {
-    do_output(LOG_CAT_WARNING, LOG_AREA_SCF, "Failed to output HOMO eigenvector to file. No HOMO eigenvector stored.");
+  create_eigvec_file(eigVecHOMO, 
+		     "HOMO", 
+		     "homo_coefficient_vec");
+  create_eigvec_file(eigVecLUMO, 
+		     "LUMO", 
+		     "lumo_coefficient_vec");
+
+}
+
+void SCF_restricted::create_eigvec_file(const generalVector &eigVec, 
+					const char *vector_name, 
+					const char *filename_id) const
+{
+  if (eigVec.is_empty()) {
+    do_output(LOG_CAT_WARNING, LOG_AREA_SCF, "Failed to output %s to file. No %s eigenvector stored.", vector_name, vector_name);
     return;
   }
-  do_output(LOG_CAT_INFO, LOG_AREA_SCF, "Storing HOMO eigenvector to file homo_coefficient_vec.txt.");
+  do_output(LOG_CAT_INFO, LOG_AREA_SCF, "Storing %s eigenvector to file %s.", vector_name, filename_id);
   int n = basisInfo.noOfBasisFuncs;
-  std::vector<ergo_real> homo_vec_perm(n);
-  eigVecHOMO.fullvector(homo_vec_perm); 
+  std::vector<ergo_real> vec_perm(n);
+  eigVec.fullvector(vec_perm); 
   // now we have the permuted vector
-  std::vector<ergo_real> homo_vec(n);
+  std::vector<ergo_real> vec(n);
   for (int ind = 0; ind < n; ind++) 
-    homo_vec[ind] = homo_vec_perm[matOpts.inversePermutationHML[ind]];
+    vec[ind] = vec_perm[matOpts.inversePermutationHML[ind]];
   char ffname[888];
-  sprintf(ffname, "homo_coefficient_vec.txt");
+  sprintf(ffname, "%s.txt", filename_id);
   std::ofstream ff(ffname);
   for (int ind = 0; ind < n; ind++) 
-    ff << homo_vec[ind] << std::endl;
+    ff << vec[ind] << std::endl;
   ff.close();  
 }
 
-void SCF_restricted::create_lumo_eigvec_file() const
-{
-  if (eigVecLUMO.is_empty()) {
-    do_output(LOG_CAT_WARNING, LOG_AREA_SCF, "Failed to output LUMO eigenvector to file. No LUMO eigenvector stored.");
-    return;
-  }
-  do_output(LOG_CAT_INFO, LOG_AREA_SCF, "Storing LUMO eigenvector to file lumo_coefficient_vec.txt.");
-  int n = basisInfo.noOfBasisFuncs;
-  std::vector<ergo_real> lumo_vec_perm(n);
-  eigVecLUMO.fullvector(lumo_vec_perm); 
-  // now we have the permuted vector
-  std::vector<ergo_real> lumo_vec(n);
-  for (int ind = 0; ind < n; ind++) 
-    lumo_vec[ind] = lumo_vec_perm[matOpts.inversePermutationHML[ind]];
-  char ffname[888];
-  sprintf(ffname, "lumo_coefficient_vec.txt");
-  std::ofstream ff(ffname);
-  for (int ind = 0; ind < n; ind++) 
-    ff << lumo_vec[ind] << std::endl;
-  ff.close();  
-}
 
 
 static void 
@@ -1513,7 +1456,18 @@ output_orbital_coeffs_in_gabedit_order(const BasisInfoStruct & basisInfo,
       ff << count+1 << "   " << orbital_vec[startIdx+6] << std::endl; count++;
       ff << count+1 << "   " << orbital_vec[startIdx+0] << std::endl; count++;
       break;
-    default: throw "error in output_orbital_coeffs_in_gabedit_order: shell types beyond f not implemented!";
+    case 4: // g-type shell
+      ff << count+1 << "   " << orbital_vec[startIdx+4] << std::endl; count++;
+      ff << count+1 << "   " << orbital_vec[startIdx+5] << std::endl; count++;
+      ff << count+1 << "   " << orbital_vec[startIdx+3] << std::endl; count++;
+      ff << count+1 << "   " << orbital_vec[startIdx+6] << std::endl; count++;
+      ff << count+1 << "   " << orbital_vec[startIdx+2] << std::endl; count++;
+      ff << count+1 << "   " << orbital_vec[startIdx+7] << std::endl; count++;
+      ff << count+1 << "   " << orbital_vec[startIdx+1] << std::endl; count++;
+      ff << count+1 << "   " << orbital_vec[startIdx+8] << std::endl; count++;
+      ff << count+1 << "   " << orbital_vec[startIdx+0] << std::endl; count++;
+      break;
+    default: throw "error in output_orbital_coeffs_in_gabedit_order: shell types beyond g not implemented!";
     }
   }
   if(count != basisInfo.noOfBasisFuncs)
@@ -1590,7 +1544,8 @@ void SCF_restricted::create_gabedit_file() const
       case 1: shellChar = 'p'; break;
       case 2: shellChar = 'd'; break;
       case 3: shellChar = 'f'; break;
-      default: throw "SCF_restricted::create_gabedit_file error: shell types beyond f not implemented!";
+      case 4: shellChar = 'g'; break;
+      default: throw "SCF_restricted::create_gabedit_file error: shell types beyond g not implemented!";
       }
       ff << shellChar << " " << basisInfo.shellList[k].noOfContr << " 1.00" << std::endl;
       for(int contridx = 0; contridx < basisInfo.shellList[k].noOfContr; contridx++) {
@@ -1623,6 +1578,7 @@ void SCF_restricted::create_gabedit_file() const
 }
 
 
+
 void SCF_restricted::update_subspace_diff()
 {
   densityMatrix.readFromFile();
@@ -1762,56 +1718,32 @@ void SCF_restricted::disturb_dens_matrix_exact_try(const symmMatrix & randomMatr
   F_ort_prev_dummy.resetSizesAndBlocks(matOpts.size_block_info,
 					   matOpts.size_block_info);
   F_ort_prev_dummy.writeToFile();
-  intervalType homoInterval_tmp1(-1e22,1e22);
-  intervalType lumoInterval_tmp1(-1e22,1e22);
-  intervalType homoInterval_tmp2(-1e22,1e22);
-  intervalType lumoInterval_tmp2(-1e22,1e22);
 
   resultDensMatrix.writeToFile();
   SDS_symm.writeToFile();
-  int use_diag = 0;
-  int use_diag_on_error = 0;
-  int n = basisInfo.noOfBasisFuncs;
-  std::map<std::string, double> puri_stats;
-  ergo_real electronicEntropyTermDummy = 0;
-  if(get_dens_from_fock_general(n,
-				noOfElectrons / 2,
-				use_diag,
-				use_diag_on_error,
-				scfopts.electronic_temperature,
-				resultDensMatrix,
-				2,
-				electronicEntropyTermDummy,
-				SDS_symm,
-				homoInterval_tmp1,
-				lumoInterval_tmp1,
-				S_symm,
-				invCholFactor,
-				invCholFactor_euclnorm,
-				scfopts.gap_expected_lower_bound,
-				matOpts.size_block_info,
-				F_ort_prev_dummy,
-				homoInterval_tmp2,
-				lumoInterval_tmp2,
-				scfopts.purification_eigvalue_err_limit,
-				scfopts.purification_subspace_err_limit,
-				scfopts.purification_truncation_norm,
-				scfopts.purification_maxmul,
-				scfopts.purification_create_m_files,
-				scfopts.purification_ignore_failure,
-				scfopts.purification_use_rand_perturbation_for_alleigsint,
-				scfopts.use_stochastic_orbs,
-				stochastic_orbitals,
-				"",
-				puri_stats,
-				scfopts.do_sparsity_investigation,
-				scfopts.sparsity_plots_resolution_m,
-				scfopts.do_comparison_to_simple_purification,
-				scfopts.do_puri_mmul_tests) != 0)
+
+
+  int use_diag = DensFromFock.get_use_diagonalization();
+  int use_diag_on_error = DensFromFock.get_use_diag_on_error();
+
+  DensFromFock.unset_use_diagonalization();
+  DensFromFock.unset_use_diag_on_error();
+
+  DensFromFock.clean_eigs_intervals();
+ 
+  if(DensFromFock.get_dens_from_fock(SDS_symm, 
+				     resultDensMatrix,
+				     F_ort_prev_dummy) != 0)
     {
-      throw "SCF_restricted::get_starting_guess_density: Error in get_dens_from_fock_general";
+      throw "SCF_restricted::disturb_dens_matrix_exact_try: Error in get_dens_from_fock";
     }
 
+  if( use_diag == 1 )
+    DensFromFock.set_use_diagonalization();
+  if( use_diag_on_error == 1 )
+    DensFromFock.set_use_diag_on_error();
+
+
   // OK, now we have computed D_Pure which is the purified version of SDS_symm.
   // But D_Pure is not in orthogonal basis.
 
@@ -1885,58 +1817,39 @@ void SCF_restricted::disturb_fock_matrix(ergo_real subspaceError)
   F_ort_prev_dummy.resetSizesAndBlocks(matOpts.size_block_info,
 					   matOpts.size_block_info);
   F_ort_prev_dummy.writeToFile();
-  intervalType homoInterval_tmp1(-1e22,1e22);
-  intervalType lumoInterval_tmp1(-1e22,1e22);
-  intervalType homoInterval_tmp2(-1e22,1e22);
-  intervalType lumoInterval_tmp2(-1e22,1e22);
 
   symmMatrix densityMatrix_dummy;
   densityMatrix_dummy.resetSizesAndBlocks(matOpts.size_block_info,
 					   matOpts.size_block_info);
   densityMatrix_dummy.writeToFile();
-  int use_diag = 0;
-  int use_diag_on_error = 0;
-  int n = basisInfo.noOfBasisFuncs;
 
-  std::map<std::string, double> puri_stats;
-  if(get_dens_from_fock_general(n,
-				noOfElectrons / 2,
-				use_diag,
-				use_diag_on_error,
-				scfopts.electronic_temperature,
-				densityMatrix_dummy,
-				2,
-				electronicEntropyTerm,
-				FockMatrix,
-				homoInterval_tmp1,
-				lumoInterval_tmp1,
-				S_symm,
-				invCholFactor,
-				invCholFactor_euclnorm,
-				scfopts.gap_expected_lower_bound,
-				matOpts.size_block_info,
-				F_ort_prev_dummy,
-				homoInterval_tmp2,
-				lumoInterval_tmp2,
-				scfopts.purification_eigvalue_err_limit,
-				scfopts.purification_subspace_err_limit,
-				scfopts.purification_truncation_norm,
-				scfopts.purification_maxmul,
-				scfopts.purification_create_m_files,
-				scfopts.purification_ignore_failure,
-				scfopts.purification_use_rand_perturbation_for_alleigsint,
-				scfopts.use_stochastic_orbs,
-				stochastic_orbitals,
-				"",
-				puri_stats,
-				scfopts.do_sparsity_investigation,
-				scfopts.sparsity_plots_resolution_m,
-				scfopts.do_comparison_to_simple_purification,
-				scfopts.do_puri_mmul_tests) != 0)
+
+
+  int use_diag = DensFromFock.get_use_diagonalization();
+  int use_diag_on_error = DensFromFock.get_use_diag_on_error();
+
+  DensFromFock.unset_use_diagonalization();
+  DensFromFock.unset_use_diag_on_error();
+
+  DensFromFock.clean_eigs_intervals();
+  
+  if(DensFromFock.get_dens_from_fock(FockMatrix, 
+				     densityMatrix_dummy,
+				     F_ort_prev_dummy) != 0)
     {
-      throw "SCF_restricted::get_starting_guess_density: Error in get_dens_from_fock_general";
+      throw "SCF_restricted::disturb_fock_matrix: Error in get_dens_from_fock";
     }
 
+  if( use_diag == 1 )
+    DensFromFock.set_use_diagonalization();
+  if( use_diag_on_error == 1 )
+    DensFromFock.set_use_diag_on_error();
+
+  intervalType homoInterval_tmp2;
+  intervalType lumoInterval_tmp2;
+  DensFromFock.get_eigs_F_ort_prev(homoInterval_tmp2, lumoInterval_tmp2);
+
+
   do_output(LOG_CAT_INFO, LOG_AREA_SCF, "SCF_restricted::disturb_fock_matrix, interval sizes: %22.11f %22.11f",
 	    (double)(homoInterval_tmp2.upp() - homoInterval_tmp2.low()), (double)(lumoInterval_tmp2.upp() - lumoInterval_tmp2.low()));
   
