@@ -1,6 +1,7 @@
-/* Ergo, version 3.4, a program for linear scaling electronic structure
+/* Ergo, version 3.5, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2014 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -32,13 +33,13 @@
 #include "utilities.h"
 #include "diis_unrestricted.h"
 #include "density_projection.h"
-#include "densfromf_sparse.h"
 #include "densfromf_full.h"
-#include "densfromf_general.h"
 #include "density_description_file.h"
 #include "densitymanager.h"
 #include "matrix_utilities.h"
 #include "units.h"
+#include "atom_labels.h"
+#include "dipole_moment.h"
 
 
 SCF_unrestricted::SCF_unrestricted(const Molecule& molecule_,
@@ -72,6 +73,9 @@ SCF_unrestricted::SCF_unrestricted(const Molecule& molecule_,
       do_output(LOG_CAT_ERROR, LOG_AREA_SCF, "error in determine_number_of_electrons_unrestricted");
       throw "error in determine_number_of_electrons_unrestricted";
     }
+
+  DensFromFock.do_unrestricted_calculations(); // set factor = 1
+
 }
 
 SCF_unrestricted::~SCF_unrestricted()
@@ -112,10 +116,10 @@ void SCF_unrestricted::initialize_matrices()
 					   matOpts.size_block_info);
   Fprev_beta.resetSizesAndBlocks(matOpts.size_block_info,
 					   matOpts.size_block_info);
-  Dprev_alpha.resetSizesAndBlocks(matOpts.size_block_info,
-					   matOpts.size_block_info);
-  Dprev_beta.resetSizesAndBlocks(matOpts.size_block_info,
-					   matOpts.size_block_info);
+  // Dprev_alpha.resetSizesAndBlocks(matOpts.size_block_info,
+  // 					   matOpts.size_block_info);
+  // Dprev_beta.resetSizesAndBlocks(matOpts.size_block_info,
+  // 					   matOpts.size_block_info);
   F_ort_prev_alpha.resetSizesAndBlocks(matOpts.size_block_info,
 					   matOpts.size_block_info);
   F_ort_prev_beta.resetSizesAndBlocks(matOpts.size_block_info,
@@ -167,7 +171,8 @@ void SCF_unrestricted::get_starting_guess_density()
       noOfElectronsList[0] = noOfElectrons_alpha + scfopts.starting_guess_spin_diff;
       noOfElectronsList[1] = noOfElectrons_beta  - scfopts.starting_guess_spin_diff;
 
-      if(load_density_and_project_sparse(guessDmatFileName,
+      if(load_density_and_project_sparse(DensFromFock,
+					 guessDmatFileName,
 					 noOfDensityMatrices,
 					 &integralInfo,
 					 basisInfo,
@@ -176,20 +181,7 @@ void SCF_unrestricted::get_starting_guess_density()
 					 noOfElectronsList,
 					 matOpts.size_block_info,
 					 matOpts.permutationHML,
-					 matOpts.sparse_threshold,
-					 invCholFactor,
-					 invCholFactor_euclnorm,
-					 scfopts.gap_expected_lower_bound,
-					 scfopts.purification_eigvalue_err_limit * scfopts.puri_eig_acc_factor_for_guess,
-					 scfopts.purification_subspace_err_limit,
-					 scfopts.purification_truncation_norm,
-					 scfopts.purification_maxmul,
-					 scfopts.purification_create_m_files,
-					 scfopts.use_diagonalization,
-					 scfopts.use_diag_on_error_guess,
-					 scfopts.purification_ignore_failure,
-					 scfopts.purification_use_rand_perturbation_for_alleigsint,
-					 scfopts.electronic_temperature) != 0)
+					 matOpts.sparse_threshold) != 0)
 	{
 	  do_output(LOG_CAT_ERROR, LOG_AREA_SCF, "error in load_density_and_project_sparse");
 	  throw "error in load_density_and_project_sparse";
@@ -216,109 +208,69 @@ void SCF_unrestricted::get_starting_guess_density()
       else
 	{
 	  do_output(LOG_CAT_INFO, LOG_AREA_SCF, 
-		    "calling get_dens_from_fock_general for alpha and beta to diagonalize H_core for starting guesses, n = %i, matOpts.sparse_threshold = %g", 
+		    "calling get_dens_from_fock for alpha and beta to diagonalize H_core for starting guesses, n = %i, matOpts.sparse_threshold = %g", 
 		    n, (double)matOpts.sparse_threshold);
 
+
+
+	  // save flags
+	  int use_stochastic_orbitals = DensFromFock.get_use_stochastic_orbitals();
+	  DensFromFock.unset_use_stochastic_orbitals();
+
+	  int use_diag_on_error = DensFromFock.get_use_diag_on_error();
+
+	  if(DensFromFock.get_use_diag_on_error_guess() == 1)
+	    DensFromFock.set_use_diag_on_error();
+      
+
 	  // ALPHA
 	  symmMatrix F_ort_prev_dummy_alpha;
 	  F_ort_prev_dummy_alpha.resetSizesAndBlocks(matOpts.size_block_info,
 					   matOpts.size_block_info);
 	  F_ort_prev_dummy_alpha.writeToFile();
-	  intervalType homoInterval_dummy1_alpha(-1e22,1e22);
-	  intervalType lumoInterval_dummy1_alpha(-1e22,1e22);
-	  intervalType homoInterval_dummy2_alpha(-1e22,1e22);
-	  intervalType lumoInterval_dummy2_alpha(-1e22,1e22);
 	  densityMatrix_alpha.writeToFile();
-	  std::map<std::string, double> puri_stats_alpha;
-	  ergo_real electronicEntropyTermDummy = 0;
-	  std::vector< std::vector<ergo_real> > stochasticOrbsDummy;
-	  if(get_dens_from_fock_general(n,
-					noOfElectrons_alpha,
-					scfopts.use_diagonalization,
-					scfopts.use_diag_on_error_guess,
-					scfopts.electronic_temperature,
-					densityMatrix_alpha,
-					1,
-					electronicEntropyTermDummy,
-					H_core_Matrix, 
-					homoInterval_dummy1_alpha,
-					lumoInterval_dummy1_alpha,
-					S_symm,
-					invCholFactor,
-					invCholFactor_euclnorm,
-					scfopts.gap_expected_lower_bound,
-					matOpts.size_block_info,
-					F_ort_prev_dummy_alpha,
-					homoInterval_dummy2_alpha,
-					lumoInterval_dummy2_alpha,
-					scfopts.purification_eigvalue_err_limit,
-					scfopts.purification_subspace_err_limit,
-					scfopts.purification_truncation_norm,
-					scfopts.purification_maxmul,
-					scfopts.purification_create_m_files,
-					scfopts.purification_ignore_failure,
-					scfopts.purification_use_rand_perturbation_for_alleigsint,
-					0,
-					stochasticOrbsDummy,
-					"",
-					puri_stats_alpha,
-					scfopts.do_sparsity_investigation,
-					scfopts.sparsity_plots_resolution_m,
-					scfopts.do_comparison_to_simple_purification,
-					scfopts.do_puri_mmul_tests) != 0)
+
+	  DensFromFock.set_no_occupied_orbs(noOfElectrons_alpha);
+	  DensFromFock.clean_eigs_intervals();
+
+	  DensFromFock.set_generate_figures(0, "_alpha");
+
+  	  if(DensFromFock.get_dens_from_fock(H_core_Matrix,
+					     densityMatrix_alpha,
+					     F_ort_prev_dummy_alpha) != 0)
 	    {
-	      throw "SCF_unrestricted::get_starting_guess_density: Error in get_dens_from_fock_general for alpha.";
+	      throw "SCF_unrestricted::get_starting_guess_density: Error in get_dens_from_fock for alpha.";
 	    }
 
+	  
 	  // BETA
 	  symmMatrix F_ort_prev_dummy_beta;
 	  F_ort_prev_dummy_beta.resetSizesAndBlocks(matOpts.size_block_info,
 					   matOpts.size_block_info);
 	  F_ort_prev_dummy_beta.writeToFile();
-	  intervalType homoInterval_dummy1_beta(-1e22,1e22);
-	  intervalType lumoInterval_dummy1_beta(-1e22,1e22);
-	  intervalType homoInterval_dummy2_beta(-1e22,1e22);
-	  intervalType lumoInterval_dummy2_beta(-1e22,1e22);
 	  densityMatrix_beta.writeToFile();
-	  std::map<std::string, double> puri_stats_beta;
-	  if(get_dens_from_fock_general(n,
-					noOfElectrons_beta,
-					scfopts.use_diagonalization,
-					scfopts.use_diag_on_error_guess,
-					scfopts.electronic_temperature,
-					densityMatrix_beta,
-					1,
-					electronicEntropyTermDummy,
-					H_core_Matrix, 
-					homoInterval_dummy1_beta,
-					lumoInterval_dummy1_beta,
-					S_symm,
-					invCholFactor,
-					invCholFactor_euclnorm,
-					scfopts.gap_expected_lower_bound,
-					matOpts.size_block_info,
-					F_ort_prev_dummy_beta,
-					homoInterval_dummy2_beta,
-					lumoInterval_dummy2_beta,
-					scfopts.purification_eigvalue_err_limit,
-					scfopts.purification_subspace_err_limit,
-					scfopts.purification_truncation_norm,
-					scfopts.purification_maxmul,
-					scfopts.purification_create_m_files,
-					scfopts.purification_ignore_failure,
-					scfopts.purification_use_rand_perturbation_for_alleigsint,
-					0,
-					stochasticOrbsDummy,
-					"",
-					puri_stats_beta,
-					scfopts.do_sparsity_investigation,
-					scfopts.sparsity_plots_resolution_m,
-					scfopts.do_comparison_to_simple_purification,
-					scfopts.do_puri_mmul_tests) != 0)
+
+	  DensFromFock.set_no_occupied_orbs(noOfElectrons_beta);
+	  DensFromFock.clean_eigs_intervals();
+
+	  DensFromFock.set_generate_figures(0, "_beta");
+
+  	  if(DensFromFock.get_dens_from_fock(H_core_Matrix,
+					     densityMatrix_beta,
+					     F_ort_prev_dummy_beta) != 0)
 	    {
-	      throw "SCF_unrestricted::get_starting_guess_density: Error in get_dens_from_fock_general for beta.";
+	      throw "SCF_unrestricted::get_starting_guess_density: Error in get_dens_from_fock for beta.";
 	    }
 
+	  // return values of flags
+	  if( use_stochastic_orbitals == 1 )
+	    DensFromFock.set_use_stochastic_orbitals();
+
+	  if( use_diag_on_error != 1 )
+	    DensFromFock.unset_use_diag_on_error();
+	    
+	  
+
 	} // END ELSE use H_core to get starting guesses
     } // END ELSE no dmat given
 
@@ -809,94 +761,108 @@ void SCF_unrestricted::save_current_fock_as_fprev()
 
 void SCF_unrestricted::get_new_density_matrix()
 {
-  int n = basisInfo.noOfBasisFuncs;
-
   do_output(LOG_CAT_INFO, LOG_AREA_SCF, 
 	    "SCF_unrestricted::get_new_density_matrix, noOfElectrons_alpha noOfElectrons_beta = %5i %5i",
 	    noOfElectrons_alpha, noOfElectrons_beta);
 
   // ALPHA
-  std::map<std::string, double> puri_stats_alpha;
-  ergo_real electronicEntropyTerm_alpha = 0;
-  std::vector< std::vector<ergo_real> > stochasticOrbsDummy;
-  if(get_dens_from_fock_general(n,
-				noOfElectrons_alpha, 
-				scfopts.use_diagonalization,
-				scfopts.use_diag_on_error,
-				scfopts.electronic_temperature,
-				densityMatrix_alpha,
-				1,
-				electronicEntropyTerm_alpha,
-				FockMatrix_alpha, 
-				homoInterval_Fprev_alpha,
-				lumoInterval_Fprev_alpha,
-				S_symm,
-				invCholFactor,
-				invCholFactor_euclnorm,
-				scfopts.gap_expected_lower_bound,
-				matOpts.size_block_info,
-				F_ort_prev_alpha,
-				homoInterval_F_ort_prev_alpha,
-				lumoInterval_F_ort_prev_alpha,
-				scfopts.purification_eigvalue_err_limit,
-				scfopts.purification_subspace_err_limit,
-				scfopts.purification_truncation_norm,
-				scfopts.purification_maxmul,
-				scfopts.purification_create_m_files,
-				scfopts.purification_ignore_failure,
-				scfopts.purification_use_rand_perturbation_for_alleigsint,
-				0,
-				stochasticOrbsDummy,
-				"",
-				puri_stats_alpha,
-				scfopts.do_sparsity_investigation,
-				scfopts.sparsity_plots_resolution_m,
-				scfopts.do_comparison_to_simple_purification,
-				scfopts.do_puri_mmul_tests) != 0)
-    {
-      throw "SCF_unrestricted::get_new_density_matrix: Error in get_dens_from_fock_general for alpha.";
+  DensFromFock.set_no_occupied_orbs( noOfElectrons_alpha);
+  DensFromFock.set_eigs_F_ort_prev(homoInterval_F_ort_prev_alpha, lumoInterval_F_ort_prev_alpha);
+  DensFromFock.set_eigs2_F_ort_prev(homo2Interval_F_ort_prev_alpha, lumo2Interval_F_ort_prev_alpha);
+  DensFromFock.set_generate_figures(SCF_step, "_alpha");
+
+ // if eigenvectors are needed, set params
+  int use_init_guess = 0;
+  if(scfopts.use_prev_vector_as_initial_guess == 1 && 
+     scfopts.min_number_of_iterations <= SCF_step &&
+     SCF_step > 1 &&
+     !eigVecLUMO_alpha.is_empty() && !eigVecHOMO_alpha.is_empty()) // ensure that we computed vectors in previous cycle
+    use_init_guess = 1;
+
+// if we use new purification
+  if(SCF_step > 1 && 
+     DensFromFock.get_use_purification() == 1 &&
+     DensFromFock.get_output_homo_and_lumo_eigenvectors() == 1)
+    {   
+      DensFromFock.compute_eigenvectors(scfopts.eigenvectors_method, 
+					scfopts.eigenvectors_iterative_method, 
+					scfopts.number_of_eigenpairs, 
+					scfopts.eigensolver_accuracy, 
+					scfopts.eigensolver_maxiter,
+					use_init_guess,
+					scfopts.try_eigv_on_next_iteration_if_fail);
+      
+      // if we need more eigenpairs, initialize pointers in the DensFromFock class
+      if(scfopts.number_of_eigenpairs == 2)
+	DensFromFock.set_second_eigenvectors_pointers(&eigVecLUMO2_alpha, &eigVecHOMO2_alpha);
     }
 
-  // BETA
-  std::map<std::string, double> puri_stats_beta;
-  ergo_real electronicEntropyTerm_beta = 0;
-  if(get_dens_from_fock_general(n,
-				noOfElectrons_beta,
-				scfopts.use_diagonalization,
-				scfopts.use_diag_on_error,
-				scfopts.electronic_temperature,
-				densityMatrix_beta,
-				1,
-				electronicEntropyTerm_beta,
-				FockMatrix_beta, 
-				homoInterval_Fprev_beta,
-				lumoInterval_Fprev_beta,
-				S_symm,
-				invCholFactor,
-				invCholFactor_euclnorm,
-				scfopts.gap_expected_lower_bound,
-				matOpts.size_block_info,
-				F_ort_prev_beta,
-				homoInterval_F_ort_prev_beta,
-				lumoInterval_F_ort_prev_beta,
-				scfopts.purification_eigvalue_err_limit,
-				scfopts.purification_subspace_err_limit,
-				scfopts.purification_truncation_norm,
-				scfopts.purification_maxmul,
-				scfopts.purification_create_m_files,
-				scfopts.purification_ignore_failure,
-				scfopts.purification_use_rand_perturbation_for_alleigsint,
-				0,
-				stochasticOrbsDummy,
-				"",
-				puri_stats_beta,
-				scfopts.do_sparsity_investigation,
-				scfopts.sparsity_plots_resolution_m,
-				scfopts.do_comparison_to_simple_purification,
-				scfopts.do_puri_mmul_tests) != 0)
+  if(DensFromFock.get_dens_from_fock(FockMatrix_alpha,
+				     densityMatrix_alpha,
+				     F_ort_prev_alpha,
+				     &eigVecLUMO_alpha,
+			             &eigVecHOMO_alpha) != 0)
     {
-      throw "SCF_unrestricted::get_new_density_matrix: Error in get_dens_from_fock_general for beta.";
+      throw "SCF_unrestricted::get_new_density_matrix: Error in get_dens_from_fock for alpha";
     }
+	
+  DensFromFock.get_eigs_F_ort_prev(homoInterval_F_ort_prev_alpha, lumoInterval_F_ort_prev_alpha);
+  DensFromFock.get_eigs2_F_ort_prev(homo2Interval_F_ort_prev_alpha, lumo2Interval_F_ort_prev_alpha);
+  DensFromFock.get_eigs_Fprev(homoInterval_Fprev_alpha, lumoInterval_Fprev_alpha);
+
+  DensFromFock.unset_generate_figures();
+  ergo_real electronicEntropyTerm_alpha = DensFromFock.get_result_entropy_term();
+
+
+  // BETA
+  DensFromFock.set_no_occupied_orbs( noOfElectrons_beta);
+  DensFromFock.set_eigs_F_ort_prev(homoInterval_F_ort_prev_beta, lumoInterval_F_ort_prev_beta);
+  DensFromFock.set_eigs2_F_ort_prev(homo2Interval_F_ort_prev_beta, lumo2Interval_F_ort_prev_beta);
+  DensFromFock.set_generate_figures(SCF_step, "_beta");
+
+ // if eigenvectors are needed, set params
+  use_init_guess = 0;
+  if(scfopts.use_prev_vector_as_initial_guess == 1 && 
+     scfopts.min_number_of_iterations <= SCF_step &&
+     SCF_step > 1 &&
+     !eigVecLUMO_beta.is_empty() && !eigVecHOMO_beta.is_empty()) // ensure that we computed vectors in previous cycle
+    use_init_guess = 1;
+
+// if we use new purification
+  if(SCF_step > 1 && 
+     DensFromFock.get_use_purification() == 1  &&
+     DensFromFock.get_output_homo_and_lumo_eigenvectors() == 1)
+    {   
+      DensFromFock.compute_eigenvectors(scfopts.eigenvectors_method, 
+					scfopts.eigenvectors_iterative_method, 
+					scfopts.number_of_eigenpairs, 
+					scfopts.eigensolver_accuracy, 
+					scfopts.eigensolver_maxiter,
+					use_init_guess,
+					scfopts.try_eigv_on_next_iteration_if_fail);
+
+      // if we need more eigenpairs, initialize pointers in the DensFromFock class
+      if(scfopts.number_of_eigenpairs == 2)
+	DensFromFock.set_second_eigenvectors_pointers(&eigVecLUMO2_beta, &eigVecHOMO2_beta);
+    }
+
+
+  if(DensFromFock.get_dens_from_fock(FockMatrix_beta,
+				     densityMatrix_beta,
+				     F_ort_prev_beta,
+				     &eigVecLUMO_beta,
+			             &eigVecHOMO_beta) != 0)
+    {
+      throw "SCF_unrestricted::get_new_density_matrix: Error in get_dens_from_fock for beta";
+    }
+	
+  DensFromFock.get_eigs_F_ort_prev(homoInterval_F_ort_prev_beta, lumoInterval_F_ort_prev_beta);
+  DensFromFock.get_eigs2_F_ort_prev(homo2Interval_F_ort_prev_beta, lumo2Interval_F_ort_prev_beta);
+  DensFromFock.get_eigs_Fprev(homoInterval_Fprev_beta, lumoInterval_Fprev_beta);
+
+  DensFromFock.unset_generate_figures();
+  ergo_real electronicEntropyTerm_beta = DensFromFock.get_result_entropy_term();
+
 
   electronicEntropyTerm = electronicEntropyTerm_alpha + electronicEntropyTerm_beta;
 
@@ -985,6 +951,8 @@ void SCF_unrestricted::save_final_potential()
 }
 
 
+void SCF_unrestricted::output_density_images_orbital(generalVector &eigVec, const std::string &filename_id){}
+
 void SCF_unrestricted::output_density_images()
 {
   Util::TimeMeter timeMeter;
@@ -1247,7 +1215,7 @@ void SCF_unrestricted::compute_dipole_moment()
   densityMatrix_beta.readFromFile();
   symmMatrix densityMatrix_tot(densityMatrix_alpha);
   densityMatrix_tot += (ergo_real)1.0 * densityMatrix_beta;
-  get_dipole_moment(densityMatrix_tot, basisInfo, matOpts.size_block_info, matOpts.permutationHML, molecule);  
+  get_dipole_moment(densityMatrix_tot, basisInfo, matOpts.size_block_info, matOpts.permutationHML, molecule, LOG_AREA_SCF, "SCF");
   densityMatrix_alpha.writeToFile();
   densityMatrix_beta.writeToFile();
 }
@@ -1329,23 +1297,395 @@ void SCF_unrestricted::create_mtx_files_D(int const scfIter) {
   }  
 }
 
-void SCF_unrestricted::create_homo_eigvec_file() const
+// void SCF_unrestricted::create_homo_eigvec_file() const
+// {
+//   // ALPHA
+//   if (eigVecHOMO_alpha.is_empty()) {
+//     do_output(LOG_CAT_WARNING, LOG_AREA_SCF, "Failed to output HOMO (alpha) eigenvector to file. No HOMO (alpha) eigenvector stored.");
+//   }
+//   else{
+//     do_output(LOG_CAT_INFO, LOG_AREA_SCF, "Storing HOMO (alpha) eigenvector to file homo_coefficient_vec_alpha.txt.");
+//     int n = basisInfo.noOfBasisFuncs;
+//     std::vector<ergo_real> homo_vec_perm(n);
+//     eigVecHOMO_alpha.fullvector(homo_vec_perm); 
+//     // now we have the permuted vector
+//     std::vector<ergo_real> homo_vec(n);
+//     for (int ind = 0; ind < n; ind++) 
+//       homo_vec[ind] = homo_vec_perm[matOpts.inversePermutationHML[ind]];
+//     char ffname[888];
+//     sprintf(ffname, "homo_coefficient_vec_alpha.txt");
+//     std::ofstream ff(ffname);
+//     for (int ind = 0; ind < n; ind++) 
+//       ff << homo_vec[ind] << std::endl;
+//     ff.close();
+//   }
+
+
+//   // BETA
+//   if (eigVecHOMO_beta.is_empty()) {
+//     do_output(LOG_CAT_WARNING, LOG_AREA_SCF, "Failed to output HOMO (beta) eigenvector to file. No HOMO (beta) eigenvector stored.");
+//   }
+//   else{
+//     do_output(LOG_CAT_INFO, LOG_AREA_SCF, "Storing HOMO (beta) eigenvector to file homo_coefficient_vec_beta.txt.");
+//     int n = basisInfo.noOfBasisFuncs;
+//     std::vector<ergo_real> homo_vec_perm(n);
+//     eigVecHOMO_beta.fullvector(homo_vec_perm); 
+//     // now we have the permuted vector
+//     std::vector<ergo_real> homo_vec(n);
+//     for (int ind = 0; ind < n; ind++) 
+//       homo_vec[ind] = homo_vec_perm[matOpts.inversePermutationHML[ind]];
+//     char ffname[888];
+//     sprintf(ffname, "homo_coefficient_vec_beta.txt");
+//     std::ofstream ff(ffname);
+//     for (int ind = 0; ind < n; ind++) 
+//       ff << homo_vec[ind] << std::endl;
+//     ff.close();
+//   }
+// }
+
+
+
+// void SCF_unrestricted::create_lumo_eigvec_file() const
+// {
+//   if (eigVecLUMO_alpha.is_empty()) {
+//     do_output(LOG_CAT_WARNING, LOG_AREA_SCF, "Failed to output LUMO (alpha) eigenvector to file. No LUMO (alpha) eigenvector stored.");
+//   }
+//   else{
+//     do_output(LOG_CAT_INFO, LOG_AREA_SCF, "Storing LUMO (alpha) eigenvector to file lumo_coefficient_vec_alpha.txt.");
+//     int n = basisInfo.noOfBasisFuncs;
+//     std::vector<ergo_real> lumo_vec_perm(n);
+//     eigVecLUMO_alpha.fullvector(lumo_vec_perm); 
+//     // now we have the permuted vector
+//     std::vector<ergo_real> lumo_vec(n);
+//     for (int ind = 0; ind < n; ind++) 
+//       lumo_vec[ind] = lumo_vec_perm[matOpts.inversePermutationHML[ind]];
+//     char ffname[888];
+//     sprintf(ffname, "lumo_coefficient_vec_alpha.txt");
+//     std::ofstream ff(ffname);
+//     for (int ind = 0; ind < n; ind++) 
+//       ff << lumo_vec[ind] << std::endl;
+//     ff.close();
+//   } 
+
+
+//   if (eigVecLUMO_beta.is_empty()) {
+//     do_output(LOG_CAT_WARNING, LOG_AREA_SCF, "Failed to output LUMO (beta) eigenvector to file. No LUMO (beta) eigenvector stored.");
+//   }
+//   else{
+//     do_output(LOG_CAT_INFO, LOG_AREA_SCF, "Storing LUMO (beta) eigenvector to file lumo_coefficient_vec_beta.txt.");
+//     int n = basisInfo.noOfBasisFuncs;
+//     std::vector<ergo_real> lumo_vec_perm(n);
+//     eigVecLUMO_beta.fullvector(lumo_vec_perm); 
+//     // now we have the permuted vector
+//     std::vector<ergo_real> lumo_vec(n);
+//     for (int ind = 0; ind < n; ind++) 
+//       lumo_vec[ind] = lumo_vec_perm[matOpts.inversePermutationHML[ind]];
+//     char ffname[888];
+//     sprintf(ffname, "lumo_coefficient_vec_beta.txt");
+//     std::ofstream ff(ffname);
+//     for (int ind = 0; ind < n; ind++) 
+//       ff << lumo_vec[ind] << std::endl;
+//     ff.close(); 
+//   }
+
+// }
+
+
+void SCF_unrestricted::create_eigenvectors_files() const
 {
-  do_output(LOG_CAT_WARNING, LOG_AREA_SCF, "Output of HOMO eigenvector not implemented for unrestricted calculations.");
-  return;
-}
-void SCF_unrestricted::create_lumo_eigvec_file() const
+  create_eigvec_file(eigVecHOMO_alpha, 
+		     eigVecHOMO_beta, 
+		     "HOMO", 
+		     "homo_coefficient_vec");
+
+  create_eigvec_file(eigVecLUMO_alpha, 
+		     eigVecLUMO_beta, 
+		     "LUMO", 
+		     "lumo_coefficient_vec");
+
+  create_eigvec_file(eigVecHOMO2_alpha, 
+  		     eigVecHOMO2_beta, 
+  		     "HOMO2", 
+  		     "homo2_coefficient_vec");
+
+  create_eigvec_file(eigVecLUMO2_alpha, 
+  		     eigVecLUMO2_beta, 
+  		     "LUMO2", 
+  		     "lumo2_coefficient_vec");
+
+} 
+
+void SCF_unrestricted::create_eigvec_file(const generalVector &eigVec_alpha, 
+					  const generalVector &eigVec_beta, 
+					  const char *vector_name, 
+					  const char *filename_id) const
 {
-  do_output(LOG_CAT_WARNING, LOG_AREA_SCF, "Output of LUMO eigenvector not implemented for unrestricted calculations.");
-  return;
-}
-void SCF_unrestricted::create_gabedit_file() const
-{
-  do_output(LOG_CAT_WARNING, LOG_AREA_SCF, "Output of gabedit file not implemented for unrestricted calculations.");
-  return;
+  if (eigVec_alpha.is_empty()) {
+    do_output(LOG_CAT_WARNING, LOG_AREA_SCF, "Failed to output %s (alpha) eigenvector to file. "
+  	      "No %s (alpha) eigenvector stored.", vector_name, vector_name);
+  }
+  else{
+    do_output(LOG_CAT_INFO, LOG_AREA_SCF, "Storing %s (alpha) eigenvector to file "
+  	      "%s.", vector_name, filename_id);
+    int n = basisInfo.noOfBasisFuncs;
+    std::vector<ergo_real> vec_perm(n);
+    eigVec_alpha.fullvector(vec_perm); 
+    // now we have the permuted vector
+    std::vector<ergo_real> vec(n);
+    for (int ind = 0; ind < n; ind++) 
+      vec[ind] = vec_perm[matOpts.inversePermutationHML[ind]];
+    char ffname[888];
+    sprintf(ffname, "%s_alpha.txt", filename_id);
+    std::ofstream ff(ffname);
+    for (int ind = 0; ind < n; ind++) 
+      ff << vec[ind] << std::endl;
+    ff.close();
+  } 
+
+
+  if (eigVec_beta.is_empty()) {
+    do_output(LOG_CAT_WARNING, LOG_AREA_SCF, "Failed to output %s (beta) eigenvector to file. "
+	      "No %s (beta) eigenvector stored.", vector_name, vector_name);
+  }
+  else{
+    do_output(LOG_CAT_INFO, LOG_AREA_SCF, "Storing %s (beta) eigenvector to file "
+	      "%s.", vector_name, filename_id);
+    int n = basisInfo.noOfBasisFuncs;
+    std::vector<ergo_real> vec_perm(n);
+    eigVec_beta.fullvector(vec_perm); 
+    // now we have the permuted vector
+    std::vector<ergo_real> vec(n);
+    for (int ind = 0; ind < n; ind++) 
+      vec[ind] = vec_perm[matOpts.inversePermutationHML[ind]];
+    char ffname[888];
+    sprintf(ffname, "%s_beta.txt", filename_id);
+    std::ofstream ff(ffname);
+    for (int ind = 0; ind < n; ind++) 
+      ff << vec[ind] << std::endl;
+    ff.close(); 
+  }
+
 }
 
 
+
+
+static void 
+output_orbital_coeffs_in_gabedit_order(const BasisInfoStruct & basisInfo,
+				       std::vector<int> const & shellIdxList,
+				       std::ofstream & ff,
+				       std::vector<ergo_real> const & orbital_vec) {
+  int count = 0;
+  for(int i = 0; i < basisInfo.noOfShells; i++) {
+    int k = shellIdxList[i];
+    int startIdx = basisInfo.shellList[k].startIndexInMatrix;
+    switch(basisInfo.shellList[k].shellType) {
+    case 0: // s-type shell
+      ff << count+1 << "   " << orbital_vec[startIdx] << std::endl; count++;
+      break;
+    case 1: // p-type shell
+      ff << count+1 << "   " << orbital_vec[startIdx+2] << std::endl; count++;
+      ff << count+1 << "   " << orbital_vec[startIdx+0] << std::endl; count++;
+      ff << count+1 << "   " << orbital_vec[startIdx+1] << std::endl; count++;
+      break;
+    case 2: // d-type shell
+      ff << count+1 << "   " << orbital_vec[startIdx+2] << std::endl; count++;
+      ff << count+1 << "   " << orbital_vec[startIdx+3] << std::endl; count++;
+      ff << count+1 << "   " << orbital_vec[startIdx+1] << std::endl; count++;
+      ff << count+1 << "   " << orbital_vec[startIdx+4] << std::endl; count++;
+      ff << count+1 << "   " << orbital_vec[startIdx+0] << std::endl; count++;
+      break;
+    case 3: // f-type shell
+      ff << count+1 << "   " << orbital_vec[startIdx+3] << std::endl; count++;
+      ff << count+1 << "   " << orbital_vec[startIdx+4] << std::endl; count++;
+      ff << count+1 << "   " << orbital_vec[startIdx+2] << std::endl; count++;
+      ff << count+1 << "   " << orbital_vec[startIdx+5] << std::endl; count++;
+      ff << count+1 << "   " << orbital_vec[startIdx+1] << std::endl; count++;
+      ff << count+1 << "   " << orbital_vec[startIdx+6] << std::endl; count++;
+      ff << count+1 << "   " << orbital_vec[startIdx+0] << std::endl; count++;
+      break;
+    case 4: // g-type shell
+      ff << count+1 << "   " << orbital_vec[startIdx+4] << std::endl; count++;
+      ff << count+1 << "   " << orbital_vec[startIdx+5] << std::endl; count++;
+      ff << count+1 << "   " << orbital_vec[startIdx+3] << std::endl; count++;
+      ff << count+1 << "   " << orbital_vec[startIdx+6] << std::endl; count++;
+      ff << count+1 << "   " << orbital_vec[startIdx+2] << std::endl; count++;
+      ff << count+1 << "   " << orbital_vec[startIdx+7] << std::endl; count++;
+      ff << count+1 << "   " << orbital_vec[startIdx+1] << std::endl; count++;
+      ff << count+1 << "   " << orbital_vec[startIdx+8] << std::endl; count++;
+      ff << count+1 << "   " << orbital_vec[startIdx+0] << std::endl; count++;
+      break;
+    default: throw "error in output_orbital_coeffs_in_gabedit_order: shell types beyond g not implemented!";
+    }
+  }
+  if(count != basisInfo.noOfBasisFuncs)
+    throw "error in output_orbital_coeffs_in_gabedit_order: (count != basisInfo.noOfBasisFuncs)";
+}
+
+
+
+void SCF_unrestricted::create_gabedit_file() const
+{
+  if (eigVecHOMO_alpha.is_empty() || eigVecLUMO_alpha.is_empty() ||
+      eigVecHOMO_beta.is_empty() || eigVecLUMO_beta.is_empty()) {
+    do_output(LOG_CAT_WARNING, LOG_AREA_SCF, "Failed to output HOMO/LUMO eigenvectors to gabedit file; no HOMO/LUMO info available.");
+    return;
+  }
+  if(basisInfo.use_6_d_funcs == 1) {
+    do_output(LOG_CAT_WARNING, LOG_AREA_SCF, "Failed to output HOMO/LUMO eigenvectors to gabedit file; not implemented for use_6_d_funcs case.");
+    return;
+  }
+
+  do_output(LOG_CAT_INFO, LOG_AREA_SCF, "Creating Gabedit file with HOMO/LUMO eigenvector info.");
+  int n = basisInfo.noOfBasisFuncs;
+
+  // ALPHA
+  // Get HOMO
+  // now we have the permuted vector
+  std::vector<ergo_real> homo_vec_alpha(n);
+  { 
+    std::vector<ergo_real> homo_vec_perm(n);
+    eigVecHOMO_alpha.fullvector(homo_vec_perm); 
+    for (int ind = 0; ind < n; ind++) 
+      homo_vec_alpha[ind] = homo_vec_perm[matOpts.permutationHML[ind]];
+    //homo_vec_alpha[ind] = homo_vec_perm[matOpts.inversePermutationHML[ind]];
+  }
+
+
+  // Get LUMO
+  // now we have the permuted vector
+  std::vector<ergo_real> lumo_vec_alpha(n);
+  {
+    std::vector<ergo_real> lumo_vec_perm(n);
+    eigVecLUMO_alpha.fullvector(lumo_vec_perm); 
+    for (int ind = 0; ind < n; ind++) 
+      lumo_vec_alpha[ind] = lumo_vec_perm[matOpts.permutationHML[ind]];
+    //lumo_vec_alpha[ind] = lumo_vec_perm[matOpts.inversePermutationHML[ind]];
+  }
+
+  // BETA
+  // Get HOMO
+  // now we have the permuted vector
+  std::vector<ergo_real> homo_vec_beta(n);
+  { 
+    std::vector<ergo_real> homo_vec_perm(n);
+    eigVecHOMO_beta.fullvector(homo_vec_perm); 
+    for (int ind = 0; ind < n; ind++) 
+      homo_vec_beta[ind] = homo_vec_perm[matOpts.permutationHML[ind]];
+    //homo_vec_beta[ind] = homo_vec_perm[matOpts.inversePermutationHML[ind]];
+  }
+
+
+  // Get LUMO
+  // now we have the permuted vector
+  std::vector<ergo_real> lumo_vec_beta(n);
+  {
+    std::vector<ergo_real> lumo_vec_perm(n);
+    eigVecLUMO_beta.fullvector(lumo_vec_perm); 
+    for (int ind = 0; ind < n; ind++) 
+      lumo_vec_beta[ind] = lumo_vec_perm[matOpts.permutationHML[ind]];
+    //lumo_vec_beta[ind] = lumo_vec_perm[matOpts.inversePermutationHML[ind]];
+  }
+
+  // Create Gabedit file.
+  const char fileName [] = "gabeditfile.gab";
+  std::ofstream ff(fileName);
+  /* FIXME: check if we should use "Cart" or "Sphe" here. That is,
+     should we use use_6_d_funcs? */
+  int use_6_d_funcs = 0;
+  ff << "[Gabedit Format] Sphe" << std::endl;
+  ff << "[Atoms] Angs" << std::endl;
+  for(int i = 0; i < molecule.getNoOfAtoms(); i++) {
+    char atomLabelString[4];
+    get_atom_label_from_charge_int(molecule.getAtom(i).charge, atomLabelString, 4);
+    ff << atomLabelString << " " << i+1 << " " << molecule.getAtom(i).charge
+       << "   " << molecule.getAtom(i).coords[0] / UNIT_one_Angstrom
+       << "   " << molecule.getAtom(i).coords[1] / UNIT_one_Angstrom
+       << "   " << molecule.getAtom(i).coords[2] / UNIT_one_Angstrom 
+       << std::endl;
+  }
+
+  std::vector<int> shellIdxList(basisInfo.noOfShells);
+  int shellIdxCounter = 0;
+  SquareFuncIntegrator sfi;
+  ff << "[Basis]" << std::endl;
+  for(int i = 0; i < molecule.getNoOfAtoms(); i++) {
+    ff << i+1 << " 0" << std::endl;
+    // Now output info about shells for this atom.
+    for(int k = 0; k < basisInfo.noOfShells; k++) {
+      // Check if this shell belongs to the current atom.
+      ergo_real absdx = std::abs(basisInfo.shellList[k].centerCoords[0] - molecule.getAtom(i).coords[0]);
+      ergo_real absdy = std::abs(basisInfo.shellList[k].centerCoords[1] - molecule.getAtom(i).coords[1]);
+      ergo_real absdz = std::abs(basisInfo.shellList[k].centerCoords[2] - molecule.getAtom(i).coords[2]);
+      ergo_real distlimit = 0.01;
+      if(absdx > distlimit || absdy > distlimit || absdz > distlimit)
+	continue;
+      // OK, now we know this shell is at least very near the current atom.
+      shellIdxList[shellIdxCounter] = k;
+      shellIdxCounter++;
+      char shellChar = 'x';
+      int shellType = basisInfo.shellList[k].shellType;
+      switch(shellType) {
+      case 0: shellChar = 's'; break;
+      case 1: shellChar = 'p'; break;
+      case 2: shellChar = 'd'; break;
+      case 3: shellChar = 'f'; break;
+      case 4: shellChar = 'g'; break;
+      default: throw "SCF_restricted::create_gabedit_file error: shell types beyond g not implemented!";
+      }
+      ff << shellChar << " " << basisInfo.shellList[k].noOfContr << " 1.00" << std::endl;
+      for(int contridx = 0; contridx < basisInfo.shellList[k].noOfContr; contridx++) {
+	ergo_real exponent = basisInfo.shellList[k].exponentList[contridx];
+	ergo_real shellFactor = sfi.getShellFactor(integralInfo, exponent, shellType, use_6_d_funcs);
+	ergo_real coeff = basisInfo.shellList[k].coeffList[contridx] / shellFactor;
+	ff << exponent << "   " << coeff << std::endl;
+      }
+    }
+    // Blank line before shells for next atom.
+    ff << std::endl;
+  }
+
+  if(shellIdxCounter != basisInfo.noOfShells)
+    throw "Error: (shellIdxCounter != basisInfo.noOfShells)";
+
+
+  // MO section.
+  ff << "[MO]" << std::endl;
+
+  // ALPHA
+  // HOMO
+  ff << "Spin=Alpha" << std::endl;
+  ff << "Occup=   1.000000" << std::endl;
+  output_orbital_coeffs_in_gabedit_order(basisInfo, shellIdxList, ff, homo_vec_alpha);
+  // LUMO
+  ff << "Spin=Alpha" << std::endl;
+  ff << "Occup=   0.000000" << std::endl;
+  output_orbital_coeffs_in_gabedit_order(basisInfo, shellIdxList, ff, lumo_vec_alpha);
+
+  // BETA
+  // HOMO
+  ff << "Spin=Beta" << std::endl;
+  ff << "Occup=   1.000000" << std::endl;
+  output_orbital_coeffs_in_gabedit_order(basisInfo, shellIdxList, ff, homo_vec_beta);
+  // LUMO
+  ff << "Spin=Beta" << std::endl;
+  ff << "Occup=   0.000000" << std::endl;
+  output_orbital_coeffs_in_gabedit_order(basisInfo, shellIdxList, ff, lumo_vec_beta);
+
+
+  // Blank line before end of file.
+  ff << std::endl;
+  // Close file.
+  ff.close();
+
+  do_output(LOG_CAT_INFO, LOG_AREA_SCF, "Gabedit file '%s' with HOMO/LUMO eigenvector info created OK.", fileName);
+}
+
+void SCF_unrestricted::create_gabedit_file_2() const
+{
+  throw "Error: SCF_unrestricted::create_gabedit_file_2() not implemented";
+}
+
 void SCF_unrestricted::update_subspace_diff()
 {
   throw "Error: SCF_unrestricted::update_subspace_diff() not implemented";
