@@ -1,6 +1,7 @@
-/* Ergo, version 3.4, a program for linear scaling electronic structure
+/* Ergo, version 3.5, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2014 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -78,6 +79,7 @@ class SCF_unrestricted : public SCF_general
   void save_final_potential();
   void add_random_disturbance_to_starting_guess();
   void output_density_images();
+  void output_density_images_orbital(generalVector &eigVec, const std::string &filename_id);
   void prepare_stochastic_orbitals();
   void output_csr_matrices_for_gao();
   void write_diag_dens_to_file();
@@ -92,9 +94,15 @@ class SCF_unrestricted : public SCF_general
   void report_density_difference();
   void create_mtx_files_F(int const scfIter);
   void create_mtx_files_D(int const scfIter);
-  void create_homo_eigvec_file() const;
-  void create_lumo_eigvec_file() const;
+  /* void create_homo_eigvec_file() const; */
+  /* void create_lumo_eigvec_file() const; */
+  void create_eigenvectors_files() const;
+  void create_eigvec_file(const generalVector &eigVec_alpha, 
+			  const generalVector &eigVec_beta,
+			  const char *vector_name, 
+			  const char *filename_id) const;
   void create_gabedit_file() const;
+  void create_gabedit_file_2() const;
   void compute_dipole_moment();
   void do_mulliken_pop_stuff();
   void compute_gradient_fixeddens();
@@ -111,6 +119,8 @@ class SCF_unrestricted : public SCF_general
   symmMatrix Dprev_beta;
   symmMatrix F_ort_prev_alpha; // Used by purification
   symmMatrix F_ort_prev_beta; // Used by purification
+  symmMatrix D_ort_prev_alpha; // Used for computing eigenvectors
+  symmMatrix D_ort_prev_beta; // Used for computing eigenvectors
   symmMatrix bestFockMatrixSoFar_alpha;
   symmMatrix bestFockMatrixSoFar_beta;
   symmMatrix bestFockMatrixSoFar2_alpha;
@@ -125,6 +135,10 @@ class SCF_unrestricted : public SCF_general
   intervalType lumoInterval_F_ort_prev_alpha;
   intervalType homoInterval_F_ort_prev_beta;
   intervalType lumoInterval_F_ort_prev_beta;
+  intervalType homo2Interval_F_ort_prev_alpha;
+  intervalType lumo2Interval_F_ort_prev_alpha;
+  intervalType homo2Interval_F_ort_prev_beta;
+  intervalType lumo2Interval_F_ort_prev_beta;
   intervalType homoInterval_Fprev_alpha;
   intervalType lumoInterval_Fprev_alpha;
   intervalType homoInterval_Fprev_beta;
@@ -133,6 +147,15 @@ class SCF_unrestricted : public SCF_general
   int alpha_beta_diff;
   int noOfElectrons_alpha;
   int noOfElectrons_beta;
+
+  generalVector eigVecLUMO_alpha;
+  generalVector eigVecHOMO_alpha;
+  generalVector eigVecLUMO_beta;
+  generalVector eigVecHOMO_beta;
+  generalVector eigVecLUMO2_alpha;
+  generalVector eigVecHOMO2_alpha;
+  generalVector eigVecLUMO2_beta;
+  generalVector eigVecHOMO2_beta;
 };
 
 
