@@ -1,6 +1,7 @@
-/* Ergo, version 3.4, a program for linear scaling electronic structure
+/* Ergo, version 3.5, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2014 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -2327,54 +2328,6 @@ determine_number_of_electrons_unrestricted(int noOfElectrons,
 }
 
 
-
-static ergo_real
-compute_dipole_moment_onecoord(const symmMatrix & densityMatrix,
-			       const BasisInfoStruct & basisInfo,
-			       mat::SizesAndBlocks const & matrix_size_block_info,
-			       std::vector<int> const & permutationHML,
-			       const Molecule& molecule,
-			       int coordIdx)
-{
-  int ix = 0;
-  int iy = 0;
-  int iz = 0;
-  switch(coordIdx) {
-  case 0: ix = 1; break;
-  case 1: iy = 1; break;
-  case 2: iz = 1; break;
-  default: throw "Error in compute_dipole_moment_onecoord.";
-  }
-  symmMatrix opMatrix;
-  opMatrix.resetSizesAndBlocks(matrix_size_block_info, matrix_size_block_info);
-  if(compute_operator_matrix_sparse_symm(basisInfo, ix, iy, iz, opMatrix, permutationHML) != 0)
-    throw "Error in compute_operator_matrix_sparse_symm";
-  ergo_real density_contrib = symmMatrix::trace_ab(densityMatrix, opMatrix);
-  // Now compute contribution from nuclei.
-  ergo_real nuclear_contrib = 0;
-  for(int i = 0; i < molecule.getNoOfAtoms(); i++)
-    nuclear_contrib += molecule.getAtom(i).charge * molecule.getAtom(i).coords[coordIdx];
-  ergo_real dipole_moment_oneCoord = nuclear_contrib - density_contrib;
-  return dipole_moment_oneCoord;
-}
-  
-void 
-get_dipole_moment(const symmMatrix & densityMatrix,
-		  const BasisInfoStruct & basisInfo,
-		  mat::SizesAndBlocks const & matrix_size_block_info,
-		  std::vector<int> const & permutationHML,
-		  const Molecule& molecule)
-{
-  ergo_real dipole_moment_x = compute_dipole_moment_onecoord(densityMatrix, basisInfo, matrix_size_block_info, permutationHML, molecule, 0);
-  ergo_real dipole_moment_y = compute_dipole_moment_onecoord(densityMatrix, basisInfo, matrix_size_block_info, permutationHML, molecule, 1);
-  ergo_real dipole_moment_z = compute_dipole_moment_onecoord(densityMatrix, basisInfo, matrix_size_block_info, permutationHML, molecule, 2);
-  do_output(LOG_CAT_INFO, LOG_AREA_SCF, "dipole_moment_x [atomic units] = %15.7f", (double)dipole_moment_x);
-  do_output(LOG_CAT_INFO, LOG_AREA_SCF, "dipole_moment_y [atomic units] = %15.7f", (double)dipole_moment_y);
-  do_output(LOG_CAT_INFO, LOG_AREA_SCF, "dipole_moment_z [atomic units] = %15.7f", (double)dipole_moment_z);
-}
-
-
-
 static void
 get_mulliken_charges(const symmMatrix & densityMatrix,
 		     const symmMatrix & S_symm,
@@ -2508,12 +2461,122 @@ static int write_gcube_file_header(FILE* f,
   return 0;
 }
 
+
+
+
+
+void get_exp_value_pos_operator(const BasisInfoStruct & basisInfo,
+				const Molecule& molecule,
+				const ergo_real* densityMatrixFull, 
+				std::vector<ergo_real> &mean,
+				std::vector<ergo_real> &std)
+{
+  
+  int nPrims1 = 0;
+  int nPrims2 = 0;
+  
+  DistributionSpecStruct* rho1 = NULL;
+  ergo_real cutoff = 1e-7;
+
+  do_output(LOG_CAT_INFO, LOG_AREA_SCF, "get_exp_value_pos_operator, calling get_density() with cutoff = %8.4g.", cutoff);
+
+  nPrims1 = get_no_of_primitives_for_density(cutoff, densityMatrixFull, basisInfo);
+  rho1 = new DistributionSpecStruct[nPrims1];
+
+  Util::TimeMeter timeMeterGetDens;
+  int n1 = get_density(basisInfo,
+		       densityMatrixFull,
+		       cutoff,
+		       nPrims1,
+		       rho1);    
+  if(n1 < 0)
+    {
+      do_output(LOG_CAT_ERROR, LOG_AREA_SCF, "aborting get_exp_value_pos_operator(): (n1 < 0)");
+      throw "aborting get_exp_value_pos_operator(): (n1 < 0)";
+    }
+
+  nPrims1 = n1;
+  timeMeterGetDens.print(LOG_AREA_SCF, "get_density() calls");
+
+  do_output(LOG_CAT_INFO, LOG_AREA_SCF, "get_exp_value_pos_operator, after get_density() calls: nPrims1 = %d, nPrims2 = %d.", nPrims1, nPrims2);
+
+  // get bounds for the integral
+  ergo_real minlist[3];
+  ergo_real maxlist[3];
+  int coordno;
+  for(coordno = 0; coordno < 3; coordno++)
+    {
+      minlist[coordno] = std::numeric_limits<ergo_real>::max();
+      maxlist[coordno] = std::numeric_limits<ergo_real>::min(); 
+    }
+  for(int i = 0; i < molecule.getNoOfAtoms(); i++)
+    {
+      for(coordno = 0; coordno < 3; coordno++)
+	{
+	  ergo_real curr = molecule.getAtom(i).coords[coordno];
+	  if(curr < minlist[coordno])
+	    minlist[coordno] = curr;
+	  if(curr > maxlist[coordno])
+	    maxlist[coordno] = curr;
+	}
+    }
+
+
+  // OK, now we have min max coords for molecule.
+  // Add some margin.
+  ergo_real margin, length;
+  for(coordno = 0; coordno < 3; coordno++)
+    {
+      length = maxlist[coordno] - minlist[coordno];
+      margin = (length+1)*10; //  sizes of molecule
+      minlist[coordno] -= margin;
+      maxlist[coordno] += margin;
+    }
+
+  Util::TimeMeter timeMeterGetInt;
+
+  mean.clear(); mean.resize(3);
+  std.clear(); std.resize(3);
+  std::vector<int> monomialIntsAdd;
+  ergo_real tmp;
+
+
+  for(coordno = 0; coordno < 3; coordno++)
+    {
+      monomialIntsAdd.clear(); monomialIntsAdd.resize(3);
+      monomialIntsAdd[coordno] = 1;
+      mean[coordno] = integrate_density_in_box_2(nPrims1,
+					rho1,
+					minlist, 
+					maxlist,
+					monomialIntsAdd);
+
+      monomialIntsAdd[coordno] = 2;
+      // get mean value of x^2
+      tmp = integrate_density_in_box_2(nPrims1,
+					rho1,
+					minlist, 
+					maxlist,
+					monomialIntsAdd);
+      std[coordno] = std::sqrt(tmp - mean[coordno]*mean[coordno]);
+    }
+
+
+  timeMeterGetInt.print(LOG_AREA_SCF, "integrate_density_in_box_2() calls");
+
+}
+
+
+
+
+
 void 
 do_density_images(const BasisInfoStruct & basisInfo,
 		  const Molecule& molecule,
 		  const ergo_real* densityMatrixFull_tot, 
 		  const ergo_real* densityMatrixFull_spin,
-		  double output_density_images_boxwidth)
+		  double output_density_images_boxwidth,
+		  const std::string &filename_id)
 {
 
   int nPrims1 = 0;
@@ -2521,7 +2584,7 @@ do_density_images(const BasisInfoStruct & basisInfo,
   
   DistributionSpecStruct* rho1 = NULL;
   DistributionSpecStruct* rho2 = NULL;
-  ergo_real cutoff = 1e-7;
+  ergo_real cutoff = 1e-7;//1e-7;
 
   do_output(LOG_CAT_INFO, LOG_AREA_SCF, "do_density_images, calling get_density() with cutoff = %8.4g.", cutoff);
 
@@ -2558,8 +2621,8 @@ do_density_images(const BasisInfoStruct & basisInfo,
   int coordno;
   for(coordno = 0; coordno < 3; coordno++)
     {
-      minlist[coordno] =  888888888;
-      maxlist[coordno] = -888888888;
+      minlist[coordno] = std::numeric_limits<ergo_real>::max();
+      maxlist[coordno] = std::numeric_limits<ergo_real>::min(); 
     }
   for(int i = 0; i < molecule.getNoOfAtoms(); i++)
     {
@@ -2600,8 +2663,12 @@ do_density_images(const BasisInfoStruct & basisInfo,
   do_output(LOG_CAT_INFO, LOG_AREA_SCF, "do_density_images, preparing to write Gabedit gcube files.");
 
   // Create Gabedit "gcube" density files.
-  FILE* f1 = fopen("density.gcube", "wt");
-  FILE* f2 = fopen("spindensity.gcube", "wt");
+  std::ostringstream name_f1, name_f2;
+  name_f1 << "density" << filename_id << ".gcube";
+  name_f2 << "spindensity" << filename_id << ".gcube";
+
+  FILE* f1 = fopen(name_f1.str().c_str(), "wt");
+  FILE* f2 = fopen(name_f2.str().c_str(), "wt");
   if(f1 == NULL || f2 == NULL)
     {
       do_output(LOG_CAT_ERROR, LOG_AREA_SCF, "aborting do_density_images: (f1 == NULL || f2 == NULL)");
@@ -2732,3 +2799,65 @@ get_hf_weight_and_cam_params(int use_dft,
 }
 
 
+void
+create_mtx_files_with_different_orderings(const symmMatrix & A,
+					  const std::string & calculation_identifier,
+					  const std::string & method_and_basis_set,
+					  const std::vector<int> & inversePermutationHML,
+					  const BasisInfoStruct & basisInfo) {
+  // Original ordering
+  std::stringstream ss_id_1;
+  ss_id_1 << calculation_identifier << " - matrix (original ordering)";
+  write_matrix_in_matrix_market_format( A, inversePermutationHML, "S_matrix_original", ss_id_1.str(), method_and_basis_set );
+  // HML ordering
+  int n = basisInfo.noOfBasisFuncs;
+  std::vector<int> permutationNone(n);
+  for(int i = 0; i < n; i++)
+    permutationNone[i] = i;
+  std::stringstream ss_id_2;
+  ss_id_2 << calculation_identifier << " - matrix (HML ordering)";
+  write_matrix_in_matrix_market_format( A, permutationNone, "S_matrix_HML", ss_id_2.str(), method_and_basis_set );
+  // Factor2 ordering
+  std::vector<int> permutationFactor2, inversePermutationFactor2;
+  std::vector<ergo_real> xcoords(n);
+  std::vector<ergo_real> ycoords(n);
+  std::vector<ergo_real> zcoords(n);
+  for(int i = 0; i < n; i++) {
+    xcoords[i] = basisInfo.basisFuncList[i].centerCoords.v[0];
+    ycoords[i] = basisInfo.basisFuncList[i].centerCoords.v[1];
+    zcoords[i] = basisInfo.basisFuncList[i].centerCoords.v[2];
+  }
+  getMatrixPermutationOnlyFactor2(xcoords, ycoords, zcoords, 1, 1, permutationFactor2, inversePermutationFactor2);
+  std::vector<int> perm(n);
+  for(int i = 0; i < n; i++)
+    perm[i] = permutationFactor2[ inversePermutationHML[i] ];
+  std::stringstream ss_id_3;
+  ss_id_3 << calculation_identifier << " - matrix (Factor2 ordering)";
+  write_matrix_in_matrix_market_format( A, perm, "S_matrix_Factor2", ss_id_3.str(), method_and_basis_set );
+  // Random ordering
+  std::vector<int> permRandom(n);
+  std::vector<bool> taken(n);
+  for(int i = 0; i < n; i++)
+    taken[i] = false;
+  for(int i = 0; i < n; i++) {
+    int randNumber = rand() % (n-i);
+    int idx = randNumber;
+    for(int k = 0; k < n; k++) {
+      if(taken[k] && k <= idx)
+	idx++;
+    }
+    assert(idx >= 0);
+    assert(idx < n);
+    assert(taken[idx] == false);
+    assert(idx >= 0 && idx < n && taken[idx] == false);
+    taken[idx] = true;
+    permRandom[i] = idx;
+  }
+  for(int i = 0; i < n; i++)
+    assert(taken[i] == true);
+  std::stringstream ss_id_4;
+  ss_id_4 << calculation_identifier << " - matrix (random ordering)";
+  write_matrix_in_matrix_market_format( A, permRandom, "S_matrix_random", ss_id_4.str(), method_and_basis_set );
+}
+
+
