@@ -1,6 +1,7 @@
-/* Ergo, version 3.4, a program for linear scaling electronic structure
+/* Ergo, version 3.5, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2014 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -40,6 +41,16 @@
 #include "dft_common.h"
 #include "xc_matrix.h"
 
+static bool value_seems_like_nan_or_inf(ergo_real x) {
+  bool ok1 = false;
+  if(x > -std::numeric_limits<ergo_real>::max())
+    ok1 = true;
+  bool ok2 = false;
+  if(x < std::numeric_limits<ergo_real>::max())
+    ok2 = true;
+  return ( ! (ok1 && ok2) );
+}
+
 static bool
 compare_matrices(char mat_name,
                  const real *computed, const long double *ref, int sz,
@@ -50,13 +61,8 @@ compare_matrices(char mat_name,
   for(int row=0; row<sz; row++) {
     for(int col=0; col<sz; col++) {
       ergo_real theDiff = computed[row + col*sz]- ref[row+col*sz];
-      bool ok1 = false;
-      if(theDiff > -std::numeric_limits<ergo_real>::max())
-        ok1 = true;
-      bool ok2 = false;
-      if(theDiff < std::numeric_limits<ergo_real>::max())
-        ok2 = true;
-      if( ! (ok1 && ok2) ) {
+      bool nanOrInfFlag = value_seems_like_nan_or_inf(theDiff);
+      if(nanOrInfFlag) {
         printf("Error! nan/inf found in compare_matrices().\n");
         failed = true;
       }
@@ -81,6 +87,35 @@ test_small(const IntegralInfo& ii, const char *functional,
 	   const int *charges, const real (*coords)[3],
            const long double (*XCRef)[2])
 {
+  // Skip tests for some functionals for single precision because they cannot handle it due to overflow or similar problems.
+#ifdef PRECISION_SINGLE
+  std::vector<std::string> functionalsToSkip;
+  functionalsToSkip.push_back("PW91X");
+  functionalsToSkip.push_back("B3LYP");
+  functionalsToSkip.push_back("B3LYP-G");
+  functionalsToSkip.push_back("B3P86");
+  functionalsToSkip.push_back("B3P86-G");
+  functionalsToSkip.push_back("B3PW91");
+  functionalsToSkip.push_back("BLYP");
+  functionalsToSkip.push_back("BP86");
+  functionalsToSkip.push_back("BPW91");
+  functionalsToSkip.push_back("Camb3lyp");
+  functionalsToSkip.push_back("HSE");
+  functionalsToSkip.push_back("KT3");
+  functionalsToSkip.push_back("OLYP");
+  functionalsToSkip.push_back("PBE0");
+  functionalsToSkip.push_back("PBE");
+  bool skip_for_single_precision = false;
+  for(unsigned int i = 0; i < functionalsToSkip.size(); i++) {
+    if(strcmp(functional, functionalsToSkip[i].c_str()) == 0)
+      skip_for_single_precision = true;
+  }
+  if(skip_for_single_precision) {
+    printf("test_small(): skipping functional '%s' for PRECISION_SINGLE case.\n", functional);
+    return 0;
+  }
+#endif
+
   BasisInfoStruct* bis = new BasisInfoStruct();
   Molecule m;
   /* The code later will change the order of atoms, this is why the
@@ -123,6 +158,10 @@ test_small(const IntegralInfo& ii, const char *functional,
   char mode;
   ergo_real dftEnergy = 0;
   dft_get_xc_mt(noOfElectrons, dmat, bis, &m, gridParams, xcmat, &dftEnergy);
+  if(value_seems_like_nan_or_inf(dftEnergy)) {
+    printf("Error: dftEnergy from dft_get_xc_mt() is nan/inf for functional '%s'.\n", functional);
+    return 1;
+  }
   /* We give some room to accumulation error. */
   static const ergo_real EPS = 0.3;
   int nrepeat = 2;
@@ -134,6 +173,10 @@ test_small(const IntegralInfo& ii, const char *functional,
       memset(xcmat, 0, n*n*sizeof(ergo_real));
       electronsR = dft_get_xc_mt(noOfElectrons, dmat, bis, &m, gridParams,
                                  xcmat, &dftEnergyAgain);
+      if(value_seems_like_nan_or_inf(dftEnergyAgain)) {
+	printf("Error: dftEnergyAgain from dft_get_xc_mt() is nan/inf for functional '%s'.\n", functional);
+	return 1;
+      }
       failed = compare_matrices('R', xcmat, &XCRef[0][0], n, EPS);
       if(std::fabs(dftEnergyAgain - dftEnergy) > EPS)
 	{
@@ -154,6 +197,10 @@ test_small(const IntegralInfo& ii, const char *functional,
                                   dmata, dmata,
                                   bis, &m, gridParams,
                                   xca, xcb, &dftEnergyU);
+      if(value_seems_like_nan_or_inf(dftEnergyU)) {
+	printf("Error: dftEnergyU from dft_get_uxc_mt() is nan/inf for functional '%s'.\n", functional);
+	return 1;
+      }
       failed = compare_matrices('A', xca, &XCRef[0][0], n, EPS)
         || compare_matrices('B', xcb, &XCRef[0][0], n, EPS);
       if (std::fabs(electronsU - electronsR) > EPS) {
@@ -253,5 +300,10 @@ test_small_many()
 
 int main(int argc, char *argv[])
 {
-  return test_small_many();
+  int result = test_small_many();
+  if(result == 0)
+    printf("xcmat_nan_inf_test finished OK\n");
+  else
+    printf("Error in xcmat_nan_inf_test: result = %d\n", result);
+  return result;
 }
