@@ -1,6 +1,7 @@
-/* Ergo, version 3.4, a program for linear scaling electronic structure
+/* Ergo, version 3.5, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2014 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -30,7 +31,7 @@
 
 #include "basisinfo.h"
 #include "matrix_typedefs.h"
-
+#include "GetDensFromFock.h"
 
 /** load_density_and_project_full loads one or two density matrices (depending
     on value of noOfDensityMatrices) from the file specified by
@@ -75,6 +76,7 @@ int load_density_and_project_full(const char *densityFileName,
     both sides, but to prepare for purification the matrix S*D*S is needed,
     so we skip multiplication by S_inv since it will anyway be cancelled out.
 
+    @param DensFromFock Instance of GetDensFromFock class contatining all data for computing the density matrix.
     @param densityFileName Name of file to load density matrices from
     @param noOfDensityMatrices Number of density matrices to load
     @param integralInfo static helper object for
@@ -93,22 +95,11 @@ int load_density_and_project_full(const char *densityFileName,
     @param matrix_size_block_info Information about HML matrix block sizes etc.
     @param matrixPermutationVec Permutation vector used when calling matrix lib.
     @param sparse_threshold Threshold used when truncating matrices.
-    @param invCholFactor Inverse Cholesky factor of S.
-    @param invCholFactor_euclnorm Euclidean norm of inverse Cholesky factor.
-    @param gap_expected_lower_bound Expected lower bound for the band gap.
-    @param purification_eigvalue_err_limit Requested accuracy in eigenvalues of D_ort. 
-    @param purification_subspace_err_limit Requested accuracy in the occupied subspace.
-    @param purification_truncation_norm Norm to be used for truncation in purification.
-    @param purification_maxmul Maximum allowed number of matrix multiplications.
-    @param purification_create_m_files Flag to create m-files with information about the purification process.
-    @param use_diagonalization Flag to turn on diagonalization.
-    @param use_diag_on_error Flag to fall back on diagonalization if purification fails.
-    @param purification_ignore_failure Continue even if purification fails to converge.
-    @param purification_use_rand_perturbation_for_alleigsint Apply a random perturbation to (try to) improve the convergence speed of Lanczos calculation of extremal eigenvalues.
-    @param electronic_temperature Electronic temperature.
+
 */
 int 
-load_density_and_project_sparse(const char *densityFileName,
+load_density_and_project_sparse(GetDensFromFock &DensFromFock,
+				const char *densityFileName,
 				int noOfDensityMatrices,
 				const IntegralInfo* integralInfo,
 				const BasisInfoStruct & basisInfo,
@@ -117,20 +108,7 @@ load_density_and_project_sparse(const char *densityFileName,
 				const int* noOfElectronsList,
 				mat::SizesAndBlocks matrix_size_block_info,
 				std::vector<int> const & matrixPermutationVec,
-				ergo_real sparse_threshold,
-				triangMatrix & invCholFactor,
-				ergo_real invCholFactor_euclnorm,
-				ergo_real gap_expected_lower_bound,
-				ergo_real purification_eigvalue_err_limit,
-				ergo_real purification_subspace_err_limit,
-				mat::normType const purification_truncation_norm,
-				int purification_maxmul,
-				int purification_create_m_files,
-				int use_diagonalization,
-				int use_diag_on_error,
-				int purification_ignore_failure,
-				int purification_use_rand_perturbation_for_alleigsint,
-				ergo_real electronic_temperature);
+				ergo_real sparse_threshold);
 
 
 
