@@ -1,6 +1,7 @@
-/* Ergo, version 3.4, a program for linear scaling electronic structure
+/* Ergo, version 3.5, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2014 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -58,5 +59,6 @@ ergo_real integrate_density_in_box(int nPrims,
 ergo_real integrate_density_in_box_2(int nPrims,
 				     DistributionSpecStruct* rho,
 				     ergo_real* minVect, 
-				     ergo_real* maxVect);
+				     ergo_real* maxVect, 
+				     std::vector<int> monomialIntsAdd = std::vector<int>(3, 0));
 
