@@ -1,6 +1,7 @@
-/* Ergo, version 3.4, a program for linear scaling electronic structure
+/* Ergo, version 3.5, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2014 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -66,7 +67,7 @@ compute_V_sparse(const BasisInfoStruct& basisInfo,
 
   ergo_real maxCharge = get_max_charge(molecule);
 
-  do_output(LOG_CAT_INFO, LOG_AREA_SCF, 
+  do_output(LOG_CAT_INFO, LOG_AREA_UNDEFINED, 
 	    "compute_V_sparse, no of atoms = %5i, maxCharge = %6.2f", 
 	    molecule.getNoOfAtoms(), (double)maxCharge);
 
@@ -77,7 +78,7 @@ compute_V_sparse(const BasisInfoStruct& basisInfo,
 							     2000000000);
   if(noOfBasisFuncIndexPairs <= 0)
     {
-      do_output(LOG_CAT_ERROR, LOG_AREA_SCF, "error in get_basis_func_pair_list_1el, noOfBasisFuncIndexPairs = %i", noOfBasisFuncIndexPairs);
+      do_output(LOG_CAT_ERROR, LOG_AREA_UNDEFINED, "error in get_basis_func_pair_list_1el, noOfBasisFuncIndexPairs = %i", noOfBasisFuncIndexPairs);
       return -1;
     }
   std::vector<basis_func_index_pair_struct_1el> basisFuncIndexPairList(noOfBasisFuncIndexPairs);
@@ -89,11 +90,11 @@ compute_V_sparse(const BasisInfoStruct& basisInfo,
 							 noOfBasisFuncIndexPairs);
   if(noOfBasisFuncIndexPairs <= 0)
     {
-      do_output(LOG_CAT_ERROR, LOG_AREA_SCF, "error in get_basis_func_pair_list_1el, noOfBasisFuncIndexPairs = %i", noOfBasisFuncIndexPairs);
+      do_output(LOG_CAT_ERROR, LOG_AREA_UNDEFINED, "error in get_basis_func_pair_list_1el, noOfBasisFuncIndexPairs = %i", noOfBasisFuncIndexPairs);
       return -1;
     }
   
-  do_output(LOG_CAT_INFO, LOG_AREA_SCF, 
+  do_output(LOG_CAT_INFO, LOG_AREA_UNDEFINED, 
 	    "noOfBasisFuncIndexPairs = %i ==> storing %6.2f %% of a full matrix", 
 	    noOfBasisFuncIndexPairs, (double)100*noOfBasisFuncIndexPairs/((double)n*n));
 
@@ -107,7 +108,7 @@ compute_V_sparse(const BasisInfoStruct& basisInfo,
 				   noOfBasisFuncIndexPairs,
 				   false, NULL, NULL) != 0)
     {
-      do_output(LOG_CAT_ERROR, LOG_AREA_SCF, "error in compute_V_and_gradient_linear");
+      do_output(LOG_CAT_ERROR, LOG_AREA_UNDEFINED, "error in compute_V_and_gradient_linear");
       return -1;
     }
   
@@ -145,20 +146,20 @@ compute_gradient_of_nucl_and_trDV(const BasisInfoStruct& basisInfo,
 
   ergo_real maxCharge = get_max_charge(molecule);
 
-  do_output(LOG_CAT_INFO, LOG_AREA_SCF, 
+  do_output(LOG_CAT_INFO, LOG_AREA_UNDEFINED, 
 	    "compute_gradient_of_tr_D_V, no of atoms = %5i, maxCharge = %6.2f", 
 	    molecule.getNoOfAtoms(), (double)maxCharge);
 
   int noOfBasisFuncIndexPairs = get_basis_func_pair_list_1el(basisInfo, threshold, maxCharge, NULL, 2000000000);
   if(noOfBasisFuncIndexPairs <= 0) {
-    do_output(LOG_CAT_ERROR, LOG_AREA_SCF, "error in get_basis_func_pair_list_1el, noOfBasisFuncIndexPairs = %i", noOfBasisFuncIndexPairs);
+    do_output(LOG_CAT_ERROR, LOG_AREA_UNDEFINED, "error in get_basis_func_pair_list_1el, noOfBasisFuncIndexPairs = %i", noOfBasisFuncIndexPairs);
     return -1;
   }
   std::vector<basis_func_index_pair_struct_1el> basisFuncIndexPairList(noOfBasisFuncIndexPairs);
   std::vector<ergo_real> V_list(noOfBasisFuncIndexPairs);
   noOfBasisFuncIndexPairs = get_basis_func_pair_list_1el(basisInfo, threshold, maxCharge, &basisFuncIndexPairList[0], noOfBasisFuncIndexPairs);
   if(noOfBasisFuncIndexPairs <= 0) {
-    do_output(LOG_CAT_ERROR, LOG_AREA_SCF, "error in get_basis_func_pair_list_1el, noOfBasisFuncIndexPairs = %i", noOfBasisFuncIndexPairs);
+    do_output(LOG_CAT_ERROR, LOG_AREA_UNDEFINED, "error in get_basis_func_pair_list_1el, noOfBasisFuncIndexPairs = %i", noOfBasisFuncIndexPairs);
     return -1;
   }
   
@@ -174,7 +175,7 @@ compute_gradient_of_nucl_and_trDV(const BasisInfoStruct& basisInfo,
     densityMatrix_sparse.get_values(rowind, colind, D_list, permutationHML, permutationHML);
   }
   
-  do_output(LOG_CAT_INFO, LOG_AREA_SCF, 
+  do_output(LOG_CAT_INFO, LOG_AREA_UNDEFINED, 
 	    "noOfBasisFuncIndexPairs = %i ==> storing %6.2f %% of a full matrix", 
 	    noOfBasisFuncIndexPairs, (double)100*noOfBasisFuncIndexPairs/((double)n*n));
 
@@ -189,7 +190,7 @@ compute_gradient_of_nucl_and_trDV(const BasisInfoStruct& basisInfo,
 				   true, &D_list[0],
 				   result_gradient_list) != 0)
     {
-      do_output(LOG_CAT_ERROR, LOG_AREA_SCF, "error in compute_V_and_gradient_linear");
+      do_output(LOG_CAT_ERROR, LOG_AREA_UNDEFINED, "error in compute_V_and_gradient_linear");
       return -1;
     }
   
@@ -238,7 +239,7 @@ compute_T_sparse(const BasisInfoStruct& basisInfo,
 			     &nvaluesList[0],
 			     &colindList[0],
 			     &valuesList[0]) != 0) {
-    do_output(LOG_CAT_ERROR, LOG_AREA_SCF, "error in compute_T_matrix_sparse");
+    do_output(LOG_CAT_ERROR, LOG_AREA_UNDEFINED, "error in compute_T_matrix_sparse");
     return -1;
   }
   // Now convert result to three vectors so prepare for HML storage.
@@ -298,16 +299,16 @@ check_diagonal_elements_of_overlap_matrix(int n, const symmMatrix & S_symm)
       if(absdiff > maxabsdiff)
 	maxabsdiff = absdiff;
     }
-  do_output(LOG_CAT_INFO, LOG_AREA_SCF, "diag elements of ovl matrix differ from 1 by at most %9.6g",
+  do_output(LOG_CAT_INFO, LOG_AREA_UNDEFINED, "diag elements of ovl matrix differ from 1 by at most %9.6g",
             (double)maxabsdiff);
   if(maxabsdiff > 1e-3)
     {
-      do_output(LOG_CAT_WARNING, LOG_AREA_SCF, "WARNING: bad overlap matrix: diag elements of ovl matrix differ from 1 by more than safe limit.");
-      do_output(LOG_CAT_WARNING, LOG_AREA_SCF, "WARNING: bad overlap matrix: diag elements of ovl matrix differ from 1 by more than safe limit.");
-      do_output(LOG_CAT_WARNING, LOG_AREA_SCF, "WARNING: bad overlap matrix: diag elements of ovl matrix differ from 1 by more than safe limit.");
-      do_output(LOG_CAT_WARNING, LOG_AREA_SCF, "WARNING: this means that basis functions are not completely normlized.");
-      do_output(LOG_CAT_WARNING, LOG_AREA_SCF, "WARNING: this means that basis functions are not completely normlized.");
-      do_output(LOG_CAT_WARNING, LOG_AREA_SCF, "WARNING: this means that basis functions are not completely normlized.");
+      do_output(LOG_CAT_WARNING, LOG_AREA_UNDEFINED, "WARNING: bad overlap matrix: diag elements of ovl matrix differ from 1 by more than safe limit.");
+      do_output(LOG_CAT_WARNING, LOG_AREA_UNDEFINED, "WARNING: bad overlap matrix: diag elements of ovl matrix differ from 1 by more than safe limit.");
+      do_output(LOG_CAT_WARNING, LOG_AREA_UNDEFINED, "WARNING: bad overlap matrix: diag elements of ovl matrix differ from 1 by more than safe limit.");
+      do_output(LOG_CAT_WARNING, LOG_AREA_UNDEFINED, "WARNING: this means that basis functions are not completely normlized.");
+      do_output(LOG_CAT_WARNING, LOG_AREA_UNDEFINED, "WARNING: this means that basis functions are not completely normlized.");
+      do_output(LOG_CAT_WARNING, LOG_AREA_UNDEFINED, "WARNING: this means that basis functions are not completely normlized.");
     }
   return 0;
 }
@@ -325,7 +326,7 @@ compute_overlap_matrix_sparse(const BasisInfoStruct& basisInfo,
 					 S_symm,
 					 permutationHML) != 0)
     {
-      do_output(LOG_CAT_ERROR, LOG_AREA_SCF, "error in compute_operator_matrix_sparse_symm");
+      do_output(LOG_CAT_ERROR, LOG_AREA_UNDEFINED, "error in compute_operator_matrix_sparse_symm");
       return -1;
     }
   
@@ -364,7 +365,7 @@ compute_operator_matrix_sparse_symm(const BasisInfoStruct& basisInfo,
 				    colindList,
 				    valuesList) != 0)
     {
-      do_output(LOG_CAT_ERROR, LOG_AREA_SCF, "error in compute_operator_matrix_sparse");
+      do_output(LOG_CAT_ERROR, LOG_AREA_UNDEFINED, "error in compute_operator_matrix_sparse");
       return -1;
     }
   
@@ -433,7 +434,7 @@ compute_J_by_boxes_sparse(const BasisInfoStruct& basisInfo,
   int n = basisInfo.noOfBasisFuncs;
 
   ergo_real maxDensityMatrixElement = densityMatrix_sparse.maxAbsValue();
-  do_output(LOG_CAT_INFO, LOG_AREA_SCF, "compute_J_by_boxes_sparse, maxDensityMatrixElement = %22.11f", 
+  do_output(LOG_CAT_INFO, LOG_AREA_UNDEFINED, "compute_J_by_boxes_sparse, maxDensityMatrixElement = %22.11f", 
 	    (double)maxDensityMatrixElement);
 
   std::vector<basis_func_index_pair_struct> basisFuncIndexPairList;
@@ -443,12 +444,12 @@ compute_J_by_boxes_sparse(const BasisInfoStruct& basisInfo,
 							     maxDensityMatrixElement,
 							     basisFuncIndexPairList);
   if(noOfBasisFuncIndexPairs <= 0) {
-    do_output(LOG_CAT_ERROR, LOG_AREA_SCF, 
+    do_output(LOG_CAT_ERROR, LOG_AREA_UNDEFINED, 
 	      "error in get_basis_func_pair_list, noOfBasisFuncIndexPairs = %i", noOfBasisFuncIndexPairs);
     return -1;
   }
   
-  do_output(LOG_CAT_INFO, LOG_AREA_SCF, 
+  do_output(LOG_CAT_INFO, LOG_AREA_UNDEFINED, 
 	    "compute_J_by_boxes_sparse: noOfBasisFuncIndexPairs = %i ==> storing %6.2f %% of a full matrix", 
 	    noOfBasisFuncIndexPairs, (double)100*noOfBasisFuncIndexPairs/((double)n*n));
   
@@ -471,7 +472,7 @@ compute_J_by_boxes_sparse(const BasisInfoStruct& basisInfo,
 
   std::vector<ergo_real> J_list(noOfBasisFuncIndexPairs);
 
-  output_current_memory_usage(LOG_AREA_SCF, "Before calling compute_J_by_boxes_linear");
+  output_current_memory_usage(LOG_AREA_UNDEFINED, "Before calling compute_J_by_boxes_linear");
   if(compute_J_by_boxes_linear(basisInfo,
 			       integralInfo,
 			       J_K_params,
@@ -481,10 +482,10 @@ compute_J_by_boxes_sparse(const BasisInfoStruct& basisInfo,
 			       &J_list[0],
 			       noOfBasisFuncIndexPairs) != 0)
     {
-      do_output(LOG_CAT_ERROR, LOG_AREA_SCF, "error in compute_J_by_boxes_linear");
+      do_output(LOG_CAT_ERROR, LOG_AREA_UNDEFINED, "error in compute_J_by_boxes_linear");
       return -1;
     }
-  output_current_memory_usage(LOG_AREA_SCF, "After calling compute_J_by_boxes_linear");
+  output_current_memory_usage(LOG_AREA_UNDEFINED, "After calling compute_J_by_boxes_linear");
 
   // Now transfer result from J_list to J
   {
@@ -500,10 +501,10 @@ compute_J_by_boxes_sparse(const BasisInfoStruct& basisInfo,
 			 J_list,
 			 permutationHML,
 			 permutationHML);
-    output_current_memory_usage(LOG_AREA_SCF, "After J.assign_from_sparse");
+    output_current_memory_usage(LOG_AREA_UNDEFINED, "After J.assign_from_sparse");
   }
 
-  timeMeter.print(LOG_AREA_SCF, "compute_J_by_boxes_sparse total");
+  timeMeter.print(LOG_AREA_UNDEFINED, "compute_J_by_boxes_sparse total");
   return 0;
 }
 
@@ -542,7 +543,7 @@ get_CSR_from_symmMatrix(int n,
 		     &rowind[0],
 		     &colind[0]) != 0)
     {
-      do_output(LOG_CAT_ERROR, LOG_AREA_SCF, "error in ergo_CSR_create.");
+      do_output(LOG_CAT_ERROR, LOG_AREA_UNDEFINED, "error in ergo_CSR_create.");
       return -1;
     }
   for(int i = 0; i < nvalues; i++)
@@ -557,7 +558,7 @@ get_CSR_from_symmMatrix(int n,
     {
       if(ergo_CSR_get_element(&CSR, rowind[i], colind[i]) != values[i])
 	{
-	  do_output(LOG_CAT_ERROR, LOG_AREA_SCF, "error checking CSR matrix.");
+	  do_output(LOG_CAT_ERROR, LOG_AREA_UNDEFINED, "error checking CSR matrix.");
 	  return -1;
 	}
     }
@@ -587,7 +588,7 @@ get_CSR_from_normalMatrix(int n,
 		     &rowind[0],
 		     &colind[0]) != 0)
     {
-      do_output(LOG_CAT_ERROR, LOG_AREA_SCF, "error in ergo_CSR_create.");
+      do_output(LOG_CAT_ERROR, LOG_AREA_UNDEFINED, "error in ergo_CSR_create.");
       return -1;
     }
   for(int i = 0; i < nvalues; i++) {
@@ -600,7 +601,7 @@ get_CSR_from_normalMatrix(int n,
   for(int i = 0; i < nvalues; i++) {
     if(ergo_CSR_get_element(&CSR, rowind[i], colind[i]) != values[i])
       {
-	do_output(LOG_CAT_ERROR, LOG_AREA_SCF, "Error in get_CSR_from_normalMatrix, error checking CSR matrix.");
+	do_output(LOG_CAT_ERROR, LOG_AREA_UNDEFINED, "Error in get_CSR_from_normalMatrix, error checking CSR matrix.");
 	return -1;
       }
   }
@@ -624,7 +625,7 @@ compute_K_by_boxes_sparse(const BasisInfoStruct& basisInfo,
 {
   Util::TimeMeter timeMeter;
   int n = basisInfo.noOfBasisFuncs;
-  output_current_memory_usage(LOG_AREA_SCF, "Beginning of compute_K_by_boxes_sparse");
+  output_current_memory_usage(LOG_AREA_UNDEFINED, "Beginning of compute_K_by_boxes_sparse");
   
   csr_matrix_struct dens_CSR;
   memset(&dens_CSR, 0, sizeof(csr_matrix_struct));
@@ -637,7 +638,7 @@ compute_K_by_boxes_sparse(const BasisInfoStruct& basisInfo,
 			     inversePermutationHML,
 			     dens_CSR) != 0)
     {
-      do_output(LOG_CAT_ERROR, LOG_AREA_SCF, "error in get_CSR_from_symmMatrix");
+      do_output(LOG_CAT_ERROR, LOG_AREA_UNDEFINED, "error in get_CSR_from_symmMatrix");
       return -1;
     }
 
@@ -650,13 +651,13 @@ compute_K_by_boxes_sparse(const BasisInfoStruct& basisInfo,
 		      &K_CSR,
 		      1) != 0)
     {
-      do_output(LOG_CAT_ERROR, LOG_AREA_SCF, "error in compute_K_by_boxes");
+      do_output(LOG_CAT_ERROR, LOG_AREA_UNDEFINED, "error in compute_K_by_boxes");
       return -1;
     }
 
   densityMatrix_sparse.writeToFile();
   
-  output_current_memory_usage(LOG_AREA_SCF, "Before calling compute_K_by_boxes");
+  output_current_memory_usage(LOG_AREA_UNDEFINED, "Before calling compute_K_by_boxes");
   if(compute_K_by_boxes(basisInfo,
 			integralInfo,
 			CAM_params,
@@ -667,10 +668,10 @@ compute_K_by_boxes_sparse(const BasisInfoStruct& basisInfo,
 			&dens_CSR,
 			1) != 0)
     {
-      do_output(LOG_CAT_ERROR, LOG_AREA_SCF, "error in compute_K_by_boxes");
+      do_output(LOG_CAT_ERROR, LOG_AREA_UNDEFINED, "error in compute_K_by_boxes");
       return -1;
     }
-  output_current_memory_usage(LOG_AREA_SCF, "After calling compute_K_by_boxes");
+  output_current_memory_usage(LOG_AREA_UNDEFINED, "After calling compute_K_by_boxes");
 
   // collect result
   // Convert K matrix from CSR to HML format.
@@ -684,7 +685,7 @@ compute_K_by_boxes_sparse(const BasisInfoStruct& basisInfo,
 			 &values[0],
 			 nvalues) != 0)
     {
-      do_output(LOG_CAT_ERROR, LOG_AREA_SCF, "error in ergo_CSR_get_values.");
+      do_output(LOG_CAT_ERROR, LOG_AREA_UNDEFINED, "error in ergo_CSR_get_values.");
       return -1;
     }
   // now the information is in the vectors rowind colind values, we can free memory for K_CSR.
@@ -700,7 +701,7 @@ compute_K_by_boxes_sparse(const BasisInfoStruct& basisInfo,
   
   densityMatrix_sparse.readFromFile();
   
-  timeMeter.print(LOG_AREA_SCF, "compute_K_by_boxes_sparse total");
+  timeMeter.print(LOG_AREA_UNDEFINED, "compute_K_by_boxes_sparse total");
   return 0;
 }
 
@@ -745,7 +746,7 @@ compute_K_by_boxes_sparse_nosymm(const BasisInfoStruct& basisInfo,
 {
   Util::TimeMeter timeMeter;
   int n = basisInfo.noOfBasisFuncs;
-  output_current_memory_usage(LOG_AREA_SCF, "Beginning of compute_K_by_boxes_sparse");
+  output_current_memory_usage(LOG_AREA_UNDEFINED, "Beginning of compute_K_by_boxes_sparse");
   
   csr_matrix_struct dens_CSR;
   memset(&dens_CSR, 0, sizeof(csr_matrix_struct));
@@ -761,7 +762,7 @@ compute_K_by_boxes_sparse_nosymm(const BasisInfoStruct& basisInfo,
 			       inversePermutationHML,
 			       dens_CSR) != 0)
     {
-      do_output(LOG_CAT_ERROR, LOG_AREA_SCF, "error in get_CSR_from_normalMatrix");
+      do_output(LOG_CAT_ERROR, LOG_AREA_UNDEFINED, "error in get_CSR_from_normalMatrix");
       return -1;
     }
 
@@ -774,7 +775,7 @@ compute_K_by_boxes_sparse_nosymm(const BasisInfoStruct& basisInfo,
 		      &K_CSR,
 		      0) != 0)
     {
-      do_output(LOG_CAT_ERROR, LOG_AREA_SCF, "error in compute_K_by_boxes");
+      do_output(LOG_CAT_ERROR, LOG_AREA_UNDEFINED, "error in compute_K_by_boxes");
       return -1;
     }
   if(create_CSR_for_K(basisInfo,
@@ -784,13 +785,13 @@ compute_K_by_boxes_sparse_nosymm(const BasisInfoStruct& basisInfo,
 		      &K_CSR_2,
 		      0) != 0)
     {
-      do_output(LOG_CAT_ERROR, LOG_AREA_SCF, "error in compute_K_by_boxes");
+      do_output(LOG_CAT_ERROR, LOG_AREA_UNDEFINED, "error in compute_K_by_boxes");
       return -1;
     }
 
   densityMatrix_sparse.writeToFile();
 
-  output_current_memory_usage(LOG_AREA_SCF, "Before calling compute_K_by_boxes");
+  output_current_memory_usage(LOG_AREA_UNDEFINED, "Before calling compute_K_by_boxes");
   if(compute_K_by_boxes(basisInfo,
 			integralInfo,
 			CAM_params,
@@ -801,10 +802,10 @@ compute_K_by_boxes_sparse_nosymm(const BasisInfoStruct& basisInfo,
 			&dens_CSR,
 			0) != 0)
     {
-      do_output(LOG_CAT_ERROR, LOG_AREA_SCF, "error in compute_K_by_boxes");
+      do_output(LOG_CAT_ERROR, LOG_AREA_UNDEFINED, "error in compute_K_by_boxes");
       return -1;
     }
-  output_current_memory_usage(LOG_AREA_SCF, "After calling compute_K_by_boxes");
+  output_current_memory_usage(LOG_AREA_UNDEFINED, "After calling compute_K_by_boxes");
 
   // collect result
   // Convert K matrix from CSR to HML format.
@@ -818,7 +819,7 @@ compute_K_by_boxes_sparse_nosymm(const BasisInfoStruct& basisInfo,
 			 &values[0],
 			 nvalues) != 0)
     {
-      do_output(LOG_CAT_ERROR, LOG_AREA_SCF, "error in ergo_CSR_get_values.");
+      do_output(LOG_CAT_ERROR, LOG_AREA_UNDEFINED, "error in ergo_CSR_get_values.");
       return -1;
     }
   // now the information is in the vectors rowind colind values, we can free memory for K_CSR.
@@ -834,6 +835,6 @@ compute_K_by_boxes_sparse_nosymm(const BasisInfoStruct& basisInfo,
   
   densityMatrix_sparse.readFromFile();
   
-  timeMeter.print(LOG_AREA_SCF, "compute_K_by_boxes_sparse total");
+  timeMeter.print(LOG_AREA_UNDEFINED, "compute_K_by_boxes_sparse total");
   return 0;
 }
