@@ -1,6 +1,7 @@
-/* Ergo, version 3.4, a program for linear scaling electronic structure
+/* Ergo, version 3.5, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2014 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -28,15 +29,8 @@
 #ifndef ATOM_LABELS_HEADER
 #define ATOM_LABELS_HEADER
 
-#ifdef __cplusplus
-#define EXTERN_C extern "C"
-#else
-#define EXTERN_C
-#endif
-
-EXTERN_C int get_charge_int_from_atom_label(const char* atomLabel);
-
-EXTERN_C int get_atom_label_from_charge_int(int charge, char* atomLabelString, size_t bufferSize);
+int get_charge_int_from_atom_label(const char* atomLabel);
 
+int get_atom_label_from_charge_int(int charge, char* atomLabelString, size_t bufferSize);
 
 #endif
