@@ -1,6 +1,7 @@
-/* Ergo, version 3.4, a program for linear scaling electronic structure
+/* Ergo, version 3.5, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2014 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -45,7 +46,7 @@ ergo_malloc(size_t noOfBytes)
     double noOfMegaBytes = noOfBytesAsDouble / 1000000;
     do_output(LOG_CAT_ERROR, LOG_AREA_MAIN, 
 	      "error in ergo_malloc, noOfBytes = %12.0f  ( %7.2f MegaBytes )", noOfBytesAsDouble, noOfMegaBytes);
-    do_output_time(LOG_CAT_INFO, LOG_AREA_SCF, "ERROR. ergo_malloc() calling exit()");
+    do_output_time(LOG_CAT_INFO, LOG_AREA_UNDEFINED, "ERROR. ergo_malloc() calling exit()");
     exit(0);
   }
   pthread_mutex_lock(&globalMemStatLock);
