@@ -1,6 +1,7 @@
-/* Ergo, version 3.4, a program for linear scaling electronic structure
+/* Ergo, version 3.5, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2014 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -37,16 +38,50 @@ static int global_memory_usage_output_flag = 0; /* mem output disabled by defaul
 /* We choose to have output disabled by default, until enable_output() is called, to make sure output is disabled on worker processes when CHT is used. */ 
 static int global_output_enabled_flag = 0; /* output disabled by default */
 
+static int use_printf = 0; /* use printf */
+
 /* general output routine  */
 void 
 do_output(int logCategory, int logArea, const char* format, ...)
 {
   va_list a;
   va_start(a, format);
-  do_voutput(logCategory, logArea, format, a);
+  if(use_printf)
+    do_voutput_printf(logCategory, logArea, format, a);
+  else
+    do_voutput(logCategory, logArea, format, a);
   va_end(a);
 }
 
+/* output in stdout using printf */
+void 
+enable_printf_output()
+{
+  global_output_enabled_flag = 1;
+  use_printf = 1;
+
+  //global_memory_usage_output_flag = 1;
+}
+
+
+int
+do_voutput_printf(int logCategory, int logArea, const char* format, va_list a)
+{
+  if(global_output_enabled_flag == 0)
+    return 0; // Output is disabled; do nothing, just return.
+  
+  char ss[8888]; /* FIXME: Do something nicer here. */
+  int r;
+
+  memset(ss, 0, sizeof(ss));
+  r = vsnprintf(ss, sizeof(ss), format, a);
+
+  printf("%s\n", ss);
+
+  return r;
+}
+
+
 int
 do_voutput(int logCategory, int logArea, const char* format, va_list a)
 {
@@ -84,6 +119,8 @@ do_voutput(int logCategory, int logArea, const char* format, va_list a)
     case LOG_AREA_DFT      : strcat(ss, "DF"); break;
     case LOG_AREA_LOWLEVEL:  strcat(ss, "LO"); break;
     case LOG_AREA_CI:        strcat(ss, "CI"); break;
+    case LOG_AREA_ED:        strcat(ss, "ED"); break;
+    case LOG_AREA_GS:        strcat(ss, "GS"); break;
     default:                 strcat(ss, "  "); break;
     }
 
@@ -108,6 +145,8 @@ do_voutput(int logCategory, int logArea, const char* format, va_list a)
   return r;
 }
 
+
+
 void
 do_output_time(int logCategory, int logArea, const char* s)
 {
