@@ -1,6 +1,7 @@
-/* Ergo, version 3.4, a program for linear scaling electronic structure
+/* Ergo, version 3.5, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2014 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -32,6 +33,7 @@
 
 #define UNIT_one_Angstrom ((ergo_real)(1.0 / 0.5291772083))
 #define UNIT_one_Debye    0.393456
-#define UNIT_one_eV       0.0367502
+#define UNIT_one_eV       0.036749322    // According to nist.gov 2014 CODATA values
+#define UNIT_one_Kelvin   3.16681046e-06 // Computed from nist.gov 2014 CODATA values
 
 #endif
