@@ -1,6 +1,7 @@
-/* Ergo, version 3.4, a program for linear scaling electronic structure
+/* Ergo, version 3.5, a program for linear scaling electronic structure
  * calculations.
- * Copyright (C) 2014 Elias Rudberg, Emanuel H. Rubensson, and Pawel Salek.
+ * Copyright (C) 2016 Elias Rudberg, Emanuel H. Rubensson, Pawel Salek,
+ * and Anastasia Kruchinina.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -27,10 +28,6 @@
 
 /** @file utilities.cc Basic OS access utilities. */
 
-/* gethostbyname() is available only in Unix98 standard - or BSD. We
-   pick the first one. */
-#define _XOPEN_SOURCE 500
-
 #include <string.h>
 #include <stdio.h>
 #include <stdlib.h>
