@@ -1,5 +1,7 @@
 #!/bin/sh
 
+currtestdirname=tmpdir_test_dft_hicu
+
 if test "$top_builddir" = ""; then
     top_builddir=..
 fi
@@ -7,24 +9,18 @@ if test "$top_srcdir" = ""; then
     top_srcdir=..
 fi
 
-if test `"$top_builddir"/source/ergo -e precision` = single; then
+. "$top_srcdir"/test/functions
+
+# Run each test in a separate directory, to allow "make check -j" to work properly.
+currdir=`pwd` ; cd $top_builddir ; top_builddir_pwd=`pwd` ; cd $currdir
+rm -rf $currtestdirname ; mkdir $currtestdirname ; cd $currtestdirname
+ln -s "$top_builddir_pwd"/source/ergo ./ergo
+
+if test `./ergo -e precision` = 'single'; then
     echo SKIPPED
     exit 0
 fi
 
-# Prefer gawk - we know exactly what it can do.
-# awk on Sun does not support functions, need to use nawk for this
-if gawk '{print 1}'</dev/null > /dev/null 2>&1; then
-   AWK=gawk
-elif nawk '{print 1}'</dev/null > /dev/null 2>&1; then
-   AWK=nawk
-else
-   AWK=awk
-fi
-
-. "$top_srcdir"/test/functions
-
-
 errorfilename=ergoscf.out.error.dfthicu
 
 echo
@@ -33,7 +29,7 @@ echo
 
 echo Testing twisted h2o BLYP/6-31G** with HiCu grid
 rm -f ergoscf.out
-"$top_builddir"/source/ergo <<EOINPUT > /dev/null
+./ergo <<EOINPUT > /dev/null
 molecule_inline
 O     0.3       0.11      0.23
 H    -1.89      0.18     -0.07
@@ -58,7 +54,7 @@ fi
 
 echo Testing H4 BHandHLYP/cc-pVDZ with HiCu grid
 rm -f ergoscf.out
-"$top_builddir"/source/ergo <<EOINPUT > /dev/null
+./ergo <<EOINPUT > /dev/null
 molecule_inline
 H   2.0   3.0   12.02
 H   2.7   3.4   12.16
@@ -85,7 +81,7 @@ fi
 
 echo Testing H2 BPW91/cc-pVTZ with HiCu grid
 rm -f ergoscf.out
-"$top_builddir"/source/ergo <<EOINPUT > /dev/null
+./ergo <<EOINPUT > /dev/null
 molecule_inline
 H   2.0   3.0   11.12
 H   2.5   3.2   11.31
@@ -110,7 +106,7 @@ fi
 
 echo Testing c2h8 LDA/4-31G with sparse XC and HiCu grid
 rm -f ergoscf.out
-"$top_builddir"/source/ergo <<EOINPUT > /dev/null
+./ergo <<EOINPUT > /dev/null
 molecule_inline Angstrom
 C     0.0       0.0       0.0
 C     0.0       0.0      10.0
@@ -141,8 +137,8 @@ fi
 
 
 
-rm ergoscf.out
-rm density.bin
+cd ..
+rm -r $currtestdirname
 
 echo
 echo DFT HiCu grid tests completed successfully!
