@@ -1,5 +1,7 @@
 #!/bin/sh
 
+currtestdirname=tmpdir_test_dft_sparse
+
 if test "$top_builddir" = ""; then
     top_builddir=..
 fi
@@ -7,30 +9,25 @@ if test "$top_srcdir" = ""; then
     top_srcdir=..
 fi
 
-if test `"$top_builddir"/source/ergo -e precision` = single; then
+. "$top_srcdir"/test/functions
+
+# Run each test in a separate directory, to allow "make check -j" to work properly.
+currdir=`pwd` ; cd $top_builddir ; top_builddir_pwd=`pwd` ; cd $currdir
+rm -rf $currtestdirname ; mkdir $currtestdirname ; cd $currtestdirname
+ln -s "$top_builddir_pwd"/source/ergo ./ergo
+
+if test `./ergo -e precision` = 'single'; then
     echo SKIPPED
     exit 0
 fi
 
-# Prefer gawk - we know exactly what it can do.
-# awk on Sun does not support functions, need to use nawk for this
-if gawk '{print 1}'</dev/null > /dev/null 2>&1; then
-   AWK=gawk
-elif nawk '{print 1}'</dev/null > /dev/null 2>&1; then
-   AWK=nawk
-else
-   AWK=awk
-fi
-
-. "$top_srcdir"/test/functions
-
 errorfilename=ergoscf.out.error.dftsparse
 
 echo
 
 echo Testing c2h8 LDA/6-31G*
 rm -f ergoscf.out
-"$top_builddir"/source/ergo <<EOINPUT > /dev/null
+./ergo <<EOINPUT > /dev/null
 molecule_inline Angstrom
 C     0.0       0.0       0.0
 C     0.0       0.0      10.0
@@ -60,7 +57,7 @@ fi
 
 echo Testing h2o BLYP/6-31G**
 rm -f ergoscf.out
-"$top_builddir"/source/ergo <<EOINPUT > /dev/null
+./ergo <<EOINPUT > /dev/null
 molecule_inline
 O     0.0       0.0       0.0
 H    -1.809     0.0       0.0
@@ -83,6 +80,9 @@ echo output file saved as $errorfilename
 exit 1
 fi
 
+cd ..
+rm -r $currtestdirname
+
 echo
 echo Sparse DFT tests completed successfully!
 echo
