@@ -1,5 +1,7 @@
 #!/bin/sh
 
+currtestdirname=tmpdir_test_fmm
+
 if test "$top_builddir" = ""; then
     top_builddir=..
 fi
@@ -7,30 +9,25 @@ if test "$top_srcdir" = ""; then
     top_srcdir=..
 fi
 
-if test `"$top_builddir"/source/ergo -e precision` = single; then
+. "$top_srcdir"/test/functions
+
+# Run each test in a separate directory, to allow "make check -j" to work properly.
+currdir=`pwd` ; cd $top_builddir ; top_builddir_pwd=`pwd` ; cd $currdir
+rm -rf $currtestdirname ; mkdir $currtestdirname ; cd $currtestdirname
+ln -s "$top_builddir_pwd"/source/ergo ./ergo
+
+if test `./ergo -e precision` = 'single'; then
     echo SKIPPED
     exit 0
 fi
 
-# Prefer gawk - we know exactly what it can do.
-# awk on Sun does not support functions, need to use nawk for this
-if gawk '{print 1}'</dev/null > /dev/null 2>&1; then
-   AWK=gawk
-elif nawk '{print 1}'</dev/null > /dev/null 2>&1; then
-   AWK=nawk
-else
-   AWK=awk
-fi
-
-. "$top_srcdir"/test/functions
-
 errorfilename=ergoscf.out.error.fmm
 
 echo
 
 echo Testing four_h2o HF/6-31G using FMM
 rm -f ergoscf.out
-"$top_builddir"/source/ergo <<EOINPUT > /dev/null
+./ergo <<EOINPUT > /dev/null
 molecule_inline Angstrom
 H                  8.675000    0.000000  -16.640000
 H                  9.579936    0.000000  -17.920455
@@ -64,7 +61,7 @@ rm ergoscf.out
 
 echo Testing four_h2o HF/6-31G* using FMM
 rm -f ergoscf.out
-"$top_builddir"/source/ergo <<EOINPUT > /dev/null
+./ergo <<EOINPUT > /dev/null
 molecule_inline Angstrom
 H                  8.675000    0.000000  -16.640000
 H                  9.579936    0.000000  -17.920455
@@ -98,6 +95,9 @@ rm ergoscf.out
 rm density.bin
 
 
+cd ..
+rm -r $currtestdirname
+
 echo
 echo FMM tests completed successfully!
 echo
