@@ -1,5 +1,7 @@
 #!/bin/sh
 
+currtestdirname=tmpdir_test_ghost
+
 if test "$top_builddir" = ""; then
     top_builddir=..
 fi
@@ -7,23 +9,18 @@ if test "$top_srcdir" = ""; then
     top_srcdir=..
 fi
 
-if test `"$top_builddir"/source/ergo -e precision` = single; then
+. "$top_srcdir"/test/functions
+
+# Run each test in a separate directory, to allow "make check -j" to work properly.
+currdir=`pwd` ; cd $top_builddir ; top_builddir_pwd=`pwd` ; cd $currdir
+rm -rf $currtestdirname ; mkdir $currtestdirname ; cd $currtestdirname
+ln -s "$top_builddir_pwd"/source/ergo ./ergo
+
+if test `./ergo -e precision` = 'single'; then
     echo SKIPPED
     exit 0
 fi
 
-# Prefer gawk - we know exactly what it can do.
-# awk on Sun does not support functions, need to use nawk for this
-if gawk '{print 1}'</dev/null > /dev/null 2>&1; then
-   AWK=gawk
-elif nawk '{print 1}'</dev/null > /dev/null 2>&1; then
-   AWK=nawk
-else
-   AWK=awk
-fi
-
-. "$top_srcdir"/test/functions
-
 errorfilename=ergoscf.out.error.ghost
 
 echo
@@ -31,7 +28,7 @@ echo
 
 echo Testing h2o HF/6-31G** with extra basis funcs from ghost molecule
 rm -f ergoscf.out
-"$top_builddir"/source/ergo <<EOINPUT > /dev/null
+./ergo <<EOINPUT > /dev/null
 molecule_inline Angstrom
 O        0.457627840    -0.311951930     0.082447166
 H        0.457627840     0.594655950    -0.201387561
@@ -59,8 +56,8 @@ fi
 
 
 
-rm ergoscf.out
-rm density.bin
+cd ..
+rm -r $currtestdirname
 
 echo
 echo Ghost-molecule tests completed successfully!
