@@ -1,10 +1,17 @@
 #!/bin/sh
 
-if test "$top_builddir" = ""; then
-    top_builddir=..
+# Run each test in a separate directory, to allow "make check -j" to work properly.
+currtestdirname=tmpdir_test_gluala2_hf
+
+rm -rf $currtestdirname
+mkdir $currtestdirname
+cd $currtestdirname
+
+if [ "$top_builddir" = "" ] || [ "$top_builddir" = ".." ] ; then
+    top_builddir=../..
 fi
-if test "$top_srcdir" = ""; then
-    top_srcdir=..
+if [ "$top_srcdir" = "" ] || [ "$top_srcdir" = ".." ] ; then
+    top_srcdir=../..
 fi
 
 if test `"$top_builddir"/source/ergo -e precision` = single; then
@@ -12,16 +19,6 @@ if test `"$top_builddir"/source/ergo -e precision` = single; then
     exit 0
 fi
 
-# Prefer gawk - we know exactly what it can do.
-# awk on Sun does not support functions, need to use nawk for this
-if gawk '{print 1}'</dev/null > /dev/null 2>&1; then
-   AWK=gawk
-elif nawk '{print 1}'</dev/null > /dev/null 2>&1; then
-   AWK=nawk
-else
-   AWK=awk
-fi
-
 . "$top_srcdir"/test/functions
 
 errorfilename=ergoscf.out.error.gluala2.hf
@@ -68,7 +65,9 @@ exit 1
 fi
 
 
-rm density.bin
+
+cd ..
+rm -r $currtestdirname
 
 echo
 echo GluAla2 Hartree-Fock test completed successfully!
