@@ -1,5 +1,7 @@
 #!/bin/sh
 
+currtestdirname=tmpdir_test_hf
+
 if test "$top_builddir" = ""; then
     top_builddir=..
 fi
@@ -7,30 +9,25 @@ if test "$top_srcdir" = ""; then
     top_srcdir=..
 fi
 
-if test `"$top_builddir"/source/ergo -e precision` = single; then
+. "$top_srcdir"/test/functions
+
+# Run each test in a separate directory, to allow "make check -j" to work properly.
+currdir=`pwd` ; cd $top_builddir ; top_builddir_pwd=`pwd` ; cd $currdir
+rm -rf $currtestdirname ; mkdir $currtestdirname ; cd $currtestdirname
+ln -s "$top_builddir_pwd"/source/ergo ./ergo
+
+if test `./ergo -e precision` = 'single'; then
     echo SKIPPED
     exit 0
 fi
 
-# Prefer gawk - we know exactly what it can do.
-# awk on Sun does not support functions, need to use nawk for this
-if gawk '{print 1}'</dev/null > /dev/null 2>&1; then
-   AWK=gawk
-elif nawk '{print 1}'</dev/null > /dev/null 2>&1; then
-   AWK=nawk
-else
-   AWK=awk
-fi
-
-. "$top_srcdir"/test/functions
-
 errorfilename=ergoscf.out.error.hf
 
 echo
 
 echo Testing h2 with bond distance 1 au, HF/STO-3G
 rm -f ergoscf.out
-"$top_builddir"/source/ergo <<EOINPUT > /dev/null
+./ergo <<EOINPUT > /dev/null
 molecule_inline
 H   0  0 0
 H   0  0 1
@@ -53,7 +50,7 @@ fi
 
 echo Testing h2 with bond distance 5 au, HF/6-31G**
 rm -f ergoscf.out
-"$top_builddir"/source/ergo <<EOINPUT > /dev/null
+./ergo <<EOINPUT > /dev/null
 molecule_inline
 H   -2.5  0  0
 H    2.5  0  0
@@ -75,7 +72,7 @@ fi
 
 echo Testing h2o HF/6-31G**
 rm -f ergoscf.out
-"$top_builddir"/source/ergo <<EOINPUT > /dev/null
+./ergo <<EOINPUT > /dev/null
 molecule_inline
 O     0.0       0.0       0.0
 H    -1.809     0.0       0.0
@@ -98,7 +95,7 @@ fi
 
 echo Testing h2o HF/6-31++G**
 rm -f ergoscf.out
-"$top_builddir"/source/ergo <<EOINPUT > /dev/null
+./ergo <<EOINPUT > /dev/null
 molecule_inline
 O     0.0       0.0       0.0
 H    -1.809     0.0       0.0
@@ -121,7 +118,7 @@ fi
 
 echo Testing h2o HF/cc-pVTZ
 rm -f ergoscf.out
-"$top_builddir"/source/ergo <<EOINPUT > /dev/null
+./ergo <<EOINPUT > /dev/null
 molecule_inline
 O     0.0       0.0       0.0
 H    -1.809     0.0       0.0
@@ -144,7 +141,7 @@ fi
 
 echo Testing cnof HF/6-31G
 rm -f ergoscf.out
-"$top_builddir"/source/ergo <<EOINPUT > /dev/null
+./ergo <<EOINPUT > /dev/null
 molecule_inline
 C     0.0       0.0       0.0
 N     1.3       0.2       0.4
@@ -168,7 +165,7 @@ fi
 
 echo Testing nh3[+] UHF/6-31G**
 rm -f ergoscf.out
-"$top_builddir"/source/ergo <<EOINPUT > /dev/null
+./ergo <<EOINPUT > /dev/null
 molecule_inline Angstrom
 N      0.000000     0.000000     0.000000
 H      0.000000     0.000000     1.012316
@@ -204,8 +201,8 @@ fi
 
 
 
-rm ergoscf.out
-rm density.bin
+cd ..
+rm -r $currtestdirname
 
 echo
 echo Hartree-Fock tests completed successfully!
