@@ -1,5 +1,7 @@
 #!/bin/sh
 
+currtestdirname=tmpdir_test_uhf
+
 if test "$top_builddir" = ""; then
     top_builddir=..
 fi
@@ -7,30 +9,25 @@ if test "$top_srcdir" = ""; then
     top_srcdir=..
 fi
 
-if test `"$top_builddir"/source/ergo -e precision` = 'single'; then
+. "$top_srcdir"/test/functions
+
+# Run each test in a separate directory, to allow "make check -j" to work properly.
+currdir=`pwd` ; cd $top_builddir ; top_builddir_pwd=`pwd` ; cd $currdir
+rm -rf $currtestdirname ; mkdir $currtestdirname ; cd $currtestdirname
+ln -s "$top_builddir_pwd"/source/ergo ./ergo
+
+if test `./ergo -e precision` = 'single'; then
     echo SKIPPED
     exit 0
 fi
 
-# Prefer gawk - we know exactly what it can do.
-# awk on Sun does not support functions, need to use nawk for this
-if gawk '{print 1}'</dev/null > /dev/null 2>&1; then
-   AWK=gawk
-elif nawk '{print 1}'</dev/null > /dev/null 2>&1; then
-   AWK=nawk
-else
-   AWK=awk
-fi
-
-. "$top_srcdir"/test/functions
-
 errorfilename=ergoscf.out.error.uhf
 
 echo
 
 echo Testing H UHF STO-3G
 rm -f ergoscf.out
-"$top_builddir"/source/ergo <<EOINPUT > /dev/null
+./ergo <<EOINPUT > /dev/null
 molecule_inline
 H    0      0     0
 EOF
@@ -58,7 +55,7 @@ rm density.bin
 
 echo Testing H UHF 6-31G**
 rm -f ergoscf.out
-"$top_builddir"/source/ergo <<EOINPUT > /dev/null
+./ergo <<EOINPUT > /dev/null
 molecule_inline
 H    0      0     0
 EOF
@@ -86,7 +83,7 @@ rm density.bin
 
 echo Testing H2[+] UHF STO-3G
 rm -f ergoscf.out
-"$top_builddir"/source/ergo <<EOINPUT > /dev/null
+./ergo <<EOINPUT > /dev/null
 molecule_inline
 H    0      0     0
 H    0    1.4     0
@@ -115,7 +112,7 @@ rm density.bin
 
 echo Testing H2 with bond distance 5 au, UHF/6-31G**
 rm -f ergoscf.out
-"$top_builddir"/source/ergo <<EOINPUT > /dev/null
+./ergo <<EOINPUT > /dev/null
 molecule_inline
 H   -2.5  0  0
 H    2.5  0  0
@@ -142,7 +139,7 @@ rm density.bin
 
 echo Testing Be[-] UHF STO-2G
 rm -f ergoscf.out
-"$top_builddir"/source/ergo <<EOINPUT > /dev/null
+./ergo <<EOINPUT > /dev/null
 molecule_inline
 Be   0    0   0
 EOF
@@ -171,7 +168,7 @@ rm density.bin
 
 echo Testing Be[-] UHF 6-31G*
 rm -f ergoscf.out
-"$top_builddir"/source/ergo <<EOINPUT > /dev/null
+./ergo <<EOINPUT > /dev/null
 molecule_inline
 Be   0    0   0
 EOF
@@ -210,6 +207,8 @@ rm density.bin
 
 
 
+cd ..
+rm -r $currtestdirname
 
 echo
 echo Unrestricted Hartree-Fock tests completed successfully!
